Package: PathoStat
Type: Package
Title: PathoStat Statistical Microbiome Analysis Package
Version: 1.38.0
Date: 2020-03-27
Author: Solaiappan Manimaran <manimaran_1975@hotmail.com>, Matthew
        Bendall <bendall@gwmail.gwu.edu>, Sandro Valenzuela Diaz
        <sandrolvalenzuelad@gmail.com>, Eduardo Castro
        <castronallar@gmail.com>, Tyler Faits <tfaits@gmail.com>, Yue
        Zhao <jasonzhao0307@gmail.com>, Anthony Nicholas Federico
        <anfed@bu.edu>, W. Evan Johnson <wej@bu.edu>
Maintainer: Solaiappan Manimaran <manimaran_1975@hotmail.com>, Yue Zhao
 <jasonzhao0307@gmail.com>
Description: The purpose of this package is to perform Statistical
        Microbiome Analysis on metagenomics results from sequencing
        data samples. In particular, it supports analyses on the
        PathoScope generated report files. PathoStat provides various
        functionalities including Relative Abundance charts, Diversity
        estimates and plots, tests of Differential Abundance, Time
        Series visualization, and Core OTU analysis.
URL: https://github.com/mani2012/PathoStat
BugReports: https://github.com/mani2012/PathoStat/issues
License: GPL (>= 2)
Depends: R (>= 3.5)
Imports: limma, corpcor,matrixStats, reshape2, scales, ggplot2,
        rentrez, DT, tidyr, plyr, dplyr, phyloseq, shiny, stats,
        methods, XML, graphics, utils, BiocStyle, edgeR, DESeq2,
        ComplexHeatmap, plotly, webshot, vegan, shinyjs, glmnet,
        gmodels, ROCR, RColorBrewer, knitr, devtools, ape
Collate: 'pathoStat.R' 'utils.R' 'taxonomy.R' 'biomarker.R'
        'allClasses.R' 'visualization.R' 'differentialAnalysis.R'
biocViews: Microbiome, Metagenomics, GraphAndNetwork, Microarray,
        PatternLogic, PrincipalComponent, Sequencing, Software,
        Visualization, RNASeq, ImmunoOncology
RoxygenNote: 6.1.1
Encoding: UTF-8
Suggests: rmarkdown, testthat
VignetteBuilder: knitr
Config/pak/sysreqs: cmake libfontconfig1-dev libfreetype6-dev
        libfribidi-dev git libglpk-dev make libharfbuzz-dev
        libmagick++-dev gsfonts libgit2-dev libicu-dev libjpeg-dev
        libpng-dev libtiff-dev libuv1-dev libwebp-dev libxml2-dev
        libssl-dev perl libx11-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:44:26 UTC
RemoteUrl: https://github.com/bioc/PathoStat
RemoteRef: RELEASE_3_23
RemoteSha: c77c976c2d70df3e8970018d26de61662d3c13a0
NeedsCompilation: no
Packaged: 2026-04-29 23:47:15 UTC; root
Built: R 4.6.0; ; 2026-04-30 01:19:57 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Bootstrap_LOOCV_LR_AUC)
export(Chisq_Test_Pam)
export(Fisher_Test_Pam)
export(GET_PAM)
export(LOOAUC_simple_multiple_noplot_one_df)
export(LOOAUC_simple_multiple_one_df)
export(TranslateIdToTaxLevel)
export(Wilcox_Test_df)
export(findRAfromCount)
export(findTaxonMat)
export(findTaxonomy)
export(findTaxonomy300)
export(getShinyInput)
export(getShinyInputCombat)
export(getShinyInputOrig)
export(getSignatureFromMultipleGlmnet)
export(grepTid)
export(loadPathoscopeReports)
export(loadPstat)
export(log2CPM)
export(pathostat1)
export(percent)
export(phyloseq_to_edgeR)
export(plotPCAPlotly)
export(plotPCoAPlotly)
export(readPathoscopeData)
export(runPathoStat)
export(savePstat)
export(setShinyInput)
export(setShinyInputCombat)
export(setShinyInputOrig)
export(summarizeTable)
exportClasses(PathoStat)
import(BiocStyle)
import(DT)
import(ape)
import(corpcor)
import(devtools)
import(edgeR)
import(ggplot2)
import(glmnet)
import(graphics)
import(knitr)
import(limma)
import(matrixStats)
import(methods)
import(phyloseq)
import(rentrez)
import(reshape2)
import(stats)
import(tidyr)
import(vegan)
import(webshot)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,estimateDispersions)
importFrom(DESeq2,nbinomWaldTest)
importFrom(DESeq2,results)
importFrom(DESeq2,sizeFactors)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(dplyr,add_rownames)
importFrom(dplyr,mutate_each)
importFrom(dplyr,select_)
importFrom(gmodels,ci)
importFrom(plotly,plot_ly)
importFrom(plyr,ddply)
importFrom(plyr,numcolwise)
importFrom(scales,percent_format)
importFrom(shiny,getShinyOption)
importFrom(shiny,runApp)
importFrom(shiny,shinyOptions)
importFrom(shinyjs,useShinyjs)
importFrom(utils,browseURL)
importFrom(utils,read.table)

PathoStat: Statistical Microbiome Analysis Toolkit
==================================================

PathoStat is a R shiny package, designed for performing Statistical Microbiome Analysis on 
metagenomics results from sequencing data samples. In particular, it supports 
analyses on the [PathoScope](https://github.com/PathoScope/PathoScope) generated report files. 

The package includes:

    1. Data Summary and Filtering 
    2. Relative Abundance plots (Stacked Bar Plot, Heatmap)
    3. Multiple species boxplot visualization
    4. Diversity analysis (Alpha and Beta diversity)
    5. Differential Expression (DEseq2, edgeR)
    6. Dimension Reduction (PCA, PCoA)
    7. Biomarker identification
    

`runPathoStat` is the pipeline function that generates the PathoStat report
and launches shiny app when in interactive mode. It combines all the functions 
into one step.

## Run Pathostat
To launch PathoStat in R, just enter the command:
```r
runPathoStat()
```

## Installation

To begin, install [Bioconductor](http://www.bioconductor.org/) and simply
run the following to automatically install PathoStat and all the dependencies 
as follows.

```r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("PathoStat")
```

If you want to install the latest development version of PathoStat from Github, 
use [devtools](https://github.com/hadley/devtools) to install it as follows:
```r
require(devtools)
install_github("compbiomed/PathoStat")
```

If all went well you should now be able to load PathoStat:
```r
require(PathoStat)
vignette('PathoStat-vignette', package='PathoStat')
runPathoStat()
```

# independent heatmap plotting function in the server using specific data from input
plotPCAPlotlyServer <- function(){
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat

  # Use shapes is optional
  if (input$select_pca_shape == "None") {
    condition.shape.vec <- NULL
  } else {
    condition.shape.vec <- physeq1@sam_data[[input$select_pca_shape]]
  }

  if (input$taxl.pca=="no rank")  {
    df.plot <- physeq1@otu_table@.Data
    if (input$select_pca_data_format == "log10 CPM"){
      df.plot <- getLogCPM(df.plot)
    } else if (input$select_pca_data_format == "RA"){
      df.plot <- getRelativeAbundance(df.plot)
    }
  # suppress warnings (otherwise, see Warning in RColorBrewer::brewer.pal(N, "Set2"))  
  storeWarn<- getOption("warn")
  options(warn = -1)     
    p <- suppressWarnings(plotPCAPlotly(df.input = df.plot,
                       condition.color.vec = physeq1@sam_data[[input$select_pca_color]],
                       condition.color.name = input$select_pca_color,
                       condition.shape.vec = condition.shape.vec,
                       condition.shape.name = input$select_pca_shape,
                       pc.a = paste("PC", input$xcol.new, sep = ""),
                       pc.b = paste("PC", input$ycol.new, sep = ""),
                       columnTitle = paste("PCA with colorbar representing", input$select_pca_color, sep = " ")))
        #restore warnings, delayed so plot is completed
    shinyjs::delay(expr =({ 
      options(warn = storeWarn) 
    }) ,ms = 10) 

   #p$condition.shape.vec = physeq1@sam_data[[input$select_pca_shape]]
   #p$condition.shape.name = input$select_pca_shape

  } else  {
    physeq2 <- tax_glom(physeq1, input$taxl.pca)
    df.plot <- physeq2@otu_table@.Data
    if (input$select_pca_data_format == "log10 CPM") {
      df.plot <- getLogCPM(df.plot)
    } else if (input$select_pca_data_format == "RA"){
      df.plot <- getRelativeAbundance(df.plot)
    }
  # suppress warnings (otherwise, see Warning in RColorBrewer::brewer.pal(N, "Set2"))  
  storeWarn<- getOption("warn")
  options(warn = -1) 
    p <- suppressWarnings(plotPCAPlotly(df.input = df.plot,
                       condition.color.vec = physeq2@sam_data[[input$select_pca_color]],
                       condition.color.name = input$select_pca_color,
                       condition.shape.vec = condition.shape.vec,
                       condition.shape.name = input$select_pca_shape,
                       pc.a = paste("PC", input$xcol.new, sep = ""),
                       pc.b = paste("PC", input$ycol.new, sep = ""),
                       columnTitle = paste("PCA with colorbar representing", input$select_pca_color, sep = " "))
    )
        #restore warnings, delayed so plot is completed
    shinyjs::delay(expr =({ 
      options(warn = storeWarn) 
    }) ,ms = 10) 
  }
  p$elementId <- NULL
  return(suppressWarnings(p))
}
# Show plot after hitting the plot button
plotPCAPlotlyServerButton <- eventReactive(input$DR_plot,{

  suppressWarnings(plotPCAPlotlyServer())
})
output$pca.plotly <- renderPlotly({
  suppressWarnings(plotPCAPlotlyServerButton())
})

# interactive PCA table
do_PCA_table <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$taxl.pca=="no rank")  {
    #test and fix the constant/zero row
    if (sum(rowSums(as.matrix(physeq1@otu_table@.Data)) == 0) > 0){
      physeq1@otu_table@.Data <- data.frame(physeq1@otu_table@.Data[-which
                                           (rowSums(as.matrix(physeq1@otu_table@.Data)) == 0),])
    }
    pca.tmp <- prcomp(t(physeq1@otu_table@.Data), scale = TRUE)
  } else  {
    physeq2 <- tax_glom(physeq1, input$taxl.pca)
    if (sum(rowSums(as.matrix(physeq2@otu_table@.Data)) == 0) > 0){
      physeq2@otu_table@.Data <- data.frame(physeq2@otu_table@.Data[-which
                                           (rowSums(as.matrix(physeq2@otu_table@.Data)) == 0),])
    }
    pca.tmp <- prcomp(t(physeq2@otu_table@.Data), scale = TRUE)
  }
  table.output.pca <- t(summary(pca.tmp)$importance)
  colnames(table.output.pca) = c("Standard deviation", "Variance Explained", "Cumulative Variance")
  table.output.pca[,2] <- scales::percent(as.numeric(table.output.pca[,2]))
  table.output.pca[,3] <- scales::percent(as.numeric(table.output.pca[,3]))
  #hide std
  DT::datatable(table.output.pca[,-1], options = list(sDom  = '<"top">t<"bottom">ip'))
}
# Show table after hitting the plot button
tablePCAServerButton <- eventReactive(input$DR_plot,{
  do_PCA_table()
})
output$PCAtable <- DT::renderDataTable({
  tablePCAServerButton()
})

# PCoA
plotPCoAPlotlyServer <- function(){
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  physeq1 <- phyloseq(otu_table(physeq1), phy_tree(physeq1),
                      tax_table(physeq1), sample_data(physeq1))

  # Use shapes is optional
  if (input$select_pca_shape == "None") {
    condition.shape.vec <- NULL
  } else {
    condition.shape.vec <- physeq1@sam_data[[input$select_pca_shape]]
  }
  # suppress warnings (otherwise, see Warning in RColorBrewer::brewer.pal(N, "Set2"))  
  storeWarn<- getOption("warn")
  options(warn = -1)  
  if (input$taxl.pca=="no rank")  {
    p <- plotPCoAPlotly(physeq.input = physeq1,
                        condition.color.vec = physeq1@sam_data[[input$select_pca_color]],
                        condition.color.name = input$select_pca_color,
                        condition.shape.vec = condition.shape.vec,
                        condition.shape.name = input$select_pca_shape,
                        method = input$pcoa.method,
                        pc.a = paste("Axis", input$xcol.new, sep = "."),
                        pc.b = paste("Axis", input$ycol.new, sep = "."),
                        columnTitle = paste("PCoA with colorbar representing", input$select_pca_color, sep = " ")
    )
  } else  {
    physeq2 <- tax_glom(physeq1, input$taxl.pca)
    p <- plotPCoAPlotly(physeq.input = physeq2,
                        condition.color.vec = physeq2@sam_data[[input$select_pca_color]],
                        condition.color.name = input$select_pca_color,
                        condition.shape.vec = condition.shape.vec,
                        condition.shape.name = input$select_pca_shape,
                        method = input$pcoa.method,
                        pc.a = paste("Axis", input$xcol.new, sep = "."),
                        pc.b = paste("Axis", input$ycol.new, sep = "."),
                        columnTitle = paste("PCoA with colorbar representing", input$select_pca_color, sep = " ")
    )
  }
  p$elementId <- NULL
          #restore warnings, delayed so plot is completed
    shinyjs::delay(expr =({ 
      options(warn = storeWarn) 
    }) ,ms = 10) 
  return(p)
}
# Show plot after hitting the plot button
plotPCoAPlotlyServerButton <- eventReactive(input$DR_plot,{
  plotPCoAPlotlyServer()
})
output$pcoa.plotly <- renderPlotly({
  plotPCoAPlotlyServerButton()
})

getOrdPCoA <- function(){
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$taxl.pca=="no rank")  {
    #test and fix the constant/zero row
    if (sum(rowSums(as.matrix(physeq1@otu_table@.Data)) == 0) > 0){
      physeq1@otu_table@.Data <- data.frame(physeq1@otu_table@.Data[-which
                                           (rowSums(as.matrix(physeq1@otu_table@.Data)) == 0),])
    }
    if (input$select_beta_div_method == "bray"){
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq1)))
      #Then use vegdist from vegan to generate a bray distance object:
      DistBC <- vegdist(dist.matrix, method = "bray")
      ord.tmp <- ordinate(physeq1, method = "PCoA", distance = DistBC)
    } else{
      Dist.tmp <- phyloseq::distance(physeq1, method = input$select_beta_div_method)
      ord.tmp <- ordinate(physeq1, method = "PCoA", distance = Dist.tmp)
    }
    #cat(dim(physeq1@otu_table))
    return(ord.tmp$values)
  } else  {
    physeq2 <- tax_glom(physeq1, input$taxl.pca)
    if (sum(rowSums(as.matrix(physeq2@otu_table@.Data)) == 0) > 0){
      physeq2@otu_table@.Data <- data.frame(physeq2@otu_table@.Data[-which
                                           (rowSums(as.matrix(physeq2@otu_table@.Data)) == 0),])
    }
    if (input$select_beta_div_method == "bray"){
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq2)))
      #Then use vegdist from vegan to generate a bray distance object:
      DistBC <- vegdist(dist.matrix, method = "bray")
      ord.tmp <- ordinate(physeq2, method = "PCoA", distance = DistBC)
    } else{
      Dist.tmp <- phyloseq::distance(physeq2, method = input$select_beta_div_method)
      ord.tmp <- ordinate(physeq2, method = "PCoA", distance = Dist.tmp)
    }
    return(ord.tmp$values)
  }
}

# Interactive PCA table
do_PCoA_table <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  ord <- getOrdPCoA()
  df.output <- ord[,c(1,3,5)]
  colnames(df.output) <- c("eigenvalue", "Variance Explained", "Cumulative Variance")
  rownames(df.output) <- paste("Axis", 1:nrow(df.output), sep = ".")
  df.output[,2] <- scales::percent(as.numeric(df.output[,2]))
  df.output[,3] <- scales::percent(as.numeric(df.output[,3]))
  # hide eigenvalue
  DT::datatable(df.output[,-1], options = list(sDom  = '<"top">t<"bottom">ip'))
}
# Show table after hitting the plot button
tablePCoAServerButton <- eventReactive(input$DR_plot,{
  do_PCoA_table()
})
output$PCoAtable <- DT::renderDataTable({
  tablePCoAServerButton()
})

library(shiny)
library(ROCR)
library(reshape2)
library(DESeq2)
library(edgeR)
library(phyloseq)
library(stats)
library(PathoStat)
library(plotly)
library(webshot)
library(vegan)
library(dplyr)
library(ape)

# check if integer(0)
is.integer0 <- function(x)
{
  is.integer(x) && length(x) == 0L
}

# Converts decimal percentage to string with specified digits
pct2str <- function(v, digits=2) {sprintf(paste0('%.',digits,'f'), v*100)}

# get RA from counts
getRelativeAbundance <- function(df){
  ra.out <- apply(df, 2, function(x) round(x/sum(x), digits = 4))
  return(ra.out)
}

getLogCPM <- function(df){
  logCPM.out <- apply(df, 2, function(y) log10(y*1e6/sum(y) + 1))
  return(logCPM.out)
}

vals <- reactiveValues(
    shiny.input = getShinyOption("pathostat.shinyInput"),
    shiny.input.backup = getShinyOption("pathostat.shinyInput"),
    taxdata = NULL,
    taxcountdata = NULL
)

updateTaxLevel <- function(){
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat

  updateSelectInput(session, "taxl",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.alpha",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.beta",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.pca",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.da",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.edger",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl.pa",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "sra_taxlev",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "hmra_taxlev",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl_single_species",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxlTable",
                    choices = colnames(pstat@tax_table@.Data))
  updateSelectInput(session, "taxl_biomarker",
                    choices = colnames(pstat@tax_table@.Data))  
}

# update samples
updateSample <- function(){
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    updateSelectInput(session, "filterSample",
                      choices = colnames(pstat@otu_table@.Data))
    updateSelectInput(session, "hmra_isolate_samples",
                      choices = colnames(pstat@otu_table@.Data))
    updateSelectInput(session, "sra_isolate_samples",
                      choices = colnames(pstat@otu_table@.Data))
}



#Update covariate names
updateCovariate <- function(){
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    covariates <- colnames(sample_data(pstat))
    # choose the covariates that has less than 8 levels
    covariates.colorbar <- c()
    for (i in 1:length(covariates)){
        num.levels <- length(unique(sample_data(pstat)[[covariates[i]]]))
        if (num.levels < 8){
            covariates.colorbar <- c(covariates.colorbar, covariates[i])
        }
    }
    # choose the covariates that has 2 levels
    covariates.two.levels <- c()
    for (i in 1:length(covariates)){
        num.levels <- length(unique(sample_data(pstat)[[covariates[i]]]))
        if (num.levels == 2){
            covariates.two.levels <- c(covariates.two.levels, covariates[i])
        }
    }
    
    ## numeric cov
    sam_temp <- as.data.frame(pstat@sam_data)
    num_select <- lapply(covariates, function(x) is.categorical(unlist(sam_temp[,x])))
    num_covariates <- covariates[!unlist(num_select)]

    updateSelectInput(session, "select_covariate_condition_biomarker",
                      choices = covariates)
    updateSelectInput(session, "select_single_species_condition",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_target_condition_biomarker",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_condition_sample_filter",
                      choices = c("Reads", covariates))
    updateSelectInput(session, "select_condition_sample_filter_micro",
                      choices = c("Taxon elements number", covariates))
    updateSelectInput(session, "select_condition_sample_filter_sidebar",
                      choices = c("Reads", covariates))
    updateSelectInput(session, "select_condition_sample_distribution",
                      choices = covariates)
    updateSelectInput(session, "select_condition",
                      choices = covariates)
    updateSelectInput(session, "select_heatmap_condition_1",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_heatmap_condition_2",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_alpha_div_condition",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_beta_condition",
                      choices = covariates.two.levels)
    updateSelectInput(session, "select_beta_heatmap_condition_1",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_beta_heatmap_condition_2",
                      choices = covariates.colorbar)
    updateSelectInput(session, "select_pca_color",
                      choices = covariates)
    updateSelectInput(session, "select_pca_shape",
                      choices = c("None", covariates.colorbar))
    updateSelectInput(session, "da.condition",
                      choices = covariates)
    updateSelectInput(session, "edger.condition",
                      choices = covariates.colorbar)
    updateSelectInput(session, "da.condition.covariate",
                      choices = covariates)
    updateSelectInput(session, "pa.condition",
                      choices = covariates.colorbar)
    updateSelectInput(session, "sra_select_conditions",
                      choices = covariates)
    updateSelectInput(session, "gra_select_conditions",
                      choices = c("All", covariates))
    updateSelectInput(session, "hmra_select_conditions",
                      choices = covariates)
    updateSelectInput(session, "bin_cov",
                      choices = num_covariates)   
    updateSelectInput(session, "bdhm_select_conditions",
                      choices = covariates.colorbar)       
    
}

tabPanel("Diversity",
  tabsetPanel(
    tabPanel("Alpha Diversity",
      br(),
      sidebarLayout(
        sidebarPanel(
          br(),
          selectizeInput('taxl.alpha', 'Taxonomy Level', choices = tax.name, selected='no rank'),
          selectInput("select_alpha_div_condition", "Compare between:", covariates.colorbar),
          selectInput("select_alpha_div_method", "Choose method:", alpha.methods),
          selectInput("select_alpha_stat_method","Non-parametric Test", c("Mann-Whitney","T-test", "Kruskal-Wallis")),
          actionButton("alpha_boxplot", "Run")
        ),
        mainPanel(
          tabsetPanel(
            tabPanel("Boxplot",
              plotlyOutput("AlphaDiversity"),
              br(),
              DT::dataTableOutput("alpha.stat.test")
            ),
            tabPanel("Alpha Diversity Table",
              br(),
              DT::dataTableOutput("table.alpha"),
              downloadButton('download_table_alpha', 'Download')
            )
          )
        )
      )
    ),
    tabPanel("Beta Diversity",
      br(),
      sidebarLayout(
        sidebarPanel(
          selectizeInput('taxl.beta', 'Taxonomy Level', choices = tax.name, selected='no rank'),
          selectInput("select_beta_div_method", "Choose method:", beta.methods),
          selectizeInput('bdhm_select_conditions', 'Color Samples by Condition', choices=covariates.colorbar, multiple=TRUE),
          radioButtons("bdhm_sort_by", "Sort By", c("No Sorting" = "nosort", "Conditions" = "conditions"), selected="nosort"),
          actionButton("beta_heatmap", "Plot Heatmap")
        ),
        mainPanel(
          tabsetPanel(
            tabPanel("Heatmap",
              br(),
              plotlyOutput("BetaDiversityHeatmap", width="800px", height="600px")
            ),
            tabPanel("Boxplot",
              fluidRow(
                column(4,
                  selectInput("select_beta_stat_method","Select Test", c("PERMANOVA", "Kruskal-Wallis", "Mann-Whitney")),
                  helpText("Only variables with 2 levels are supported" ),
                  selectInput("select_beta_condition", "Select condition", covariates.two.levels),
                  numericInput("num.permutation.permanova", "Number of permutations", value = 999, max = 2000)
                ),
                column(8,
                  br(),
                  DT::dataTableOutput("beta.stat.test"),
                  actionButton("beta_boxplot", "Run")
                )
              ),
              plotlyOutput("BetaDiversityBoxplot")
            ),
            tabPanel("Beta Diversity Table",
              br(),
              DT::dataTableOutput("table.beta"),
              downloadButton('download_table_beta', 'Download')
            )
          )
        )
      )
    )
  )
)

library(PathoStat)

### Example 1: Run PathoStat
runPathoStat()

### Example 2: L1 set
#example2_data_dir <- system.file("example/data2/L1", package = "PathoStat")
#pstat <- createPathoStat(input_dir=example2_data_dir, 
#    sample_data_file="sample_data.tsv")
# Save for future use
#savePstat(pstat, outdir=".", outfileName="pstat_data_2_L1.rda")
data_dir <- system.file("data", package = "PathoStat")
# Load and run PathoStat
pstat <- loadPstat(indir=data_dir, infileName="pstat_data_2_L1.rda")
runPathoStat(pstat)




### Example 2: L2 set
#example2_data_dir <- system.file("example/data2/L2", package = "PathoStat")
#pstat <- createPathoStat(input_dir=example2_data_dir, 
#    sample_data_file="sample_data.tsv")
# Save for future use
#savePstat(pstat, outdir=".", outfileName="pstat_data_2_L2.rda")
data_dir <- system.file("data", package = "PathoStat")
# Load and run PathoStat
pstat <- loadPstat(indir=data_dir, infileName="pstat_data_2_L2.rda")
runPathoStat(pstat)

### Example 3: Asthma dataset
data_dir <- system.file("data", package = "PathoStat")
pstat <- loadPstat(indir=data_dir, infileName="asthma_pstat_data.rda")
runPathoStat(pstat)

library(shiny)
library(shinyjs)
library(phyloseq)
library(plotly)
source(file.path("utils", "helpers.R"))



alpha.methods <- c("Shannon", "Simpson", "InvSimpson")
# Weigthed Unifrac, Bray-Curtis
beta.methods <- c("wUniFrac", "bray")

tax.name <- c('superkingdom', 'kingdom', 'phylum', 'class', 'order', 'family',
    'genus', 'species', 'no rank')
norm.methods <- c('EBayes coreOTU Normalization',
    'Quantile coreOTU Normalization', 'Library Size Scaling')
measure.type <- c('Final Guess', 'Final Best Hit', 'Final High Confidence Hit')
minbatch <- function(batch1){
    batch2 <- as.factor(batch1)
    batch3 <- split(batch1,batch2)
    return(min(unlist(lapply(1:length(batch3),
        function(x) length(batch3[[x]])))))
}

shinyInput <- getShinyInput()

pstat <- shinyInput$pstat
covariates <- colnames(sample_data(pstat))

# choose the covariates that has less than 8 levels
covariates.colorbar <- c()
for (i in 1:length(covariates)){
  num.levels <- length(unique(sample_data(pstat)[[covariates[i]]]))
  if (num.levels < 8){
    covariates.colorbar <- c(covariates.colorbar, covariates[i])
  }
}

# choose the covariates that has 2 levels
covariates.two.levels <- c()
for (i in 1:length(covariates)){
  num.levels <- length(unique(sample_data(pstat)[[covariates[i]]]))
  if (num.levels == 2){
    covariates.two.levels <- c(covariates.two.levels, covariates[i])
  }
}

is.categorical <- function(v) {
  if (class(v) == "integer" || class(v) == "numeric") {
    return(F)
  } else {
    return(T)
  }
}

# numeric cov
    sam_temp <- as.data.frame(pstat@sam_data)
    num_select <- lapply(covariates, function(x) is.categorical(unlist(sam_temp[,x])))
    num_covariates <- covariates[!unlist(num_select)]



maxbatchElems <- minbatch(c(pstat@sam_data[,1])[[1]])
maxcondElems <- minbatch(c(pstat@sam_data[,2])[[1]])
defaultDisp <- 30
defaultGenesDisp <- 10
maxGenes <- dim(pstat@otu_table)[1]




#sample name
sample.names.all <- colnames(pstat@otu_table@.Data)

# Alpha diversity boxplots
plotAlphaServer <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$taxl.alpha != "no rank") {
    physeq1 <- tax_glom(physeq1, input$taxl.alpha)
  }
  meta.data <- physeq1@sam_data
  meta.data$sample.name <- rownames(meta.data)
  meta.data$richness <- suppressWarnings(estimate_richness(physeq = physeq1, split = T, measures = input$select_alpha_div_method)[,1])
  colnames(meta.data)[which(colnames(meta.data) == input$select_alpha_div_condition)] <- "condition"
  g <- ggplot(meta.data, aes(condition, richness, text=sample.name, color = condition)) +
    geom_point() + geom_boxplot() +
    labs(title = paste("Alpha diversity between ",
                       input$select_alpha_div_condition,
                       " (", input$select_alpha_div_method, ")", sep = ""))
  g <- ggplotly(g, tooltip="text")
  g$elementId <- NULL # To suppress a shiny warning
  return(g)
}

plotAlphaBoxplotButton <- eventReactive(input$alpha_boxplot,{
  plotAlphaServer()
})
output$AlphaDiversity <- renderPlotly({
  plotAlphaBoxplotButton()
})

# Alpha diversity statistics
do_alpha_stats <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$taxl.alpha !="no rank")  {
    physeq1 <- tax_glom(physeq1, input$taxl.alpha)
  }
  meta.data <- physeq1@sam_data
  meta.data$sample.name <- rownames(meta.data)
  meta.data$richness <- suppressWarnings(estimate_richness(physeq = physeq1, split = T, measures = input$select_alpha_div_method)[,1])
  colnames(meta.data)[which(colnames(meta.data) == input$select_alpha_div_condition)] <- "condition"
  meta.data <- data.frame(meta.data)
  meta.data$condition <- as.factor(meta.data$condition)

  if (length(unique(meta.data$condition)) == 2){
    if (input$select_alpha_stat_method == "Mann-Whitney"){
      tmp <- wilcox.test(richness ~ condition, data = meta.data)
      output <- c(tmp$method, tmp$p.value)
      output.table <- data.frame(output)
      rownames(output.table) <- c("Method", "P-value")
      output.table
    } else if (input$select_alpha_stat_method == "T-test"){
      tmp <- t.test(richness ~ condition, data = meta.data)
      output <- c(tmp$method, tmp$p.value)
      output.table <- data.frame(output)
      rownames(output.table) <- c("Method", "P-value")
      output.table
    } else{
      print("Condition level number is 2, please use Mann-Whitney test.")
    }

  } else if (length(unique(meta.data$condition)) > 2){
    if (input$select_alpha_stat_method == "Mann-Whitney"){
      result.list <- list()
      meta.data.list <- list()
      for (i in 1:length(unique(meta.data$condition))){
        meta.data.list[[i]] <- meta.data[which(meta.data$condition != unique(meta.data$condition)[i]),]
        result.list[[i]] <- wilcox.test(richness ~ condition, data = meta.data.list[[i]])
      }
      output.table <- NULL
      group.name <- c()
      for (i in 1:length(result.list)){
        output.tmp <- c(result.list[[i]]$method, result.list[[i]]$p.value)
        output.table <- cbind(output.table, output.tmp)
        group.name[i] <- paste(unique(meta.data.list[[i]]$condition), collapse = " and ")
      }
      rownames(output.table) <- c("Method", "P-value")
      colnames(output.table) <- group.name
      output.table

    } else if (input$select_alpha_stat_method == "T-test"){
              result.list <- list()
      meta.data.list <- list()
      for (i in 1:length(unique(meta.data$condition))){
        meta.data.list[[i]] <- meta.data[which(meta.data$condition != unique(meta.data$condition)[i]),]
        result.list[[i]] <- t.test(richness ~ condition, data = meta.data.list[[i]])
      }
      output.table <- NULL
      group.name <- c()
      for (i in 1:length(result.list)){
        output.tmp <- c(result.list[[i]]$method, result.list[[i]]$p.value)
        output.table <- cbind(output.table, output.tmp)
        group.name[i] <- paste(unique(meta.data.list[[i]]$condition), collapse = " and ")
      }
      rownames(output.table) <- c("Method", "P-value")
      colnames(output.table) <- group.name
      output.table

    } else{
      tmp <- kruskal.test(richness ~ condition, data = meta.data)
      output <- c(tmp$method, tmp$p.value)
      output.table <- data.frame(output)
      rownames(output.table) <- c("Method", "P-value")
      output.table
    }

  } else{
    "Condition level must be at least 2."
  }
}
plotAlphaBoxplotButton3 <- eventReactive(input$alpha_boxplot,{
  do_alpha_stats()
})
output$alpha.stat.test <- DT::renderDataTable({
  plotAlphaBoxplotButton3()
}, options = list(sDom  = '<"top">t<"bottom">ip'))

# Alpha diversity table
do_alpha_table <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$taxl.alpha !="no rank")  {
    physeq1 <- tax_glom(physeq1, input$taxl.alpha)
  }
  meta.data <- physeq1@sam_data
  meta.data$sample.name <- rownames(meta.data)
  meta.data$richness <- suppressWarnings(estimate_richness(physeq = physeq1, split = T, measures = input$select_alpha_div_method)[,1])
  colnames(meta.data)[which(colnames(meta.data) == input$select_alpha_div_condition)] <- "condition"
  rownames(meta.data) <- 1:nrow(meta.data)
  DT::datatable(meta.data %>% dplyr::select(sample.name, condition, richness))
}
plotAlphaBoxplotButton2 <- eventReactive(input$alpha_boxplot,{
  do_alpha_table()
})
output$table.alpha <- DT::renderDataTable({
  plotAlphaBoxplotButton2()
})

# Download alpha diversity table
output$download_table_alpha <- downloadHandler(
  filename = function() { paste('Alpha_diversity_table', '.csv', sep='') },
  content = function(file) {
    shinyInput <- vals$shiny.input
    physeq1 <- shinyInput$pstat
    if (input$taxl.alpha !="no rank")  {
      physeq1 <- tax_glom(physeq1, input$taxl.alpha)
    }
    meta.data <- physeq1@sam_data
    meta.data$sample.name <- rownames(meta.data)
    meta.data$richness <- suppressWarnings(estimate_richness(physeq = physeq1, split = T, measures = input$select_alpha_div_method)[,1])
    colnames(meta.data)[which(colnames(meta.data) == input$select_alpha_div_condition)] <- "condition"
    rownames(meta.data) <- 1:nrow(meta.data)
    meta.data <- as_tibble(meta.data)
    meta.data <- meta.data %>% select(sample.name, condition, richness)
    write.csv(data.frame(meta.data), file)
  }
)

do_beta_heatmap <- function(){
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  tables <- pstat.extraction(shinyInput$pstat)
  SAM_DATA <- tables$SAM

  if (input$taxl.beta != "no rank") {
    physeq2 <- tax_glom(physeq1, input$taxl.beta)
  } else {
    physeq2 <- physeq1
  }
  if (input$select_beta_div_method == "bray"){
    #First get otu_table and transpose it:
    dist.matrix <- t(data.frame(otu_table(physeq2)))
    #Then use vegdist from vegan to generate a bray distance object:
    dist.mat <- vegdist(dist.matrix, method = "bray")
  }else{
    dist.mat = phyloseq::distance(physeq2, method = input$select_beta_div_method)
  }
  dist.mat <- as.matrix(dist.mat)
  dist.mat <- dist.mat[order(match(rownames(dist.mat), rev(rownames(dist.mat)))),,drop=FALSE]

  if (!is.null(input$bdhm_select_conditions)) {
    df.sam <- SAM_DATA[,input$bdhm_select_conditions,drop=FALSE]
    if (input$bdhm_sort_by == "conditions") {
      for (i in ncol(df.sam):1) {
        df.sam <- df.sam[order(df.sam[[i]]),,drop=FALSE]
      }
      dist.mat <- dist.mat[order(match(rownames(dist.mat), rownames(df.sam))),,drop=FALSE]
      dist.mat <- dist.mat[,order(match(colnames(dist.mat), rownames(df.sam))),drop=FALSE]
    } else {
      df.sam <- df.sam[order(match(rownames(df.sam), rownames(dist.mat))),,drop=FALSE]
    }
  }

  m <- data.matrix(dist.mat)
  hover.txt <- c()
  for (i in 1:ncol(dist.mat)) {
    hover.txt <- cbind(hover.txt, dist.mat[[i]])
  }
  hm.beta <- plot_ly(x = colnames(m), y = rownames(m), z = m,
                     type = "heatmap",
                     colors= "RdPu",
                     hoverinfo = "x+y+z") %>%
    layout(xaxis = list(showticklabels = FALSE, title = "", ticks = "", tickangle = -45),
           yaxis = list(showticklabels = FALSE, type = 'category', ticks = ""))

  if (!is.null(input$bdhm_select_conditions)) {
    hover.txt <- c()
    for (i in 1:ncol(df.sam)) {
      hover.txt <- cbind(hover.txt, df.sam[[i]])
    }
    df.sam[] <- lapply(df.sam, factor)
    
    # Y-axis of subplot
    m <- data.matrix(df.sam)
    m.row.normalized <- apply(m, 2, function(x)(x-min(x))/(max(x)-min(x)))
    hm.sam.y <- plot_ly(x = colnames(m.row.normalized), 
                        y = rownames(m.row.normalized), 
                        z = m.row.normalized, 
                        type = "heatmap",
                        showscale=FALSE,
                        hoverinfo = "x+y+text",
                        transpose=FALSE,
                        text=hover.txt) %>%
      layout(xaxis = list(title = "", tickangle = -45),
             yaxis = list(showticklabels = FALSE, type = 'category', ticks = ""),
             orientation=TRUE)
    
    # X-axis of subplot
    m <- data.matrix(df.sam)
    m.row.normalized <- apply(m, 2, function(x)(x-min(x))/(max(x)-min(x)))
    m.row.normalized = t(m.row.normalized)
    m.row.normalized = m.row.normalized[order(match(rownames(m.row.normalized), rev(rownames(m.row.normalized)))),,drop=FALSE]
    hm.sam.x <- plot_ly(x = colnames(m.row.normalized), 
                        y = rownames(m.row.normalized), 
                        z = m.row.normalized, 
                        type = "heatmap",
                        showscale=FALSE,
                        hoverinfo = "x+y+text",
                        transpose=FALSE,
                        text=t(hover.txt)) %>%
      layout(xaxis = list(showticklabels = FALSE, type = 'category', ticks = ""),
             yaxis = list(title = "", tickangle = -45),
             orientation=TRUE)
  }

  empty <- plotly_empty(type = "scatter")
  
  if (!is.null(input$bdhm_select_conditions)) {
    hm.sam.beta.top <- subplot(empty, hm.sam.x, widths=c(0.1,  0.9))
    hm.sam.beta.bot <- subplot(hm.sam.y, hm.beta, widths=c(0.1,  0.9))
    hm.sam.beta <- subplot(hm.sam.beta.top, hm.sam.beta.bot, nrows=2, heights=c(0.1,  0.9))
    hm.sam.beta$elementId <- NULL # To suppress a shiny warning
    return(hm.sam.beta)
  } else {
    hm.beta$elementId <- NULL # To suppress a shiny warning
    return(hm.beta)
  }
}
plotBetaHeatmapServerButton <- eventReactive(input$beta_heatmap,{
  do_beta_heatmap()
})
output$BetaDiversityHeatmap <- renderPlotly({
  plotBetaHeatmapServerButton()
})

# Beta diversity boxplots
plotBetaBoxplotServer <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat

  if (input$taxl.beta !="no rank")  {
    physeq1 <- tax_glom(physeq1, input$taxl.beta)
  }

  meta.data <- physeq1@sam_data
  meta.data$sample.name <- rownames(meta.data)
  colnames(meta.data)[which(colnames(meta.data) == input$select_beta_condition)] <- "condition"

  if (input$select_beta_div_method == "bray") {
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq1)))
      #Then use vegdist from vegan to generate a bray distance object:
      dist.tmp <- vegdist(dist.matrix, method = "bray")
  } else {
      dist.tmp = phyloseq::distance(physeq1, method = input$select_beta_div_method)
  }
  dist.mat <- as.matrix(dist.tmp)
  dist.within.a <- c()
  dist.within.b <- c()
  dist.between <- c()
  for (i in 1:nrow(dist.mat)){
    for (j in 1:nrow(dist.mat)) {
      if (meta.data$condition[i] == unique(meta.data$condition)[1] &
          meta.data$condition[j] == unique(meta.data$condition)[1]){
        dist.within.a <- c(dist.within.a, dist.mat[i,j])
      } else if (meta.data$condition[i] == unique(meta.data$condition)[2] &
                 meta.data$condition[j] == unique(meta.data$condition)[2]){
        dist.within.b <- c(dist.within.b, dist.mat[i,j])
      } else{
        dist.between <- c(dist.between, dist.mat[i,j])
      }
    }
  }
  y.axis <- list(
    title = paste(input$select_beta_div_method, "Distance", sep = " ")
  )
  p <- plot_ly(y = ~dist.within.a, type = "box", name = paste("Within", unique(meta.data$condition)[1])) %>%
    add_trace(y = ~dist.within.b, name = paste("Within", unique(meta.data$condition)[2])) %>%
    add_trace(y = ~dist.between, name = "Between 2 conditions") %>%
    layout(yaxis = y.axis)
  p$elementId <- NULL # To suppress a shiny warning
  p
}
plotBetaBoxplotServerButton <- eventReactive(input$beta_boxplot,{
  plotBetaBoxplotServer()
})
output$BetaDiversityBoxplot <- renderPlotly({
  plotBetaBoxplotServerButton()
})

# Beta diversity table
do_beta_table <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat

  if (input$taxl.beta=="no rank")  {
      if (input$select_beta_div_method == "bray"){
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq1)))
      #Then use vegdist from vegan to generate a bray distance object:
      dist.mat <- vegdist(dist.matrix, method = "bray")
  }else{
      dist.mat = phyloseq::distance(physeq1, method = input$select_beta_div_method)
  }

  } else{
    physeq2 <- tax_glom(physeq1, input$taxl.beta)
      if (input$select_beta_div_method == "bray"){
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq2)))
      #Then use vegdist from vegan to generate a bray distance object:
      dist.mat <- vegdist(dist.matrix, method = "bray")
      }else{
      dist.mat = phyloseq::distance(physeq2, method = input$select_beta_div_method)
      }
  }
  dist.mat <- as.matrix(dist.mat)
  return(dist.mat)
}
plotBetaBoxplotServerButton2 <- eventReactive(input$beta_heatmap,{
  do_beta_table()
})
output$table.beta <- DT::renderDataTable({
  plotBetaBoxplotServerButton2()
}, options=list(paging = TRUE, scrollX = TRUE))

# Download beta diversity table
output$download_table_beta <- downloadHandler(
  filename = function() { paste('Beta_diversity_table', '.csv', sep='') },
  content = function(file) {
      shinyInput <- vals$shiny.input
    physeq1 <- shinyInput$pstat

    if (input$taxl.beta=="no rank")  {
      dist.mat = phyloseq::distance(physeq1, method = input$select_beta_div_method)
    } else{
      physeq2 <- tax_glom(physeq1, input$taxl.beta)
      dist.mat = phyloseq::distance(physeq2, method = input$select_beta_div_method)
    }
    dist.mat <- as.matrix(dist.mat)
    write.csv(data.frame(dist.mat), file)
  }
)

do_beta_stats <- function() {
  shinyInput <- vals$shiny.input
  physeq1 <- shinyInput$pstat
  if (input$select_beta_stat_method == "PERMANOVA"){
    if (input$taxl.beta !="no rank")  {
      physeq1 <- tax_glom(physeq1, input$taxl.beta)
    }
    meta.data <- physeq1@sam_data
    meta.data$sample.name <- rownames(meta.data)
    colnames(meta.data)[which(colnames(meta.data) == input$select_beta_condition)] <- "condition"
    meta.data <- data.frame(meta.data)
    meta.data$condition <- as.factor(meta.data$condition)

    set.seed(99)

    if (input$select_beta_div_method == "bray"){
      #First get otu_table and transpose it:
      dist.matrix <- t(data.frame(otu_table(physeq1)))
      #Then use vegdist from vegan to generate a bray distance object:
      dist.tmp <- vegdist(dist.matrix, method = "bray")
    }else{
      dist.tmp = phyloseq::distance(physeq1, method = input$select_beta_div_method)
    }

    beta.div <- adonis2(dist.tmp~condition, data=meta.data,
                        permutations = input$num.permutation.permanova, strata="PLOT")
    beta.div

  } else {

    if (input$taxl.beta !="no rank")  {
      physeq1 <- tax_glom(physeq1, input$taxl.beta)
    }

    meta.data <- physeq1@sam_data
    meta.data$sample.name <- rownames(meta.data)
    colnames(meta.data)[which(colnames(meta.data) == input$select_beta_condition)] <- "condition"

    dist.tmp = phyloseq::distance(physeq1, method = input$select_beta_div_method)
    dist.mat <- as.matrix(dist.tmp)
    dist.within.a <- c()
    dist.within.b <- c()
    dist.between <- c()
    for (i in 1:nrow(dist.mat)){
      for (j in 1:nrow(dist.mat)) {
        if (meta.data$condition[i] == unique(meta.data$condition)[1] &
            meta.data$condition[j] == unique(meta.data$condition)[1]){
          dist.within.a <- c(dist.within.a, dist.mat[i,j])
        } else if (meta.data$condition[i] == unique(meta.data$condition)[2] &
                   meta.data$condition[j] == unique(meta.data$condition)[2]){
          dist.within.b <- c(dist.within.b, dist.mat[i,j])
        } else{
          dist.between <- c(dist.between, dist.mat[i,j])
        }

      }
    }
    dist.list <- list(dist.within.a, dist.within.b, dist.between)
    names(dist.list) <- c(unique(meta.data$condition)[1], unique(meta.data$condition)[2], "between")

    if (input$select_beta_stat_method == "Mann-Whitney"){
      result.list <- list()
      group.name <- c()
      for (i in 1:length(dist.list)){
        dist.list.tmp <- dist.list[which(names(dist.list) != names(dist.list)[i])]

        group.name[i] <- paste(names(dist.list.tmp), collapse = " and ")
        result.list[[i]] <- wilcox.test(dist.list.tmp[[1]], dist.list.tmp[[2]])
      }
      output.table <- NULL
      for (i in 1:length(result.list)){
        output.tmp <- c(result.list[[i]]$method, result.list[[i]]$p.value)
        output.table <- cbind(output.table, output.tmp)
      }
      rownames(output.table) <- c("Method", "P-value")
      colnames(output.table) <- group.name
      output.table
    } else {
      tmp <- kruskal.test(list(dist.within.a, dist.within.b, dist.between))
      output <- c(tmp$method, tmp$p.value)
      output.table <- data.frame(output)
      rownames(output.table) <- c("Method", "P-value")
      output.table
    }
  }
}
plotBetaBoxplotServerButton3 <- eventReactive(input$beta_boxplot, {
  do_beta_stats()
})
output$beta.stat.test <- DT::renderDataTable({
  plotBetaBoxplotServerButton3()
}, options = list(sDom  = '<"top">t<"bottom">ip'))

#' Compute log2(counts per mil reads) and library size for each sample
#'
#' @param qcounts quantile normalized counts
#' @param lib.size default is colsums(qcounts)
#' @return list containing log2(quantile counts per mil reads) and library sizes
#' @export
#' @examples
#' log2CPM(matrix(1:12, nrow = 3))

log2CPM <- function(qcounts, lib.size = NULL) {
    if (is.null(lib.size))
        lib.size <- colSums(qcounts)
    qcounts <- apply(qcounts, seq_len(2), FUN = function(x) {
        ifelse(is.null(x) || is.na(x) || is.nan(x), 0, x)
    })
    minimum <- min(qcounts)
    if (minimum < 0) {
        qcounts <- qcounts - minimum
    }
    avg <- mean(colMeans(qcounts))
    qcounts <- apply(qcounts, seq_len(2), FUN = function(x) {
        ifelse(x < 0, avg, x)
    })
    y <- t(log2(t(qcounts + 0.5)/(lib.size + 1) * 1e+06))
    return(list(y = y, lib.size = lib.size))
}

#' Reads the data from PathoScope reports and returns a list of
#' final guess relative abundance and count data
#'
#' @param input_dir Directory where the tsv files from PathoScope are located
#' @param pathoreport_file_suffix PathoScope report files suffix
#' @param use.input.files whether input dir to pathoscope files
#' or directly pathoscope files
#' @param input.files.path.vec vector of pathoscope file paths
#' @param input.files.name.vec vector of pathoscope file names
#' @return List of final guess relative abundance and count data
#' @importFrom utils read.table
#' @export
#' @examples
#' example_data_dir <- system.file("example/data", package = "PathoStat")
#' pathoreport_file_suffix <- "-sam-report.tsv"
#' datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
#' input.files.name.vec = as.character(1:6))

readPathoscopeData <-
    function(input_dir=".", pathoreport_file_suffix="-sam-report.tsv",
    use.input.files = FALSE, input.files.path.vec = NULL,
    input.files.name.vec = NULL) {
    if (use.input.files == FALSE){
        if (input_dir == ".") {
            input_dir <- getwd()
        }
        pattern <- paste("*", pathoreport_file_suffix, sep="")
        filenames <- list.files(input_dir, pattern = pattern, full.names = TRUE)


    } else{
        filenames <- input.files.path.vec
    }
    ltbl <- lapply(filenames, read.table, skip=1, header=TRUE, sep="\t", stringsAsFactors=TRUE)

    lgenomes <- lapply(ltbl, function(tbl) {return(levels(tbl[,1]))})
    genomes <- unique(unlist(lgenomes))
    genomes <- c(genomes, "others")
    lfl <- lapply(filenames, readLines, n = 1)
    lnumReads <- unlist(lapply(lfl, function(fl)
    {return(as.numeric(strsplit(fl, "\t")[[1]][2]))}))
    samplenames <- unlist(lapply(input.files.name.vec, function(x) {
        return(strsplit(x, pathoreport_file_suffix)[[1]])
    }))
    
    dat <- matrix(0L, nrow = length(genomes), ncol = length(samplenames))
    countdat <- matrix(0L, nrow = length(genomes), ncol = length(samplenames))    
    
    for (i in seq(length(samplenames))){
      index.tmp <- match(ltbl[[i]][,1], genomes)
      dat[index.tmp,i] <- ltbl[[i]][,3]
      countdat[index.tmp,i] <- ltbl[[i]][,4]
      read.sum <- sum(ltbl[[i]][,4])
      read.num.other <- lnumReads[i] - read.sum
      dat[nrow(dat),i] <- 1 - sum(ltbl[[i]][,3])
      countdat[nrow(dat),i] <- read.num.other
    }
    
    dat <- data.frame(dat)
    rownames(dat) <- genomes
    colnames(dat) <- samplenames
    countdat <- data.frame(countdat)
    rownames(countdat) <- genomes
    colnames(countdat) <- samplenames
    return(list(data = dat, countdata = countdat))
}

#' Loads all data from a set of PathoID reports.
#' For each column in the PathoID report, construct a
#' matrix where the rows are genomes and the columns are samples.
#' Returns a list where each element is named according to
#' the PathoID column. For example, ret[["Final.Best.Hit.Read.Numbers"]]
#' on the result of this function will get you the final count matrix.
#' Also includes elements "total_reads" and "total_genomes" from the
#' first line of the PathoID report.
#'
#' @param reportfiles Paths to report files
#' @param nrows Option to read first N rows of PathoScope reports
#' @return Returns a list where each element is named according to
#' the PathoID column. For example, ret[["Final.Best.Hit.Read.Numbers"]]
#' on the result of this function will get you the final count matrix.
#' Also includes elements "total_reads" and "total_genomes" from the
#' first line of the PathoID report.
#' @export
#' @examples
#' input_dir <- system.file("example/data", package = "PathoStat")
#' reportfiles <- list.files(input_dir, pattern = "*-sam-report.tsv",
#' full.names = TRUE)

loadPathoscopeReports <- function(reportfiles, nrows=NULL) {
    # Report basenames
    report_base <- gsub('-sam-report', '', gsub('\\.tsv$', '',
    basename(reportfiles)))

        # Get total_reads and total_genomes from first line of report
    totals <- data.frame(row.names=report_base, do.call(rbind,
    lapply(reportfiles, function(rf){read.table(rf, sep='\t', nrows=1)})
    ))
    totals <- totals[,c('V2','V4')]
    colnames(totals) <- c('total_reads','total_genomes')

    # Read all reports into list
    mlist <- lapply(reportfiles, function(rf){
    read.table(rf, sep='\t', header=TRUE, stringsAsFactors=FALSE,
    row.names=1, skip=1, comment.char="")
    })

    if(!is.null(nrows)) mlist <- lapply(mlist, 
    function(tbl){tbl[seq_len(nrows), ]})

    # Get column names for variables in reports
    vals <- colnames(mlist[[1]])
    # Get unique genomes across all reports
    genomes <- unique(do.call(c,lapply(mlist,function(tbl){rownames(tbl)})))

    # Set unobserved genome to 0
    mlist.allgenomes <- lapply(mlist, function(tbl){
    z <- tbl[genomes,]
    rownames(z) <- genomes
    z[is.na(z)] <- 0
    z
    })
    # Sanity check: colnames and rownames should be the same for all tables
    stopifnot(all(vapply(mlist.allgenomes,
    function(tbl){all(rownames(tbl)==genomes)})))
    stopifnot(all(vapply(mlist.allgenomes,
    function(tbl){all(colnames(tbl)==vals)})))

    ret <- lapply(vals, function(v) {
    z <- data.frame(row.names=genomes, do.call(cbind, lapply(
    mlist.allgenomes,function(tbl){ tbl[,v]})))
    colnames(z) <- report_base
    z
    })
    names(ret) <- vals
    ret[['total_reads']] <- totals[report_base, 'total_reads']
    names(ret[['total_reads']]) <- report_base
    ret[['total_genomes']] <- totals[report_base, 'total_genomes']
    names(ret[['total_genomes']]) <- report_base

    ret
}

prop_hash <- function(tbl) {
    prop_hash <- new.env()
    sum <- 0
    for (i in seq_len(nrow(tbl))) {
        prop_hash[[as.character(tbl[i, 1])]] <- tbl[i, 2]
        sum <- sum + tbl[i, 2]
    }
    if (sum < 1) {
        prop_hash[["others"]] <- 1 - sum
    }
    return(prop_hash)
}

proportion <- function(hasht, genomes, countdata = FALSE, numReads = 1) {
    prop <- c()
    for (genome in genomes) {
        if (is.null(hasht[[as.character(genome)]])) {
            prop <- c(prop, 0)
        } else {
            if (countdata) {
                prop <- c(prop, round(numReads * hasht[[as.character(genome)]]))
            } else {
                prop <- c(prop, hasht[[as.character(genome)]])
            }
        }
    }
    return(prop)
}


proportionc <- function(lhasht, genomes, lnumReads, i) {
    propc <- proportion(lhasht[[i]], genomes, countdata = TRUE,
        numReads=lnumReads[i])
    return(propc)
}

#' Greps the tid from the given identifier string
#'
#' @param id Given identifier string
#' @return tid string
#' @export
#' @examples
#' grepTid("ti|700015|org|Coriobacterium_glomerans_PW2")

grepTid <- function(id) {
    tid <- strsplit(id, "\\|")
    tid <- sapply(tid, function(x) x[2])
    return(tid)
}


#' Save the pathostat object to R data(.rda) file
#'
#' @param pstat pathostat object
#' @param outdir Output Directory of the .rda file
#' @param outfileName File name of the .rda file
#' @return outfile .rda file
#' @export
#' @examples
#' data_dir_test <- system.file("data", package = "PathoStat")
#' pstat_test <- loadPstat(indir=data_dir_test,
#' infileName="pstat_data_2_L1.rda")
#' outfile <- savePstat(pstat_test)
savePstat <- function(pstat, outdir=".", outfileName="pstat_data.rda") {
    if (outdir == ".") {
        outdir <- getwd()
    }
    outfile <- file.path(outdir, outfileName)
    save(pstat, file=outfile)
    return(outfile)
}

#' Load the R data(.rda) file with pathostat object
#'
#' @param indir Input Directory of the .rda file
#' @param infileName File name of the .rda file
#' @return pstat pathostat object (NULL if it does not exist)
#' @export
#' @examples
#' data_dir <- system.file("data", package = "PathoStat")
#' infileName <- "pstat_data.rda"
#' pstat <- loadPstat(data_dir, infileName)

loadPstat <- function(indir=".", infileName="pstat_data.rda") {
    if (indir == ".") {
        indir <- getwd()
    }
    infile <- file.path(indir, infileName)
    pstat <- NULL
    load(file=infile)
    return(pstat)
}

#' Return the Relative Abundance (RA) data for the given count OTU table
#'
#' @param count_otu Count OTU table
#' @return ra_otu Relative Abundance (RA) OTU table
#' @export
#' @examples
#' data_dir <- system.file("data", package = "PathoStat")
#' infileName <- "pstat_data.rda"
#' pstat_test <- loadPstat(data_dir, infileName)
#' ra_otu <- findRAfromCount(phyloseq::otu_table(pstat_test))

findRAfromCount <- function(count_otu) {
    ra_otu <- otu_table(count_otu)
    numcol <- dim(count_otu)[2]
    for (i in seq_len(numcol))  {
        ra_otu[,i] <- ra_otu[,i]/sum(ra_otu[,i])
    }
    return(ra_otu)
}

#' Format taxonomy table for rendering
#'
#' @param ttable Taxonomy table
#' @import DT tidyr
#' @return Formatted table suitable for rendering with. DT::renderDataTable
formatTaxTable <- function(ttable) {
    ttable
}


#' Summarize sample
#'
#' Creates a table of summary metrics
#'
#' @param pstat Input pstat
#'
#' @return A data.frame object of summary metrics.
#' @export summarizeTable
#' @examples
#' data_dir_test <- system.file("data", package = "PathoStat")
#' pstat_test <- loadPstat(indir=data_dir_test,
#' infileName="pstat_data_2_L1.rda")
#' st.tmp <- summarizeTable(pstat_test)

summarizeTable <- function(pstat){
    return(data.frame("Metric" = c("Number of Samples",
    "Number of taxon elements",
    "Average number of reads per sample",
    "Average number of taxon elements per sample",
    "Samples with <5 taxon elements",
    "Taxon elements with reads across all samples"),
    "Value" = c(ncol(pstat@otu_table@.Data),
    nrow(pstat@otu_table@.Data),
    as.integer(mean(colSums(pstat@otu_table@.Data))),
    as.integer(mean(colSums(pstat@otu_table@.Data > 0))),
    sum(colSums(pstat@otu_table@.Data != 0) < 5),
    sum(rowSums(pstat@otu_table@.Data) == 0))))
}





#' Compute percentage
#'
#' @param x a number or a vector
#' @param digits how many digit of percentage
#' @param format numeric format, "f" for float
#' @return the percentage
#' @export
#' @examples
#' pecent.vec <- percent(c(0.9, 0.98))
percent <- function(x, digits = 2, format = "f") {
    paste0(formatC(100 * x, format = format, digits = digits), "%")
}

#' Plot PCA
#'
#' @param df.input Input data object that contains
#' the data to be plotted. Required
#' @param condition.color.vec color vector. Required
#' @param condition.color.name color variable name. Required
#' @param condition.shape.vec shape vector. Required
#' @param condition.shape.name shape variable name. Required
#' @param columnTitle Title to be displayed at top of heatmap.
#' @param pc.a pc.1
#' @param pc.b pc.2
#' @importFrom plotly plot_ly
#' @importFrom shinyjs useShinyjs
#' @import webshot vegan devtools
#' @return the plot
#' @export
#' @examples
#' data('iris')
#' plotPCAPlotly(t(iris[,1:4]),
#' condition.color.vec = c(rep(1,100), rep(0,50)),
#' condition.shape.vec = c(rep(0,100), rep(1,50)))

plotPCAPlotly <- function(df.input,
    condition.color.vec, condition.color.name = "condition",
    condition.shape.vec=NULL, condition.shape.name = "condition",
    columnTitle = "Title", pc.a = "PC1", pc.b = "PC2"){

    # Test and fix the constant/zero row
    if (sum(rowSums(as.matrix(df.input)) == 0) > 0){
        df.input <- df.input[-which(rowSums(as.matrix(df.input)) == 0),]
    }

    # conduct PCA
    pca.tmp<- prcomp(t(df.input), scale = TRUE)
    tmp.df <- data.frame(pca.tmp$x)

    # add color variable
    tmp.df[[paste(condition.color.name)]] <- condition.color.vec
    # add shape variable
    if (!is.null(condition.shape.vec)) {
        tmp.df[[paste(condition.shape.name)]] <- condition.shape.vec
        p <- suppressWarnings(plot_ly(tmp.df,
        x = as.formula(paste("~", pc.a, sep = "")),
        y = as.formula(paste("~", pc.b, sep = "")),
        mode = "markers",
        color = as.formula(paste("~", 
            condition.color.name, sep = "")),
            symbol = as.formula(paste("~", 
            condition.shape.name, sep = "")),
            type = "scatter",
            text = rownames(tmp.df),
            marker = list(size = 10)))
    } else {
        p <- suppressWarnings(plot_ly(tmp.df,
            x = as.formula(paste("~", pc.a, sep = "")),
            y = as.formula(paste("~", pc.b, sep = "")),
            mode = "markers",
            color = as.formula(paste("~", 
            condition.color.name, sep = "")),
            type = "scatter",
            text = rownames(tmp.df),
            marker = list(size = 10)))        
    }
    return(p)
}

#' Plot PCoA
#'
#' @param physeq.input Input data object that contains
#' the data to be plotted. Required
#' @param method which distance metric
#' @param condition.color.vec color vector. Required
#' @param condition.color.name color variable name. Required
#' @param condition.shape.vec shape vector. Required
#' @param condition.shape.name shape variable name. Required
#' @param columnTitle Title to be displayed at top of heatmap.
#' @param pc.a pc.1
#' @param pc.b pc.2
#' @importFrom plotly plot_ly
#' @return the plot
#' @export
#' @examples
#' data_dir_test <- system.file("data", package = "PathoStat")
#' pstat_test <- loadPstat(indir=data_dir_test,
#' infileName="pstat_data_2_L1.rda")
#' plotPCoAPlotly(pstat_test, condition.color.vec = rbinom(33,1,0.5),
#' condition.shape.vec = rbinom(33,1,0.5))

plotPCoAPlotly <- function(physeq.input,
    condition.color.vec, condition.color.name = "condition",
    condition.shape.vec=NULL, condition.shape.name = "condition",
    method = "bray", columnTitle = "Title",
    pc.a = "Axis.1", pc.b = "Axis.2"){
    # conduct PCoA
    # wUniFrac or bray

    #test and fix the constant/zero row
    if (sum(rowSums(as.matrix(physeq.input@otu_table@.Data)) == 0) > 0){
        physeq.input@otu_table@.Data <-
        physeq.input@otu_table@.Data[-which(rowSums(
        as.matrix(physeq.input@otu_table@.Data)) == 0),]
    }

    if (method == "bray"){
        #First get otu_table and transpose it:
        dist.matrix <- t(data.frame(otu_table(physeq.input)))
        #Then use vegdist from vegan to generate a bray distance object:
        DistBC <- vegdist(dist.matrix, method = "bray")
        #DistBC = phyloseq::distance(physeq.input, method = method)
        ordBC = ordinate(physeq.input, method = "PCoA", distance = DistBC)
        tmp.df <- data.frame(ordBC$vectors)
    } else {
        DistUF = phyloseq::distance(physeq.input, method = method)
        ordUF = ordinate(physeq.input, method = "PCoA", distance = DistUF)
        tmp.df <- data.frame(ordUF$vectors)
    }

    # add color variable
    tmp.df[[paste(condition.color.name)]] <- condition.color.vec
    # add shape variable
    if (!is.null(condition.shape.vec)) {
        tmp.df[[paste(condition.shape.name)]] <- condition.shape.vec
        p <- suppressWarnings(plot_ly(tmp.df,
                    x = as.formula(paste("~", pc.a, sep = "")),
                    y = as.formula(paste("~", pc.b, sep = "")),
                    mode = "markers",
                    color = as.formula(paste("~", 
                    condition.color.name, sep = "")),
                    symbol = as.formula(paste("~", 
                    condition.shape.name, sep = "")),
                    type = "scatter",
                    text = rownames(tmp.df),
                    marker = list(size = 10)))
    } else {
        p <- suppressWarnings(plot_ly(tmp.df,
                    x = as.formula(paste("~", pc.a, sep = "")),
                    y = as.formula(paste("~", pc.b, sep = "")),
                    mode = "markers",
                    color = as.formula(paste("~", 
                    condition.color.name, sep = "")),
                    type = "scatter",
                    text = rownames(tmp.df),
                    marker = list(size = 10)))
    }
    return(p)
}

################################################################################
#' Convert phyloseq OTU count data into DGEList for edgeR package
#'
#' Further details.
#'
#' @param physeq (Required).
#' @param group (Required). A character vector or factor giving the experimental
#' group/condition for each sample/library.
#' @param method (Optional).
#'
#' @param ... Additional arguments passed on to
#'
#' @import edgeR phyloseq
#' @return dispersion
#' @export
#' @examples
#' data_dir_test <- system.file("data", package = "PathoStat")
#' pstat_test <- loadPstat(indir=data_dir_test,
#' infileName="pstat_data_2_L1.rda")
#' phyloseq_to_edgeR(pstat_test, group="Sex")


phyloseq_to_edgeR = function(physeq, group, method="RLE", ...){

    # Enforce orientation.
    if( !taxa_are_rows(physeq) ){ physeq <- t(physeq) }
    x = as(otu_table(physeq), "matrix")
    # Add one to protect against overflow, log(0) issues.
    x = x + 1
    # Check `group` argument
    if( identical(all.equal(length(group), 1), TRUE) & nsamples(physeq) > 1 ){
        # Assume that group was a sample variable name (must be categorical)
        group = get_variable(physeq, group)
    }
    # Define gene annotations (`genes`) as tax_table
    taxonomy = tax_table(physeq, errorIfNULL=FALSE)
    if( !is.null(taxonomy) ){
        taxonomy = data.frame(as(taxonomy, "matrix"))
    }
    # Now turn into a DGEList
    y = DGEList(counts=x, group=group, genes=taxonomy, remove.zeros = TRUE, ...)
    # Calculate the normalization factors
    z = calcNormFactors(y, method=method)
    # Check for division by zero inside `calcNormFactors`
    if( !all(is.finite(z$samples$norm.factors)) ){
        stop("Something wrong with edgeR::calcNormFactors on this data,
    non-finite $norm.factors, consider changing `method` argument")
    }
    # Estimate dispersions
    return(estimateTagwiseDisp(estimateCommonDisp(z)))
}
################################################################################



#' Mann-whitney test for a dataframe
#'
#' @param df Input data object that contains the data to be tested. Required
#' @param label.vec.num The target binary condition. Required
#' @param pvalue.cutoff choose p-value cut-off
#' @return df.output object
#' @export
#' @examples
#' data('iris')
#' Wilcox_Test_df(t(iris[,1:4]),
#' c(rep(1,100), rep(0,50)))

Wilcox_Test_df <- function(df, label.vec.num, pvalue.cutoff = 0.05) {
    df.output <- NULL
    #save raw values
    label.vec.save <- unique(label.vec.num)

    # transform label into 1 and 0
    label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
    label.vec.num[label.vec.num != 1] <- 0
    label.vec.num <- as.numeric(label.vec.num)
    for (i in seq_len(nrow(df))){
    # remove zero-variance rows
    if (sum(df[i,] == 1) == length(label.vec.num) |
    sum(df[i,] == 0) == length(label.vec.num)){
        next
    }
    tmp.result <- suppressWarnings(wilcox.test(df[i,which(label.vec.num == 1)],
    df[i,which(label.vec.num == 0)], correct=FALSE, exact=FALSE))
    if (tmp.result$p.value <= pvalue.cutoff & rownames(df)[i] != "others"){
        num.1 <- sum((df[i,which(label.vec.num == 1)] > 0))
        num.2 <- sum((df[i,which(label.vec.num == 0)] > 0))
        df.output <- rbind(df.output, c(rownames(df)[i],
        round(as.numeric(tmp.result$p.value), 4), num.1, num.2))
    }
    }
    if (is.null(df.output)){
        return(0)
    }
    df.output.prevalence <- percent(round((as.numeric(df.output[,3])+
    as.numeric(df.output[,4]))/ncol(df),4))
    df.output <- cbind(df.output, df.output.prevalence)
    colnames(df.output) <- c("Name", "FDR",
    label.vec.save[1],label.vec.save[2], "prevalence")
    # FDR adjustment
    df.output <- data.frame(df.output, stringsAsFactors = FALSE)
    df.output[,2] <- as.numeric(df.output[,2])
    df.output[,3] <- as.numeric(df.output[,3])
    df.output[,4] <- as.numeric(df.output[,4])
    df.output[,2] <- p.adjust(df.output[,2], method = "fdr",
    n = length(df.output[,2]))
    df.output <- df.output[order(df.output[,2]),]
    foldChange <- c()
    for (i in seq_len(nrow(df.output))){
        foldChange[i] <- round((max(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1))))) /
        min(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1)))))), 
        digits = 2)
    }
    df.output <- cbind(df.output, foldChange)
    colnames(df.output)[ncol(df.output)] <- "Group Size adjusted fold change"
    return(df.output)
}






#' transform cpm counts to presence-absence matrix
#'
#' @param df Input data object that contains the data to be tested.
#' Required
#' @return df.output object
#' @export
#' @examples
#' GET_PAM(data.frame(a = c(1,3,0), b = c(0,0.1,2)))

GET_PAM <- function(df) {
    for (i in seq_len(nrow(df))){
        df[i,] <- as.numeric(df[i,] > 0)
    }
    return(df)
}



#' Given PAM and disease/control annotation,
#' do Chi-square test for each row of PAM
#'
#' @param pam Input data object that contains the data to be tested. Required
#' @param label.vec.num The target binary condition. Required
#' @param pvalue.cutoff choose p-value cut-off
#' @return df.output object
#' @export
#' @examples
#' tmp <- matrix(rbinom(12,1,0.5), nrow = 3)
#' rownames(tmp) <- c("a", "b", "c")
#' Chisq_Test_Pam(tmp, c(1,1,0,0))


Chisq_Test_Pam <- function(pam, label.vec.num, pvalue.cutoff = 0.05) {
    df.output <- NULL

    #save raw values
    label.vec.save <- unique(label.vec.num)

    # transform label into 1 and 0
    label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
    label.vec.num[label.vec.num != 1] <- 0
    label.vec.num <- as.numeric(label.vec.num)

    for (i in seq_len(nrow(pam))){
    # remove zero-variance rows
    if (sum(pam[i,] == 1) == length(label.vec.num) |
        sum(pam[i,] == 0) == length(label.vec.num)){
        next
    }
    tmp.result <- suppressWarnings(chisq.test(pam[i,], 
    label.vec.num, correct=FALSE))
    if (tmp.result$p.value <= pvalue.cutoff  & rownames(pam)[i] != "others"){
        num.1 <- sum(pam[i,] == 1 & label.vec.num == 1)
        num.2 <- sum(pam[i,] == 1 & label.vec.num == 0)
        df.output <- rbind(df.output, c(rownames(pam)[i],
        round(as.numeric(tmp.result$p.value), 4), num.1, num.2))
    }
    }
    if (is.null(df.output)){
        return(0)
    }
    df.output.prevalence <- percent(round((as.numeric(df.output[,3])+
    as.numeric(df.output[,4]))/ncol(pam),4))
    df.output <- cbind(df.output, df.output.prevalence)
    colnames(df.output) <- c("Name", "FDR",
    label.vec.save[1], label.vec.save[2], "prevalence")
    # FDR adjustment
    df.output <- data.frame(df.output, stringsAsFactors = FALSE)
    df.output[,2] <- as.numeric(df.output[,2])
    df.output[,3] <- as.numeric(df.output[,3])
    df.output[,4] <- as.numeric(df.output[,4])
    df.output[,2] <- p.adjust(df.output[,2], method = "fdr",
    n = length(df.output[,2]))
    df.output <- df.output[order(df.output[,2]),]
    foldChange <- c()
    for (i in seq_len(nrow(df.output))){
        foldChange[i] <- round((max(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1))))) /
        min(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1)))))), 
        digits = 2)
    }
    df.output <- cbind(df.output, foldChange)
    colnames(df.output)[ncol(df.output)] <- "Group Size adjusted fold change"
    return(df.output)
}



#' Given PAM and disease/control annotation,
#' do Chi-square test for each row of PAM
#'
#' @param pam Input data object that contains the data to be tested. Required
#' @param label.vec.num The target binary condition. Required
#' @param pvalue.cutoff choose p-value cut-off
#' @return df.output object
#' @export
#' @examples
#' tmp <- matrix(rbinom(12,1,0.5), nrow = 3)
#' rownames(tmp) <- c("a", "b", "c")
#' Fisher_Test_Pam(tmp, c(1,1,0,0))

Fisher_Test_Pam <- function(pam, label.vec.num, pvalue.cutoff = 0.05) {
    df.output <- NULL

    #save raw values
    label.vec.save <- unique(label.vec.num)

    #  transform label into 1 and 0
    label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
    label.vec.num[label.vec.num != 1] <- 0
    label.vec.num <- as.numeric(label.vec.num)
    for (i in seq_len(nrow(pam))){
        # remove zero-variance rows
        if (sum(pam[i,] == 1) == length(label.vec.num) |
        sum(pam[i,] == 0) == length(label.vec.num)){
        next
    }
    tmp.result <- suppressWarnings(fisher.test(pam[i,], label.vec.num))
    #print(tmp.result$p.value)
    if (tmp.result$p.value <= pvalue.cutoff  & rownames(pam)[i] != "others"){
        more.in.case <- sum(pam[i,] == 1 & label.vec.num == 1) >
        sum(pam[i,] == 1 & label.vec.num == 0)
        num.1 <- sum(pam[i,] == 1 & label.vec.num == 1)
        num.2 <- sum(pam[i,] == 1 & label.vec.num == 0)
        df.output <- rbind(df.output, c(rownames(pam)[i],
        round(as.numeric(tmp.result$p.value), 4), num.1, num.2))
    }
    }
    #return(df.output)
    if (is.null(df.output)){
        return(0)
    }
    df.output.prevalence <- percent(round((as.numeric(df.output[,3])+
    as.numeric(df.output[,4]))/ncol(pam),4))
    df.output <- cbind(df.output, df.output.prevalence)
    colnames(df.output) <- c("Name", "FDR", label.vec.save[1],
    label.vec.save[2], "prevalence")
    # FDR adjustment
    df.output <- data.frame(df.output, stringsAsFactors = FALSE)
    df.output[,2] <- as.numeric(df.output[,2])
    df.output[,3] <- as.numeric(df.output[,3])
    df.output[,4] <- as.numeric(df.output[,4])
    df.output[,2] <- p.adjust(df.output[,2], method = "fdr",
    n = length(df.output[,2]))
    df.output <- df.output[order(df.output[,2]),]
    foldChange <- c()

    for (i in seq_len(nrow(df.output))){
        foldChange[i] <- round((max(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1))))) /
        min(as.numeric(c((df.output[i,4] / 
        sum(label.vec.num == 0)),
        (df.output[i,3] / 
        sum(label.vec.num == 1)))))), 
        digits = 2)
    }
    df.output <- cbind(df.output, foldChange)
    colnames(df.output)[ncol(df.output)] <- "Group Size adjusted fold change"
    return(df.output)
}

################################################################################
#' PathoStat class to store PathoStat input data including phyloseq object
#'
#' Contains all currently-supported BatchQC output data classes:
#'
#' slots:
#' \describe{
#'     \item{average_count}{a single object of class otu_tableOrNULL}
#'     \item{besthit_count}{a single object of class otu_tableOrNULL}
#'     \item{highconf_count}{a single object of class otu_tableOrNULL}
#'     \item{lowconf_count}{a single object of class otu_tableOrNULL}
#' }
#'
#' @export pathostat1
#' @examples
#' otumat = matrix(sample(1:100, 100, replace = TRUE), nrow = 10, ncol = 10)
#' rownames(otumat) <- paste0("OTU", 1:nrow(otumat))
#' colnames(otumat) <- paste0("Sample", 1:ncol(otumat))
#' taxmat = matrix(sample(letters, 70, replace = TRUE), 
#' nrow = nrow(otumat), ncol = 7)
#' rownames(taxmat) <- rownames(otumat)
#' colnames(taxmat) <- c("Domain", "Phylum", "Class", 
#' "Order", "Family", "Genus", "Species")
#' OTU = phyloseq::otu_table(otumat, taxa_are_rows = TRUE)
#' TAX = phyloseq::tax_table(taxmat)
#' physeq = phyloseq::phyloseq(OTU, TAX)
#' pathostat1(physeq)
#' 
#' @exportClass PathoStat
pathostat1 <- setClass(Class="PathoStat",
    representation=representation(
        average_count="otu_tableOrNULL",
        besthit_count="otu_tableOrNULL",
        highconf_count="otu_tableOrNULL",
        lowconf_count="otu_tableOrNULL"
    ), contains = "phyloseq",
    prototype=prototype(average_count=NULL, besthit_count=NULL,
        highconf_count=NULL, lowconf_count=NULL)
)
###############################################################################

tabPanel("Upload",
useShinyjs(),
tags$style(appCSS),
tags$div(
    class = "jumbotron",
    tags$div(
        class = "container",
        fluidRow(
            column(7, h1("PathoStat")),
            column(5, br(),br(),img(src = "bu_bioinfo_logo.png", height = 80, width = 80))
            # column(2, br(),br(),img(src = "cbm_logo.png", height = 80, width = 140)),
            # column(1, br(),br(),img(src = "bu_bioinfo_logo.png", height = 80, width = 80))

        ),
        p("Statistical Microbiome Analysis Toolkit")
    )
),
sidebarLayout(
   sidebarPanel(
       radioButtons("uploadChoice", "Upload:",
                    c("Example data" = "example",
                      "Count File" = "count",
                      "PathoScope Files" = "pathofiles",
                      "PathoStat file" = "pathostat.file"
                    )),
       br(),
       p(" "),
       conditionalPanel(condition = sprintf("input['%s'] == 'example'", "uploadChoice"),
                        helpText("Example data is loaded and ready to go!")
       ),
       conditionalPanel(condition = sprintf("input['%s'] == 'pathostat.file'", "uploadChoice"),
                        fileInput("rdfile", ".rda/.rds file (required):",
                                  accept = c(
                                    ".rda",
                                    ".rds"
                                  )
                        ),
                        radioButtons("rdtype", "Filetype",
                                     choices = c(rda = "rda",
                                                 rds = "rds"
                                     ),
                                     selected = "rda"
                        ),
                        withBusyIndicatorUI(
                          actionButton("uploadPathoStat",
                                       "Upload",
                                       class = "btn-primary")
                        )

       ),
       conditionalPanel(condition = sprintf("input['%s'] == 'count'", "uploadChoice"),
                        fileInput("countsfile", "Counts .csv file (required):",
                                  accept = c(
                                      "text/csv",
                                      "text/comma-separated-values",
                                      "text/tab-separated-values",
                                      "text/plain",
                                      ".csv",
                                      ".tsv"
                                  )
                        ),
                        fileInput("taxon.table", "Taxonomy table .csv file (required):",
                                  accept = c(
                                    "text/csv",
                                    "text/comma-separated-values",
                                    "text/tab-separated-values",
                                    "text/plain",
                                    ".csv",
                                    ".tsv"
                                  )
                        ),
                        fileInput("annotfile.count", "Annotation .csv file (required):",
                                  accept = c(
                                      "text/csv",
                                      "text/comma-separated-values",
                                      "text/tab-separated-values",
                                      "text/plain",
                                      ".csv",
                                      ".tsv"
                                  )
                        ),
                        numericInput("metadata_sample_name_col_count", "Which column in metadata is sample name?",
                                     value = 1),
                        # Input: Checkbox if file has header ----
                        checkboxInput("header.count", "Header", TRUE),

                        # Input: Select separator ----
                        radioButtons("sep.count", "Separator",
                                     choices = c(Tab = "\t",
                                                 Comma = ",",
                                                 Semicolon = ";"
                                     ),
                                     selected = ","),
                        withBusyIndicatorUI(
                            actionButton("uploadDataCount",
                                         "Upload",
                                         class = "btn-primary")
                        ),
                        helpText("After click Upload, please wait until seeing a green check.")
       ),
       conditionalPanel(condition = sprintf("input['%s'] == 'pathofiles'", "uploadChoice"),
                        h5("Upload PathoScope generated .tsv files:"),
                        fileInput("countsfile.pathoscope", "PathoScope outputs (required):",
                                  multiple = TRUE,
                                  accept = c(
                                      "text/csv",
                                      "text/comma-separated-values",
                                      "text/tab-separated-values",
                                      "text/plain",
                                      ".csv",
                                      ".tsv"
                                  )
                        ),
                        fileInput("annotfile.ps", "Annotation .tsv file (required):",
                                  accept = c(
                                      "text/csv",
                                      "text/comma-separated-values",
                                      "text/tab-separated-values",
                                      "text/plain",
                                      ".csv",
                                      ".tsv"
                                  )
                        ),
                        textInput("report_suffix", "Report suffix", value = "-sam-report.tsv"),
                        numericInput("metadata_sample_name_col", "Which column in metadata is sample name?",
                                     value = 1),
                        # Input: Checkbox if file has header ----
                        checkboxInput("header.ps", "Header", TRUE),

                        # Input: Select separator ----
                        radioButtons("sep.ps", "Separator",
                                     choices = c(Tab = "\t",
                                                 Comma = ",",
                                                 Semicolon = ";"
                                     ),
                                     selected = "\t"),
                        withBusyIndicatorUI(
                            actionButton("uploadDataPs",
                                         "Upload",
                                         class = "btn-primary")
                        ),
                        helpText("After click, please wait for 30s until seeing a green check.")

       )
   ),
   mainPanel(
       conditionalPanel(condition = "input.uploadChoice === 'pathofiles'",
                        h4("Please click \"open in browser\" for enabling functions like multiple files upload."),
                        helpText("Counts Table: column names must be sample name"),
                        DT::dataTableOutput("contents.count"),
                        helpText("Annotation table"),
                        DT::dataTableOutput("contents.meta")
       ),
       conditionalPanel(condition = "input.uploadChoice === 'count'",

                        tags$img(src='count_table_example.png', height = 180, width = 800),
                        helpText("Counts Table: column names must be sample name"),
                        helpText("The first column must be microbe name"),

                        DT::dataTableOutput("contents.count.2"),
                        helpText("Taxonomy Table: column names must be taxonomy levels, like family, genus, species..."),
                        helpText("The first column must be microbe name"),

                        DT::dataTableOutput("contents.taxonomy"),
                        helpText("Annotation table"),

                        DT::dataTableOutput("contents.meta.2")
        )
      )
   )
)

#' pathostat object generated from example pathoscope report files
#'
#' This example data consists of 33 samples from a diet study with 11 subjects
#' taking 3 different diets in random order
#'
#' @name pstat_data
#' @format pathostat object extension of phyloseq-class experiment-level object:
#' \describe{
#'     \item{otu_table}{OTU table with  41 taxa and 33 samples}
#'     \item{sample_data}{Sample Data with 33 samples by 18 sample variables}
#'     \item{tax_table}{Taxonomy Table with 41 taxa by 9 taxonomic ranks}
#'     \item{sample_data}{Phylogenetic Tree with 41 tips and 40 internal nodes}
#' }
#' @return pathostat object
"pstat"

###############################################################################
#' Statistical Microbiome Analysis on the pathostat input and generates a
#' html report and produces interactive shiny app plots
#'
#' @param pstat phyloseq extension pathostat object
#' @param report_dir Output report directory path
#' @param report_option_binary 9 bits Binary String representing the plots to
#'  display and hide in the report
#' @param interactive when TRUE, opens the interactive shinyApp
#' @return outputfile The output file with all the statistical plots
#' @import stats graphics reshape2 ggplot2 rentrez phyloseq ape
#' @import corpcor knitr limma matrixStats methods BiocStyle
#' @import edgeR
#' @importFrom DESeq2 DESeqDataSetFromMatrix sizeFactors estimateDispersions
#'  nbinomWaldTest results
#' @importFrom plyr ddply numcolwise
#' @importFrom dplyr add_rownames mutate_each select_ add_rownames
#' @importFrom scales percent_format
#' @importFrom utils browseURL
#' @importFrom shiny runApp
#' @importFrom shiny shinyOptions getShinyOption
#' @export
#' @examples
#' runPathoStat(interactive = FALSE)
runPathoStat <- function(pstat=NULL,
    report_dir=".", report_option_binary="111111111",interactive=TRUE) {

    if (is.null(pstat))  {
        data_dir <- system.file("data", package = "PathoStat")
        infileName <- "pstat_data.rda"
        pstat <- loadPstat(data_dir, infileName)
    }
    if (report_dir == ".") {
        report_dir = getwd()
    }

    #test and fix the constant/zero row
    row.remove.index <- c()
    if (sum(rowSums(as.matrix(pstat@otu_table@.Data)) == 0) > 0){
        row.remove.index <-
        which(rowSums(as.matrix(pstat@otu_table@.Data)) == 0)
        pstat@otu_table@.Data <- pstat@otu_table@.Data[-row.remove.index,]
        pstat@tax_table@.Data <- pstat@tax_table@.Data[-row.remove.index,]
    }


    # remove variables with identical values
    col.remove.index <- c()
    for (i in seq_len(ncol(pstat@sam_data))){
        if(dim(unique(pstat@sam_data[,i]))[1] < 2){
            col.remove.index <- c(col.remove.index, i)
        }
    }
    if (!is.null(col.remove.index)){
        pstat@sam_data <- pstat@sam_data[,-col.remove.index]
    }

    shinyInput <- list(pstat = pstat, report_dir = report_dir)
    setShinyInput(shinyInput)
    rmdfile <- system.file("reports/pathostat_report.Rmd", package =
        "PathoStat")
    report_option_vector <- unlist(strsplit(as.character(report_option_binary),
        ""))
    static_lib_dir <- system.file("reports/libs", package = "PathoStat")
    file.copy(static_lib_dir, report_dir, recursive = TRUE)
    shinyInput <- getShinyInput()
    setShinyInputOrig(shinyInput)
    setShinyInputCombat(NULL)

    if (interactive) {
        appDir <- system.file("shiny", "PathoStat", package = "PathoStat")
        if (appDir == "") {
            stop("Could not find shiny directory. Try re-installing PathoStat.",
                call. = FALSE)
        }
        shiny::runApp(appDir, display.mode = "normal")
    }
}

#' Getter function to get the shinyInput option
#' @return shinyInput option
#' @export
#' @examples
#' getShinyInput()
getShinyInput <- function() {
    shinyInput <- getShinyOption("pathostat.shinyInput")
    return(shinyInput)
}
#' Setter function to set the shinyInput option
#' @param x shinyInput option
#' @return shinyInput option
#' @export
#' @examples
#' setShinyInput(NULL)
setShinyInput <- function(x) {
    shinyOptions(pathostat.shinyInput = x)
}

#' Getter function to get the shinyInputOrig option
#' @return shinyInputOrig option
#' @export
#' @examples
#' getShinyInputOrig()
getShinyInputOrig <- function() {
    shinyInputOrig <- getShinyOption("pathostat.shinyInputOrig")
    return(shinyInputOrig)
}
#' Setter function to set the shinyInputOrig option
#' @param x shinyInputOrig option
#' @return shinyInputOrig option
#' @export
#' @examples
#' setShinyInputOrig(NULL)
setShinyInputOrig <- function(x) {
    shinyOptions(pathostat.shinyInputOrig = x)
}

#' Getter function to get the shinyInputCombat option
#' @return shinyInputCombat option
#' @export
#' @examples
#' getShinyInputCombat()
getShinyInputCombat <- function() {
    shinyInputCombat <- getShinyOption("pathostat.shinyInputCombat")
    return(shinyInputCombat)
}
#' Setter function to set the shinyInputCombat option
#' @param x shinyInputCombat option
#' @return shinyInputCombat option
#' @export
#' @examples
#' setShinyInputCombat(NULL)
setShinyInputCombat <- function(x) {
    shinyOptions(pathostat.shinyInputCombat = x)
}

tabPanel("Summary and Filter",
  tabsetPanel(
    tabPanel("Filtering",
      br(),
      sidebarLayout(
        sidebarPanel(
          br(),
          selectInput("filter_type", "Select filter type", c("By Metadata", "By Samples", "By Microbes")),

          # Filtering by metadata
          conditionalPanel(condition = "input.filter_type == 'By Metadata'",
            selectInput("select_condition_sample_filter_sidebar", "Select a variable", c("Reads", covariates)),
            conditionalPanel(condition = "output.filter_type == 'num.continuous'",
              helpText("Please select range of the feature to keep"),
              numericInput("num_filter_min", "Min", 0),
              numericInput("num_filter_max", "Max", 0),
              withBusyIndicatorUI(
                actionButton("filter_num", "Filter")
              )
            ),
            conditionalPanel(condition = "output.filter_type == 'cat'",
              helpText("Please select levels of the feature to keep"),
              uiOutput("filter_cat_options"),
              withBusyIndicatorUI(
                actionButton("filter_cat", "Filter")
              )
            )
          ),
          
          # Filtering by sample
          conditionalPanel(condition = "input.filter_type == 'By Samples'",
            selectizeInput('filterSample', 'Choose sample name(s) for removal', 
                           choices = sample.names.all, selected=NULL, multiple = TRUE),
            withBusyIndicatorUI(
              actionButton("filterSampleButton", "Filter")
            )
          ),

          # Filtering by microbes
          conditionalPanel(condition = "input.filter_type == 'By Microbes'",
            br(),
            selectInput("filter_type_micro", "Select filter type",
                 c("By mapped read number",
                   "By relative abundace",
                   "By prevalence")
            ),
            conditionalPanel(condition = "input.filter_type_micro == 'By mapped read number'",
              helpText("Please select the average minimum read mapped"),
              numericInput("read_filter_min_micro", "Min", 0, min = 0, max = 10000),
              withBusyIndicatorUI(
                actionButton("filter_read_micro", "Filter")
              )
            ),
            conditionalPanel(condition = "input.filter_type_micro == 'By relative abundace'",
              helpText("Please select average minimum RA"),
              numericInput("ra_filter_min_micro", "Min", 0, min = 0, max = 1),
              withBusyIndicatorUI(
                actionButton("filter_ra_micro", "Filter")
              )
            ),
            conditionalPanel(condition = "input.filter_type_micro == 'By prevalence'",
              helpText("Please select minimum prevalence"),
              numericInput("prev_filter_min", "Min", 0, min = 0, max = 1),
              withBusyIndicatorUI(
                actionButton("filter_prev_micro", "Filter")
              )
            )
          ),
          br(),
          withBusyIndicatorUI(
            actionButton("reset_button", "Reset")
          ),
          br(),    
          downloadButton('download_rds', 'Download (.rds)'),
          width=3
        ),
        mainPanel(
          fluidRow(
            column(6,
              h4("Overall Summary Table", align="center"),
              br(),
              tableOutput("contents_summary")
            ),
            column(6,
              h4("Variable Summary Plots", align="center"),
              br(),
              plotlyOutput("summary_plot_top", height="300px"),
              plotlyOutput("summary_plot_bottom", height="300px")
            )
          ), 
          width=9
        )
      )
    ),
    tabPanel("Categorize",
      tags$br(),
      sidebarLayout(
        sidebarPanel(
          selectizeInput('bin_cov', 'Covariate', choices=num_covariates, multiple=FALSE),
          uiOutput("nbins"),
          textInput('bin_breaks', 'Custom Breaks (Comma Delimited)'),
          verbatimTextOutput("bin_to1"),
          textInput('bin_labels', 'Custom Labels (Comma Delimited)'),
          verbatimTextOutput("bin_to2"),
          textInput("new_covariate", "Covariate Label", value = "new_cov"),
          actionButton("create_bins", "Create Bins"),
          width=5
        ),
        mainPanel(
          plotlyOutput("unbin_plot", height="200px"),
          br(),
          plotlyOutput("bin_plot"),
          width=7
        )
      )
    ),    
    tabPanel("Read Counts & Relative Abundance",
      br(),
      sidebarLayout(
        sidebarPanel(
          br(),
          selectizeInput('taxlTable', 'Taxonomy Level', choices = tax.name, selected='no rank'),
          width=3
        ),
        mainPanel(
          tabsetPanel(
            tabPanel("Relative Abundance",
              br(),
              DT::dataTableOutput("TaxRAtable", width='95%'),
              downloadButton('downloadData', 'Download (.csv)')
            ),
            tabPanel("Read Counts",
              br(),
              DT::dataTableOutput("TaxCountTable",width='95%'),
              downloadButton('downloadCountData', 'Download (.csv)')
            )
            #coreOTUModuleUI("coreOTUModule")
          ), 
          width=9
        )
      )
    )
  )
)

log.cpm <- function(df){
  df.cpm <- apply(df, 2, function(y) log10(y*1e6/sum(y) + 1))
  return(df.cpm)
}

count.to.ra <- function(otu_count) {
  otu_relabu <- otu_count
  for (i in 1:ncol(otu_count))  {
    otu_relabu[,i] <- otu_relabu[,i]/sum(otu_relabu[,i])
  }
  return(otu_relabu)
}

# Trying to not use the pstat object directly
pstat.extraction <- function(p) {
  tables <- list()

  # Taxon Levels
  # unranked taxon x classifications
  tables$TAX <- as.data.frame(p@tax_table)

  # Counts
  # unranked taxon x samples
  tables$OTU <- as.data.frame(p@otu_table)

  # Relative Abundance
  # unranked taxon x samples
  tables$RLA <- as.data.frame(count.to.ra(p@otu_table))

  # Sample Data
  # samples x traits
  tables$SAM <- as.data.frame(p@sam_data)
  return(tables)
}

# Takes the relative abundances at an unranked level and
# upsamples them to higher taxonomical levels
upsample.ra <- function(ra.unranked, tax.table, newlev) {
  df = ra.unranked
  df$newlev = tax.table[[newlev]]
  df.melt = melt(df, id.vars = c("newlev"))
  df.melt.agg = aggregate(.~variable+newlev, data=df.melt, FUN=sum)
  df.ra = dcast(df.melt.agg, variable~newlev)
  rownames(df.ra) = df.ra$variable
  df.ra$variable = NULL
  return(df.ra)
}

# Used to dynamically adjust plot height
output$dynamic_ra_plot <- renderUI({
  height = paste(input$plot_sra_height, "px", sep="")
  plotlyOutput("ra_plot", width="800px", height=height)
})

# Used to dynamically generate selectable organisms based on taxlev
output$sra_order_organisms <- renderUI({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  TAX_TABLE <- tables$TAX
  choices <- unique(TAX_TABLE[[input$sra_taxlev]])
  selectizeInput('sra_order_organisms', label='Reorder Organisms', choices=choices, multiple=TRUE)
})

plot_ra <- function() {

  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat

  # Future proofing this plot
  tables <- pstat.extraction(pstat)
  TAX_TABLE <- tables$TAX
  RA_TABLE <- tables$RLA
  SAM_DATA <- tables$SAM

  # Isolate samples of interest
  if (!is.null(input$sra_isolate_samples)) {
    SAM_DATA <- SAM_DATA[rownames(SAM_DATA) %in% input$sra_isolate_samples,,drop=FALSE] 
    RA_TABLE <- RA_TABLE[,colnames(RA_TABLE) %in% input$sra_isolate_samples,drop=FALSE] 
  }

  # Sum by taxon level
  df.ra <- upsample.ra(RA_TABLE, TAX_TABLE, input$sra_taxlev)

  # If grouping is selected
  if (input$group_samples & !is.null(input$gra_select_conditions)) {
    if (input$gra_select_conditions == "All") {
      df.ra$covariate <- rep("All", nrow(df.ra))
    } else {
      df.ra$covariate <- SAM_DATA[[input$gra_select_conditions]] 
    }
    df.ra.melted <- melt(df.ra, id.vars = "covariate")
    df.avg.ra <- aggregate( . ~ variable + covariate , data = df.ra.melted, mean)
    df.avg.ra <- dcast(data = df.avg.ra,formula = covariate~variable)
    rownames(df.avg.ra) <- df.avg.ra$covariate
    df.sam <- df.avg.ra[,"covariate",drop=FALSE]
    colnames(df.sam) <- input$gra_select_conditions
    df.avg.ra$covariate <- NULL
    df.avg.ra <- df.avg.ra[,order(colSums(df.avg.ra))]
    df.ra <- df.avg.ra
  }

  # Reorder by most prominent organisms
  df.ra <- df.ra[,order(colSums(df.ra))]

  # Put selected organisms first
  if (!is.null(input$sra_order_organisms)) {
    organisms.order <- c(setdiff(colnames(df.ra), input$sra_order_organisms), rev(input$sra_order_organisms))
    df.ra <- df.ra[,organisms.order]
  }

  # Order samples by organisms if not by conditons
  if (input$sra_sort_by == "organisms") {
    for (i in 1:ncol(df.ra)) {
      df.ra <- df.ra[order(df.ra[,i]),]
    }
  }

  # If any conditions are selected make a side bar
  if (!is.null(input$sra_select_conditions) || (input$group_samples & input$gra_select_conditions != "All")) {
    
    if (!input$group_samples) {
      df.sam <- SAM_DATA[,input$sra_select_conditions,drop=FALSE]
    }

    # Order samples by conitions if not by organisms
    if (input$sra_sort_by == "conditions") {
      for (i in ncol(df.sam):1) {
        df.sam <- df.sam[order(df.sam[[i]]),,drop=FALSE]
      }
      # Reorder stacked barplot
      df.ra <- df.ra[order(match(rownames(df.ra), rownames(df.sam))),,drop=FALSE]
    } else {
      df.sam <- df.sam[order(match(rownames(df.sam), rownames(df.ra))),,drop=FALSE]
    }

    # Retain hover-text information before conditions are factorized
    hover.txt <- c()
    for (i in 1:ncol(df.sam)) {
      hover.txt <- cbind(hover.txt, df.sam[[i]])
    }

    # Plotly | Heatmap
    df.sam[] <- lapply(df.sam, factor)
    m <- data.matrix(df.sam)
    m.row.normalized <- apply(m, 2, function(x)(x-min(x))/(max(x)-min(x)))
    hm <- plot_ly(x = colnames(m), y = rownames(m), z = m.row.normalized, 
                  type = "heatmap",
                  showscale=FALSE,
                  hoverinfo = "x+y+text",
                  text=hover.txt) %>%
           layout(xaxis = list(title = "", tickangle = -45),
                  yaxis = list(showticklabels = FALSE, type = 'category', ticks = ""))
  }

  # Plotly | Stacked Bar Plots
  df.plot <- df.ra
  df.plot$samples <- rownames(df.plot)
  sbp <- plot_ly(df.plot, y = ~samples, x = df.plot[[colnames(df.plot)[1]]], 
                 type = 'bar', 
                 orientation = 'h', 
                 name = substr(colnames(df.plot)[1], 1, 40)) %>%
          layout(font = list(size = 10),
                 yaxis = list(title = '', type = 'category',
                              tickmode = "array",
                              tickvals = rownames(df.plot),
                              showticklabels = FALSE,
                              categoryorder = 'trace'),
                 xaxis = list(title = 'Relative Abundance'),
                 barmode = 'stack',
                 showlegend = input$sra_show_legend)
  for (i in 2:(ncol(df.plot)-1)) {
    sbp <- add_trace(sbp, x=df.plot[[colnames(df.plot)[i]]], name=substr(colnames(df.plot)[i], 1, 40))
  } 

  # Create a multiplot if any conditions are selected
  if (!is.null(input$sra_select_conditions) || (input$group_samples & input$gra_select_conditions != "All")) {
    hm.sbp <- subplot(hm, sbp, widths=c(0.1,  0.9))
    hm.sbp$elementId <- NULL # To suppress a shiny warning
    return(hm.sbp)
  } else {
    sbp$elementId <- NULL # To suppress a shiny warning
    return(sbp)
  }
}

# Only plots if button is pressed
do_plot_ra <- eventReactive(input$plot_sra, {
  plot_ra()
})
output$ra_plot <- renderPlotly({
  do_plot_ra()
})


# Used to dynamically adjust plot height
output$dynamic_hmra_plot <- renderUI({
  height = paste(input$plot_hmra_height, "px", sep="")
  plotlyOutput("hmra_plot", width="800px", height=height)
})

# Used to dynamically generate selectable organisms based on taxlev
output$hmra_isolate_organisms <- renderUI({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  TAX_TABLE <- tables$TAX
  choices <- unique(TAX_TABLE[[input$hmra_taxlev]])
  selectizeInput('hmra_isolate_organisms', label='Isolate Organisms', choices=choices, multiple=TRUE)
})

plot_hmra <- function() {

  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat

  # Future proofing this plot
  tables <- pstat.extraction(pstat)
  TAX_TABLE <- tables$TAX
  OTU_TABLE <- tables$OTU
  RA_TABLE <- tables$RLA
  SAM_DATA <- tables$SAM

  # Isolate samples of interest
  if (!is.null(input$hmra_isolate_samples)) {
    SAM_DATA <- SAM_DATA[rownames(SAM_DATA) %in% input$hmra_isolate_samples,,drop=FALSE] 
    OTU_TABLE <- OTU_TABLE[,colnames(OTU_TABLE) %in% input$hmra_isolate_samples,drop=FALSE] 
    RA_TABLE <- RA_TABLE[,colnames(RA_TABLE) %in% input$hmra_isolate_samples,drop=FALSE] 
  }

  if (input$hmra_logcpm) {
    df.ra = OTU_TABLE
  } else {
    df.ra = RA_TABLE
  }

  # Sum by taxon level
  df.ra <- upsample.ra(df.ra, TAX_TABLE, input$hmra_taxlev)

  if (input$hmra_logcpm) {
    df.ra = log.cpm(df.ra)
  }

  if (!is.null(input$hmra_isolate_organisms)) {
    df.ra <- df.ra[,input$hmra_isolate_organisms,drop=FALSE]
  }

  # Reorder by most prominent organisms
  df.ra <- df.ra[,order(colSums(df.ra)),drop=FALSE]

  # Order samples by organisms if not by conditons
  if (input$hmra_sort_by == "organisms") {
    for (i in 1:ncol(df.ra)) {
      df.ra <- df.ra[order(df.ra[,i]),,drop=FALSE]
    }
  }

  if (!is.null(input$hmra_select_conditions)) {
    df.sam <- SAM_DATA[,input$hmra_select_conditions,drop=FALSE]
    if (input$hmra_sort_by == "conditions") {
      for (i in ncol(df.sam):1) {
        df.sam <- df.sam[order(df.sam[[i]]),,drop=FALSE]
      }
      # Reorder stacked barplot
      df.ra <- df.ra[order(match(rownames(df.ra), rownames(df.sam))),,drop=FALSE]
    } else {
      df.sam <- df.sam[order(match(rownames(df.sam), rownames(df.ra))),,drop=FALSE]
    }
  }

  m <- data.matrix(df.ra)
  hover.txt <- c()
  for (i in 1:ncol(df.ra)) {
    hover.txt <- cbind(hover.txt, df.ra[[i]])
  }
  hm.ra <- plot_ly(x = colnames(m), y = rownames(m), z = m,
                type = "heatmap",
                colors= "RdPu",
                hoverinfo = "x+y+z+text",
                text=hover.txt) %>%
    layout(xaxis = list(showticklabels = FALSE, title = "", ticks = "", tickangle = -45),
           yaxis = list(showticklabels = FALSE, type = 'category', ticks = ""))


  if (!is.null(input$hmra_select_conditions)) {
    hover.txt <- c()
    for (i in 1:ncol(df.sam)) {
      hover.txt <- cbind(hover.txt, df.sam[[i]])
    }
    df.sam[] <- lapply(df.sam, factor)
    m <- data.matrix(df.sam)
    m.row.normalized <- apply(m, 2, function(x)(x-min(x))/(max(x)-min(x)))
    hm.sam <- plot_ly(x = colnames(m), y = rownames(m), z = m.row.normalized, 
                  type = "heatmap",
                  showscale=FALSE,
                  hoverinfo = "x+y+text",
                  text=hover.txt) %>%
      layout(xaxis = list(title = "", tickangle = -45),
             yaxis = list(showticklabels = FALSE, type = 'category', ticks = ""))
  }

  # Create a multiplot if any conditions are selected
  if (!is.null(input$hmra_select_conditions)) {
    hm.sam.ra <- subplot(hm.sam, hm.ra, widths=c(0.1,  0.9))
    hm.sam.ra$elementId <- NULL # To suppress a shiny warning
    return(hm.sam.ra)
  } else {
    hm.ra$elementId <- NULL # To suppress a shiny warning
    return(hm.ra)
  }
}

# Only plots if button is pressed
do_plot_hmra <- eventReactive(input$plot_hmra, {
  plot_hmra()
})
output$hmra_plot <- renderPlotly({
  do_plot_hmra()
})

###### Boxplots Moved Here

output$single_species_ui <- renderUI({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  species.name.vec <- TranslateIdToTaxLevel(pstat, rownames(pstat@otu_table@.Data), input$taxl_single_species)
  tagList(
    selectInput("select_single_species_name_plot", 
      "Select names (support multiple)", 
      species.name.vec, selected = species.name.vec[1], multiple = TRUE)
  )
})

plotSingleSpeciesBoxplotServer <- eventReactive(input$boxplotButton,{
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat

  if (input$taxl_single_species !="no rank")  {
    pstat <- tax_glom(pstat, input$taxl_single_species)
  }

  if (length(input$select_single_species_name_plot) == 1){
    condition.vec <- pstat@sam_data@.Data[[which(pstat@sam_data@names == input$select_single_species_condition)]]
    # change microbe names to selected taxon level
    species.name.vec <- TranslateIdToTaxLevel(pstat, rownames(pstat@otu_table@.Data), input$taxl_single_species)
    #cat(paste("condition:", length(condition.vec)))
    read.num.condition.1 <- rep(0, length(condition.vec))

    if (input$ssv_format == "read count"){
      for (i in 1:nrow(pstat@otu_table@.Data)){
        if (species.name.vec[i] == input$select_single_species_name_plot){
          read.num.condition.1 <- read.num.condition.1 + pstat@otu_table@.Data[i,]
        }
      }
      df.tmp <- data.frame(condition = condition.vec, read_Number = read.num.condition.1)
      p <- plot_ly(df.tmp, y = ~read_Number, color = ~condition, type = "box")
    }else if(input$ssv_format == "log10 CPM"){
      df.cpm <- getLogCPM(pstat@otu_table@.Data)
      for (i in 1:nrow(df.cpm)){
        if (species.name.vec[i] == input$select_single_species_name_plot){
          read.num.condition.1 <- read.num.condition.1 + df.cpm[i,]
        }
      }
      df.tmp <- data.frame(condition = condition.vec, logCPM = read.num.condition.1)
      p <- plot_ly(df.tmp, y = ~logCPM, color = ~condition, type = "box")
    }else{
      df.ra <- getRelativeAbundance(pstat@otu_table@.Data)
      for (i in 1:nrow(df.ra)){
        if (species.name.vec[i] == input$select_single_species_name_plot){
          read.num.condition.1 <- read.num.condition.1 + df.ra[i,]
        }
      }
      df.tmp <- data.frame(condition = condition.vec, RA = read.num.condition.1)
      p <- plot_ly(df.tmp, y = ~RA, color = ~condition, type = "box")
    }

    ## plot
    p

  } else{
    condition.vec <- pstat@sam_data@.Data[[which(pstat@sam_data@names == input$select_single_species_condition)]]
    condition.vec.all <- c()
    species.vec <- c()
    read.num.condition.all <- c()
    species.name.vec <- TranslateIdToTaxLevel(pstat, rownames(pstat@otu_table@.Data), input$taxl_single_species)
    for (i in 1:length(input$select_single_species_name_plot)){
      condition.vec.all <- c(condition.vec.all, condition.vec)
      species.vec <- c(species.vec, rep(input$select_single_species_name_plot[i], length(condition.vec)))
      read.num.condition.tmp <- rep(0, length(condition.vec))
      for (j in 1:nrow(pstat@otu_table@.Data)){
        if (species.name.vec[j] == input$select_single_species_name_plot[i]){
          if (input$ssv_format == "read count"){
            read.num.condition.tmp <- read.num.condition.tmp + pstat@otu_table@.Data[j,]
          }else if(input$ssv_format == "log10 CPM"){
            df.cpm <- getLogCPM(pstat@otu_table@.Data)
            read.num.condition.tmp <- read.num.condition.tmp + df.cpm[j,]
          }else{
            df.ra <- getRelativeAbundance(pstat@otu_table@.Data)
            read.num.condition.tmp <- read.num.condition.tmp + df.ra[j,]
          }
        }
      }
      read.num.condition.all <- c(read.num.condition.all, read.num.condition.tmp)
    }

    if (input$ssv_format == "read count"){
    df.tmp <- data.frame(condition = condition.vec.all, read_Number = read.num.condition.all, name = species.vec)
    p <- plot_ly(df.tmp, x = ~name, y = ~read_Number, color = ~condition, type = "box") %>%
      layout(boxmode = "group")
    suppressMessages(p)
    }else if(input$ssv_format == "log10 CPM"){
      df.tmp <- data.frame(condition = condition.vec.all, logCPM = read.num.condition.all, name = species.vec)
      p <- plot_ly(df.tmp, x = ~name, y = ~logCPM, color = ~condition, type = "box") %>%
        layout(boxmode = "group")
      suppressMessages(p)
    }else{
      df.tmp <- data.frame(condition = condition.vec.all, RA = read.num.condition.all, name = species.vec)
      p <- plot_ly(df.tmp, x = ~name, y = ~RA, color = ~condition, type = "box") %>%
        layout(boxmode = "group")
      suppressMessages(p)
    }
  }
})

output$single_species_boxplot <- renderPlotly({
  # suppress warnings
  storeWarn<- getOption("warn")
  options(warn = -1)
  plott <- plotSingleSpeciesBoxplotServer()
  #restore warnings, delayed so plot is completed
  shinyjs::delay(expr =({
    options(warn = storeWarn)
  }) ,ms = 100)

  plott
})

tabPanel("Dimension Reduction",
  sidebarLayout(
    sidebarPanel(
      numericInput('xcol.new', 'Principal Component (x-axis)', 1, min = 1, max = 50),
      numericInput('ycol.new', 'Principal Component (y-axis)', 2, min = 1, max = 50),
      selectizeInput('taxl.pca', 'Taxonomy Level', choices = tax.name, selected='no rank'),
      selectInput("select_pca_color", "Color points by:", covariates),
      selectInput("select_pca_shape", "Shape points by:", c("None", covariates.colorbar)),
      actionButton("DR_plot", "Plot"),
      width=3
    ),
    mainPanel(
      tabsetPanel(
        tabPanel("PCA",
          fluidRow(
            column(9,
              plotlyOutput("pca.plotly", height="500px"),
              selectInput("select_pca_data_format", "Select data type", c("read count", "log10 CPM", "RA"))
            ),
            column(3,
              DT::dataTableOutput("PCAtable")
            )
          )
        ),
        tabPanel("PCoA",
          fluidRow(
            column(9,
              plotlyOutput("pcoa.plotly", height="500px"),
              selectInput("pcoa.method", "PCoA method:", beta.methods)
            ),
            column(3,
              DT::dataTableOutput("PCoAtable")
            )
          )
        )
      )
    )
  )
)

sam.name <- rownames(as.data.frame(pstat@sam_data))

tabPanel("Visualization",
  tabsetPanel(
    tabPanel("Stacked Barplots",
      tags$br(),
      sidebarLayout(
        sidebarPanel(
          # Sort the samples by a condition
          conditionalPanel(
            condition = "input.group_samples == false",
            selectizeInput('sra_select_conditions', 'Color Samples by Condition', choices=covariates, multiple=TRUE)
          ),
          conditionalPanel(
            condition = "input.group_samples == true",
            selectizeInput('gra_select_conditions', 'Color Samples by Condition', choices=c("All", covariates))
          ),

          # Sample aggregation
          checkboxInput("group_samples", "Group Samples by Condition"),

          # Select taxon level
          selectInput("sra_taxlev", "Tax Level", choices=tax.name, selected="family"),

          # Dynamically generate based on tax level
          uiOutput("sra_order_organisms"),

          # Sort the bars
          radioButtons("sra_sort_by", "Sort By",
          c("No Sorting" = "nosort", "Conditions" = "conditions", "Organisms" = "organisms"), selected="nosort"),

          # Isolate Samples
          selectizeInput('sra_isolate_samples', 'Isolate Samples', choices=sam.name, multiple=TRUE),

          # Legend toggle
          checkboxInput("sra_show_legend", "Show Legend", value=TRUE),

          sliderInput("plot_sra_height", "Plot Height", 600, 1000, value=600, step=50, post="px"),

          actionButton("plot_sra", "Plot"),
          width=3
        ),
        mainPanel(
          uiOutput("dynamic_ra_plot"),
          width=9        
        )
      )
    ),
    tabPanel("Heatmap",
      tags$br(),
      sidebarLayout(
        sidebarPanel(
          # Sort the samples by a condition
          selectizeInput('hmra_select_conditions', 'Color Samples by Condition', choices=covariates, multiple=TRUE),

          # Select taxon level
          selectInput("hmra_taxlev", "Tax Level", choices=tax.name, selected="family"),

          # Dynamically generate based on tax level
          uiOutput("hmra_isolate_organisms"),

          # Sort the bars
          radioButtons("hmra_sort_by", "Sort By",
          c("No Sorting" = "nosort", "Conditions" = "conditions", "Organisms" = "organisms"), selected="nosort"),

          # Isolate Samples
          selectizeInput('hmra_isolate_samples', 'Isolate Samples', choices=sam.name, multiple=TRUE),

          # Legend toggle
          checkboxInput("hmra_logcpm", "log(CPM)", value=FALSE),

          sliderInput("plot_hmra_height", "Plot Height", 600, 1000, value=600, step=50, post="px"),

          actionButton("plot_hmra", "Plot"),
          width=3
        ),
        mainPanel(
          uiOutput("dynamic_hmra_plot"),
          width=9        
        )
      )
    ),
    tabPanel("Boxplots",
      tags$br(),
      sidebarLayout(
        sidebarPanel(
          br(),
          selectizeInput('taxl_single_species', 'Taxonomy Level', choices = tax.name, selected='no rank'),
          selectInput("select_single_species_condition", "Select condition", covariates.colorbar),
          selectInput("ssv_format", "Select data format", c("read count", "relative abundance", "log10 CPM")),
          uiOutput("single_species_ui"),
          actionButton("boxplotButton", "Plot"),
          width=3
        ),
        mainPanel(
          plotlyOutput("single_species_boxplot", width="800px"),
          width=9        
        )
      )
    )
  )
)

tabPanel("Differential Analysis",
    tabsetPanel(
         tabPanel("Differential Abundance",
                  selectInput("DAmethod", "Select method",
                    c("DESeq2", "edgeR")),
                  conditionalPanel(condition = "input.DAmethod == 'DESeq2'",
                                   sidebarLayout(
                                     sidebarPanel(
                                       selectizeInput('taxl.da', 'Taxonomy Level', choices = tax.name,
                                                      selected='no rank'),
                                       selectizeInput('da.condition', 'Select condition',
                                                      choices = covariates),
                                       conditionalPanel(condition = "output.da_condition_type == 'multiple'",
                                                        helpText("Please select 2 levels to compare"),
                                                        uiOutput("da_condition_options")
                                       ),
                                       selectizeInput('da.condition.covariate', 'Select (multiple) covariates',
                                                      choices = covariates, multiple = TRUE),
                                       helpText("Continuous covariates would be automatically cut into factors with 3 levels."),
                                       numericInput('da.count.cutoff', 'Minumum count cut-off', 500,
                                                    min = 1, max = 5000),
                                       numericInput('da.padj.cutoff', 'Choose padj cut-off', 0.5,
                                                    min = 1e-100, max = 1),
                                       actionButton("run_deseq2", "Run"),
                                       width=3
                                     ),
                                     mainPanel(
                                       tabPanel("DeSeq2",
                                         DT::dataTableOutput("DeSeq2Table.new")
                                       ), width=9
                                     )
                                   )
                                   ),
                  conditionalPanel(condition = "input.DAmethod == 'edgeR'",
                                   sidebarLayout(
                                     sidebarPanel(
                                       selectizeInput('taxl.edger', 'Taxonomy Level', choices = tax.name,
                                                      selected='no rank'),
                                       selectizeInput('edger.condition', 'Select condition',
                                                      choices = covariates.colorbar),
                                       conditionalPanel(condition = "output.edger_condition_type == 'multiple'",
                                                        helpText("Please select 2 levels to compare"),
                                                        uiOutput("edger_condition_options")
                                       ),
                                       helpText("Continuous covariates would be automatically cut into factors with 3 levels."),
                                       numericInput('edger.padj.cutoff', 'Choose padj cut-off', 0.5,
                                                    min = 1e-100, max = 1),
                                       actionButton("run_edgeR", "Run"),
                                       width=3
                                     ),
                                     mainPanel(
                                       tabPanel("edgeR",
                                         DT::dataTableOutput("edgerTable.new"),
                                         downloadButton("download_edger_tb", "Download this table")
                                       ), width=9
                                     )
                                   )
                  )
         ),

         tabPanel("Statistical Test",
         sidebarLayout(
           sidebarPanel(
             selectizeInput('taxl.pa', 'Taxonomy Level', choices = tax.name,
                            selected='no rank'),
             selectizeInput('pa.condition', 'Select condition',
                            choices = covariates.colorbar),
             conditionalPanel(condition = "output.pa_condition_type == 'multiple'",
                              helpText("Please select 2 levels to compare"),
                              uiOutput("pa_condition_options")
             ),
             numericInput('pa.count.cutoff', 'Minumum count cut-off', 500,
                          min = 1, max = 5000),
             numericInput('pa.padj.cutoff', 'Choose padj cut-off', 0.5,
                          min = 1e-100, max = 1),
             actionButton("run_stat", "Run"),
             width=3
           ),
           mainPanel(
             tabPanel("Test output",
                      br(),
                                 selectInput('pa_method', 'Select test method',
                                                choices = c("Fisher Exact Test", "Chi-squared Test", "Mann-Whitney Test")),
                                 conditionalPanel(condition = "input.pa_method == 'Mann-Whitney Test'",
                                                  selectInput('pa_mann_data_type', 'Select data type', choices = c("read count", "log10 CPM", "RA"))),
                                 DT::dataTableOutput("pa.test"),
                                 downloadButton("download_pa_test", "Download this table")
             ), width=9
           )
         )
    )
    )


)

source(file.path("utils", "old_server_stuff.R"),  local = TRUE)

observeEvent(input$uploadPathoStat,{
  withBusyIndicatorServer("uploadPathoStat", {
    if (input$rdtype == 'rda') {
      load(input$rdfile$datapath)
    }
    if (input$rdtype == 'rds') {
      pstat <- readRDS(input$rdfile$datapath)
    }
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput
    vals$shiny.input.backup <- shinyInput
    # Update ui
    updateCovariate()
    updateSample()
  })
})

observeEvent(input$uploadDataCount,{
  withBusyIndicatorServer("uploadDataCount", {

  df.input <- read.csv(input$countsfile$datapath,
                       header = input$header.count,
                       row.names = 1,
                       stringsAsFactors = FALSE,
                       sep = input$sep.count,
                       comment.char="",
                       check.names = FALSE)

  df.taxon.input <- read.csv(input$taxon.table$datapath,
                            header = input$header.count,
                            sep = input$sep.count,
                            row.names= 1,
                            stringsAsFactors=FALSE,
                            comment.char="",
                            check.names = FALSE)

  df.meta.input <- read.csv(input$annotfile.count$datapath,
                            header = input$header.count,
                            sep = input$sep.count,
                            row.names=input$metadata_sample_name_col_count,
                            stringsAsFactors=FALSE,
                            comment.char="",
                            check.names = FALSE)

  # Choose only the samples in metadata that have counts data as well
  df.meta.input <- df.meta.input[match(colnames(df.input), rownames(df.meta.input)), ]

  # Test and fix the constant/zero row
  row.remove.index <- c()
  if (sum(rowSums(as.matrix(df.input)) == 0) > 0){
      row.remove.index <- which(rowSums(as.matrix(df.input)) == 0)
      df.input <- df.input[-row.remove.index,]
  }

  OTU <- otu_table(df.input, taxa_are_rows = TRUE)
  TAX <- tax_table(as.matrix(df.taxon.input))
  physeq <- phyloseq(OTU, TAX)

  # Change NA/NULL to 0
  # Remove variables with identical values
  col.remove.index <- c()
  for (i in 1:ncol(df.meta.input)){
      if(length(unique(df.meta.input[,i])) < 2){
          col.remove.index <- c(col.remove.index, i)
      }
  }
  if (!is.null(col.remove.index)){
      df.meta.input <- df.meta.input[,-col.remove.index]
  }

  sampledata = sample_data(data.frame(df.meta.input))
  random_tree = rtree(ntaxa(physeq), rooted=TRUE, tip.label=
                          taxa_names(physeq))
  physeq1 <- merge_phyloseq(physeq, sampledata, random_tree)
  pstat <- pathostat1(physeq1)
  shinyInput <- list(pstat = pstat)
  vals$shiny.input <- shinyInput
  vals$shiny.input.backup <- shinyInput
  # Update ui
  updateCovariate()
  updateSample()
  updateTaxLevel()
  })
})

observeEvent(input$uploadDataPs, {
  withBusyIndicatorServer("uploadDataPs", {

    df.path.vec <- c()
    df.name.vec <- c()
    for(i in 1:length(input$countsfile.pathoscope[,1])){
        df.path.vec[i] <- input$countsfile.pathoscope[[i, 'datapath']]
        df.name.vec[i] <- input$countsfile.pathoscope[[i, 'name']]
    }

    datlist <- readPathoscopeData(input_dir, pathoreport_file_suffix = input$report_suffix,
                                  use.input.files = TRUE,
                                  input.files.path.vec = df.path.vec,
                                  input.files.name.vec = df.name.vec)
    countdat <- datlist$countdata

    df.meta.input <- read.csv(input$annotfile.ps$datapath,
                              header = input$header.ps,
                              sep = input$sep.ps,
                              row.names=input$metadata_sample_name_col,
                              stringsAsFactors=FALSE)

    # Choose only the samples in metadata that have counts data as well
    df.meta.input <- df.meta.input[match(colnames(countdat), rownames(df.meta.input)), ]

    # Test and fix the constant/zero row
    row.remove.index <- c()
    if (sum(rowSums(as.matrix(countdat)) == 0) > 0){
        row.remove.index <- which(rowSums(as.matrix(countdat)) == 0)
        countdat <- countdat[-row.remove.index,]
    }

    ids <- rownames(countdat)
    tids <- unlist(lapply(ids, FUN = grepTid))
    taxonLevels <- findTaxonomy(tids)
    taxmat <- findTaxonMat(ids, taxonLevels)
    # Test and fix the constant/zero row
    if (!is.null(row.remove.index)){
        taxmat <- taxmat[-row.remove.index,]
    }

    OTU <- otu_table(countdat, taxa_are_rows = TRUE)
    TAX <- tax_table(taxmat)
    physeq <- phyloseq(OTU, TAX)

    # Change NA/NULL to 0
    # Remove variables with identical values
    col.remove.index <- c()
    for (i in 1:ncol(df.meta.input)){
        if(length(unique(df.meta.input[,i])) < 2){
            col.remove.index <- c(col.remove.index, i)
        }
    }
    if (!is.null(col.remove.index)){
        df.meta.input <- df.meta.input[,-col.remove.index]
    }

    sampledata = sample_data(data.frame(df.meta.input))
    random_tree = rtree(ntaxa(physeq), rooted=TRUE, tip.label=
                            taxa_names(physeq))
    physeq1 <- merge_phyloseq(physeq, sampledata, random_tree)
    pstat <- pathostat1(physeq1)
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput
    vals$shiny.input.backup <- shinyInput
    # Update ui
    updateCovariate()
    updateSample()
    updateTaxLevel()
  })
})

  ## New DA analysis section

  run.edger <- eventReactive(input$run_edgeR, {
         shinyInput <- vals$shiny.input
      pstat <- shinyInput$pstat
      if (input$taxl.edger !="no rank"){
          pstat <- tax_glom(pstat, input$taxl.edger)
      }



      # number of samples in each level of target variable
      target.var.index <- which(pstat@sam_data@names == input$edger.condition)
      label.vec.num <- pstat@sam_data@.Data[[target.var.index]]



      # if selected condition has multiple levels
      if (length(input$edger_condition_options_use) == 2){
        sample.keep.index <- which(label.vec.num %in% input$edger_condition_options_use)
        label.vec.num <- label.vec.num[sample.keep.index]
        pstat@otu_table@.Data <- pstat@otu_table@.Data[,sample.keep.index]
        pstat@sam_data@row.names <- pstat@sam_data@row.names[sample.keep.index]
        for (i in 1:length(pstat@sam_data@names)){
          pstat@sam_data@.Data[[i]] <- pstat@sam_data@.Data[[i]][sample.keep.index]
        }
      }


      label.vec.save <- unique(label.vec.num)

      # transform label into 1 and 0
      label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
      label.vec.num[label.vec.num != 1] <- 0
      label.vec.num <- as.numeric(label.vec.num)


      dge = phyloseq_to_edgeR(pstat, group=input$edger.condition)
      # Perform binary test
      et = exactTest(dge)
      # Extract values from test results
      tt = topTags(et, n=nrow(dge$table), adjust.method="BH", sort.by="PValue")
      res = tt@.Data[[1]]
      sigtab = res[(res$FDR < input$edger.padj.cutoff), ]
      sigtab = cbind(as(sigtab, "data.frame"), as(tax_table(pstat)[rownames(sigtab), ], "matrix"))

          if (nrow(sigtab) == 0){
              return(as.matrix("No differentially abundant items found!"))
          } else{
              sigtab = cbind(as(sigtab, "data.frame"), as(tax_table(pstat)[rownames(sigtab), ], "matrix"))
              sigtab$FDR <- as.numeric(formatC(sigtab$FDR, format = "e", digits = 2))
              sigtab$logFC <- as.numeric(formatC(sigtab$logFC, format = "e", digits = 2))


              sigtab <- sigtab[,c(which(colnames(sigtab) == input$taxl.edger)[1],
                                  which(colnames(sigtab) == "FDR"),
                                  which(colnames(sigtab) == "logFC"))]
              rownames(sigtab) <- 1:nrow(sigtab)



              # remove "others"
              index.sigtab.save <- c()
              for (i in 1:nrow(sigtab)){
                if (sigtab[i,1] != "others"){
                  index.sigtab.save <- c(index.sigtab.save, i)
                }
              }
              sigtab <- sigtab[index.sigtab.save,]

              num.1 <- c()
              num.2 <- c()
              species.names.tmp <- TranslateIdToTaxLevel(pstat, rownames(pstat@otu_table@.Data), input$taxl.edger)
              for (i in 1:nrow(sigtab)){
                species.index <- which(species.names.tmp == sigtab[i,1])
                num.1 <- c(num.1, sum((pstat@otu_table@.Data[species.index,which(label.vec.num == 1)] > 0)))
                num.2 <- c(num.2, sum((pstat@otu_table@.Data[species.index,which(label.vec.num == 0)] > 0)))
              }

              sigtab <- cbind(sigtab, num.1)
              sigtab <- cbind(sigtab, num.2)


              df.output.prevalence <- percent(round((num.1 + num.2)/ncol(pstat@otu_table@.Data),4))
              sigtab <- cbind(sigtab, df.output.prevalence)


              colnames(sigtab)[ncol(sigtab)-2] <- label.vec.save[1]
              colnames(sigtab)[ncol(sigtab)-1] <- label.vec.save[2]
              colnames(sigtab)[ncol(sigtab)] <- "prevalence"


              output$download_edger_tb <- downloadHandler(
                  filename = function() { paste('download_edger_table', '.csv', sep='') },
                  content = function(file) {
                      dist.mat <- as.matrix(sigtab)
                      write.csv(data.frame(dist.mat), file)
                  }
              )
              foldChange <- c()
              for (i in 1:nrow(sigtab)){
                foldChange[i] <- round((max(as.numeric(c((sigtab[i,5] / sum(label.vec.num == 0)),
                                                         (sigtab[i,4] / sum(label.vec.num == 1))))) /
                                          min(as.numeric(c((sigtab[i,5] / sum(label.vec.num == 0)),
                                                           (sigtab[i,4] / sum(label.vec.num == 1)))))), 
                                       digits = 2)
              }
              sigtab <- cbind(sigtab, foldChange)
              colnames(sigtab)[ncol(sigtab)] <- "Group Size adjusted fold change"
              return(sigtab)

          }

  })
  output$edgerTable.new <- DT::renderDataTable({
    run.edger()
  },
  options = list(
      paging = TRUE
  ))




  run.deseq2 <- eventReactive(input$run_deseq2, {
         shinyInput <- vals$shiny.input
    physeq1 <- shinyInput$pstat
    if (input$taxl.da !="no rank"){
      physeq1 <- tax_glom(physeq1, input$taxl.da)
    }
    physeq1 <- prune_samples(sample_sums(physeq1) > input$da.count.cutoff, physeq1)




    # number of samples in each level of target variable
    target.var.index <- which(physeq1@sam_data@names == input$da.condition)
    label.vec.num <- physeq1@sam_data@.Data[[target.var.index]]


    # if selected condition has multiple levels
    if (length(input$da_condition_options_use) == 2){
      sample.keep.index <- which(label.vec.num %in% input$da_condition_options_use)
      label.vec.num <- label.vec.num[sample.keep.index]
      physeq1@otu_table@.Data <- physeq1@otu_table@.Data[,sample.keep.index]
      physeq1@sam_data@row.names <- physeq1@sam_data@row.names[sample.keep.index]
      for (i in 1:length(physeq1@sam_data@names)){
        physeq1@sam_data@.Data[[i]] <- physeq1@sam_data@.Data[[i]][sample.keep.index]
      }
    }




    label.vec.save <- unique(label.vec.num)

    # transform label into 1 and 0
    label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
    label.vec.num[label.vec.num != 1] <- 0
    label.vec.num <- as.numeric(label.vec.num)


    # deal with continuous covariates and multiple covariates
    # target condition is the last one in formula.
    if (!is.null(input$da.condition.covariate)){
      for (i in 1:length(input$da.condition.covariate)){
        num.levels <- length(unique(sample_data(physeq1)[[input$da.condition.covariate[i]]]))
        if (num.levels >= 8){
          sam.index <- which(physeq1@sam_data@names %in% input$da.condition.covariate[i])
          physeq1@sam_data@.Data[[sam.index]] <- cut(physeq1@sam_data@.Data[[sam.index]], breaks = 3)
        }
      }

      diagdds = phyloseq_to_deseq2(physeq1,
                                   as.formula(paste("~",
                                                    paste(
                                                      paste(input$da.condition.covariate,
                                                            collapse = " + "),
                                                      input$da.condition,
                                                      sep = " + "),
                                                    sep = " ")))
    } else{
      diagdds = phyloseq_to_deseq2(physeq1, as.formula(paste("~",input$da.condition, sep = " ")))
    }

    # calculate geometric means prior to estimate size factors
    gm_mean = function(x, na.rm=TRUE){
      exp(sum(log(x[x > 0]), na.rm=na.rm) / length(x))
    }
    geoMeans = apply(counts(diagdds), 1, gm_mean)
    diagdds = estimateSizeFactors(diagdds, geoMeans = geoMeans)
    diagdds = DESeq(diagdds, fitType="local")
    res = results(diagdds)
    res = res[order(res$padj, na.last=NA), ]

    if (nrow(res) != 0){
      sigtab = res[(res$padj < input$da.padj.cutoff), ]
      if (nrow(sigtab) == 0){
        return(as.matrix("No differentially abundant items found!"))
      } else{
        sigtab = cbind(as(sigtab, "data.frame"), as(tax_table(physeq1)[rownames(sigtab), ], "matrix"))
        sigtab$padj <- as.numeric(formatC(sigtab$padj, format = "e", digits = 2))
        sigtab$log2FoldChange <- as.numeric(formatC(sigtab$log2FoldChange, format = "e", digits = 2))

        rownames(sigtab) <- 1:nrow(sigtab)






        sigtab <- sigtab[,c(which(colnames(sigtab) == input$taxl.da[1]),
                            which(colnames(sigtab) == "padj"),
                            which(colnames(sigtab) == "log2FoldChange"))]





        # remove "others"
        index.sigtab.save <- c()
        for (i in 1:nrow(sigtab)){
          if (sigtab[i,1] != "others"){
            index.sigtab.save <- c(index.sigtab.save, i)
          }
        }
        sigtab <- sigtab[index.sigtab.save,]

        num.1 <- c()
        num.2 <- c()
        species.names.tmp <- TranslateIdToTaxLevel(physeq1, rownames(physeq1@otu_table@.Data), input$taxl.da)
        for (i in 1:nrow(sigtab)){
          species.index <- which(species.names.tmp == sigtab[i,1])
          num.1 <- c(num.1, sum((physeq1@otu_table@.Data[species.index,which(label.vec.num == 1)] > 0)))
          num.2 <- c(num.2, sum((physeq1@otu_table@.Data[species.index,which(label.vec.num == 0)] > 0)))
        }

        sigtab <- cbind(sigtab, num.1)
        sigtab <- cbind(sigtab, num.2)


        df.output.prevalence <- percent(round((num.1 + num.2)/ncol(physeq1@otu_table@.Data),4))
        sigtab <- cbind(sigtab, df.output.prevalence)


        colnames(sigtab)[ncol(sigtab)-2] <- label.vec.save[1]
        colnames(sigtab)[ncol(sigtab)-1] <- label.vec.save[2]
        colnames(sigtab)[ncol(sigtab)] <- "prevalence"

        output$download_deseq_tb <- downloadHandler(
          filename = function() { paste('download_deseq2_table', '.csv', sep='') },
          content = function(file) {
            dist.mat <- as.matrix(sigtab)
            write.csv(data.frame(dist.mat), file)
          }
        )
        foldChange <- c()
        for (i in 1:nrow(sigtab)){
        foldChange[i] <- round((max(as.numeric(c((sigtab[i,5] / sum(label.vec.num == 0)),
                                                 (sigtab[i,4] / sum(label.vec.num == 1))))) /
                       min(as.numeric(c((sigtab[i,5] / sum(label.vec.num == 0)),
                                        (sigtab[i,4] / sum(label.vec.num == 1)))))), digits = 2)
        }
        sigtab <- cbind(sigtab, foldChange)
        colnames(sigtab)[ncol(sigtab)] <- "Group Size adjusted fold change"
        return(sigtab)

      }

    }else{
      return(as.matrix("No differentially abundant items found!"))
    }

  })


  output$DeSeq2Table.new <- DT::renderDataTable({
    run.deseq2()
  },
  options = list(
      paging = TRUE
  ))



  ### check whether selected condition for DA has two levels or more
  output$da_condition_type <- reactive({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    target.var.index <- which(pstat@sam_data@names == input$da.condition)
    if (is.integer0(target.var.index)){
      target.var.index <- 1
    }
    label.vec <- pstat@sam_data@.Data[[target.var.index]]
    label.level.num <- length(unique(label.vec))
    factor.non.categorical <- covariates[which(!covariates %in% covariates.colorbar)]
    if (label.level.num == 2 || input$da.condition %in% factor.non.categorical){
      return("binary")
    } else{
      return("multiple")
    }

  })
  outputOptions(output, "da_condition_type", suspendWhenHidden = FALSE)

  # select 2 levels
  output$da_condition_options <- renderUI({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    variable.vec <- sample_data(pstat)[[
      which(pstat@sam_data@names == input$da.condition)]]
    filter.option.vec <- sort(unique(variable.vec))
    tagList(
      selectInput("da_condition_options_use", "Select 2 levels", choices = filter.option.vec, multiple = TRUE)
    )
  })


  ### check whether selected condition for edger has two levels or more
  output$edger_condition_type <- reactive({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    target.var.index <- which(pstat@sam_data@names == input$edger.condition)
    if (is.integer0(target.var.index)){
      target.var.index <- 1
    }    
    label.vec <- pstat@sam_data@.Data[[target.var.index]]
    label.level.num <- length(unique(label.vec))
    if (label.level.num == 2){
      return("binary")
    } else{
      return("multiple")
    }

  })
  outputOptions(output, "edger_condition_type", suspendWhenHidden = FALSE)

  # select 2 levels
  output$edger_condition_options <- renderUI({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    variable.vec <- sample_data(pstat)[[
      which(pstat@sam_data@names == input$edger.condition)]]
    filter.option.vec <- sort(unique(variable.vec))
    tagList(
      selectInput("edger_condition_options_use", "Select 2 levels", choices = filter.option.vec, multiple = TRUE)
    )
  })







  # Presence-Absence Variance analysis

  run.pav <- eventReactive(input$run_stat, {
          shinyInput <- vals$shiny.input
    physeq1 <- shinyInput$pstat
    if (input$taxl.pa !="no rank"){
      physeq1 <- tax_glom(physeq1, input$taxl.pa)
    }
    physeq1 <- prune_samples(sample_sums(physeq1) > input$pa.count.cutoff, physeq1)


    target.var.index <- which(physeq1@sam_data@names == input$pa.condition)
    label.vec.num <- physeq1@sam_data@.Data[[target.var.index]]
    # if selected condition has multiple levels
    if (length(input$pa_condition_options_use) == 2){
      sample.keep.index <- which(label.vec.num %in% input$pa_condition_options_use)
      label.vec.num <- label.vec.num[sample.keep.index]
      physeq1@otu_table@.Data <- physeq1@otu_table@.Data[,sample.keep.index]
      physeq1@sam_data@row.names <- physeq1@sam_data@row.names[sample.keep.index]
      for (i in 1:length(physeq1@sam_data@names)){
        physeq1@sam_data@.Data[[i]] <- physeq1@sam_data@.Data[[i]][sample.keep.index]
      }
    }


    df.pam <- GET_PAM(physeq1@otu_table@.Data)


    # change microbe names to selected taxon level
    rownames(df.pam) <- TranslateIdToTaxLevel(physeq1, rownames(df.pam), input$taxl.pa)

    if (input$pa_method == "Fisher Exact Test"){
      output.mat <- Fisher_Test_Pam(df.pam,
                                    physeq1@sam_data[[input$pa.condition]],
                                    input$pa.padj.cutoff)
      output$download_pa_test <- downloadHandler(
        filename = function() { paste(input$pa_method, '.csv', sep='') },
        content = function(file) {
          write.csv(output.mat, file)
        }
      )

      DT::datatable(output.mat)
    } else if(input$pa_method == "Chi-squared Test"){
      output.mat <- Chisq_Test_Pam(df.pam,
                                   physeq1@sam_data[[input$pa.condition]],
                                   input$pa.padj.cutoff)
      output$download_pa_test <- downloadHandler(
        filename = function() { paste(input$pa_method, '.csv', sep='') },
        content = function(file) {
          write.csv(output.mat, file)
        }
      )
      DT::datatable(output.mat, rownames = TRUE)
    } else if (input$pa_method == "Mann-Whitney Test"){
      df.test <- physeq1@otu_table@.Data
      rownames(df.test) <- TranslateIdToTaxLevel(physeq1, rownames(df.test), input$taxl.pa)

      if (input$pa_mann_data_type == "log10 CPM"){
        df.test <- getLogCPM(df.test)
      }else if (input$pa_mann_data_type == "RA"){
        df.test <- getRelativeAbundance(df.test)
      }


      output.mat <- Wilcox_Test_df(df.test,
                                   physeq1@sam_data[[input$pa.condition]],
                                   input$pa.padj.cutoff)
      output$download_pa_test <- downloadHandler(
        filename = function() { paste(input$pa_method, '.csv', sep='') },
        content = function(file) {
          write.csv(output.mat, file)
        }
      )
      DT::datatable(output.mat)
    }
  })
  output$pa.test <- DT::renderDataTable({
    run.pav()
  })


  ### check whether selected condition for pa has two levels or more
  output$pa_condition_type <- reactive({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    target.var.index <- which(pstat@sam_data@names == input$pa.condition)
    if (is.integer0(target.var.index)){
      target.var.index <- 1
    }    
    label.vec <- pstat@sam_data@.Data[[target.var.index]]
    label.level.num <- length(unique(label.vec))
    if (label.level.num == 2){
      return("binary")
    } else{
      return("multiple")
    }

  })
  outputOptions(output, "pa_condition_type", suspendWhenHidden = FALSE)

  # select 2 levels
  output$pa_condition_options <- renderUI({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    variable.vec <- sample_data(pstat)[[
      which(pstat@sam_data@names == input$pa.condition)]]
    filter.option.vec <- sort(unique(variable.vec))
    tagList(
      selectInput("pa_condition_options_use", "Select 2 levels", choices = filter.option.vec, multiple = TRUE)
    )
  })

### biomarker
  getBiomarker <- function(){
    if (input$select_model_biomarker == "Lasso Logistic Regression"){
      shinyInput <- vals$shiny.input
      physeq1 <- shinyInput$pstat
      if (input$taxl_biomarker !="no rank"){
        physeq1 <- tax_glom(physeq1, input$taxl_biomarker)

      }

      target.var.index <- which(physeq1@sam_data@names == input$select_target_condition_biomarker)
      label.vec.num <- physeq1@sam_data@.Data[[target.var.index]]
      # if selected condition has multiple levels
      if (length(input$biomarker_condition_options_use) == 2){
        sample.keep.index <- which(label.vec.num %in% input$biomarker_condition_options_use)
        label.vec.num <- label.vec.num[sample.keep.index]
        physeq1@otu_table@.Data <- physeq1@otu_table@.Data[,sample.keep.index]
        physeq1@sam_data@row.names <- physeq1@sam_data@row.names[sample.keep.index]
        for (i in 1:length(physeq1@sam_data@names)){
          physeq1@sam_data@.Data[[i]] <- physeq1@sam_data@.Data[[i]][sample.keep.index]
        }
      }


      # use log CPM as normalization.
      df.input <- physeq1@otu_table@.Data
      for(i in 1:ncol(df.input)){
        if (is.numeric(df.input[,i])){
          df.input[,i] <- log10(df.input[,i]*1e6/sum(df.input[,i]) + 0.1)
        }
      }

      # change microbe names to selected taxon level
      rownames(df.input) <- TranslateIdToTaxLevel(physeq1, rownames(df.input), input$taxl_biomarker)

      if (!is.null(input$select_covariate_condition_biomarker)){

        target.tmp <- physeq1@sam_data@.Data[[2]]

        covariate.vec <- input$select_covariate_condition_biomarker
        df.list.tmp <- list()
        for (i in 1:length(covariate.vec)){
          df.list.tmp[[covariate.vec[i]]] <- physeq1@sam_data@.Data[[which(physeq1@sam_data@names == covariate.vec[i])]]
        }
        df.covariate <- data.frame(df.list.tmp)
        rownames(df.covariate) <- colnames(df.input)

        df.input <- cbind.data.frame(t(df.input), df.covariate)
      } else{
        df.input <- t(df.input)
      }


      target.vec <- physeq1@sam_data[[input$select_target_condition_biomarker]]

      output.fs <- getSignatureFromMultipleGlmnet(df.input, target.vec, nfolds = input$num.cv.nfolds, nRun = input$num.biomarker.run)
    }

    # number of samples in each level of target variable
    target.var.index <- which(physeq1@sam_data@names == input$select_target_condition_biomarker)
    label.vec.num <- physeq1@sam_data@.Data[[target.var.index]]
    label.vec.save <- unique(label.vec.num)

    # transform label into 1 and 0
    label.vec <- label.vec.num
    label.vec.num[label.vec.num == unique(label.vec.num)[1]] <- 1
    label.vec.num[label.vec.num != 1] <- 0

    label.vec.num <- as.numeric(label.vec.num)
    num.1 <- c()
    num.2 <- c()
    species.names.tmp <- TranslateIdToTaxLevel(physeq1, rownames(physeq1@otu_table@.Data), input$taxl_biomarker)
    for (i in 1:length(output.fs$feature)){
      species.index <- which(species.names.tmp == output.fs$feature[i])
      num.1 <- c(num.1, sum((physeq1@otu_table@.Data[species.index,which(label.vec.num == 1)] > 0)))
      num.2 <- c(num.2, sum((physeq1@otu_table@.Data[species.index,which(label.vec.num == 0)] > 0)))
    }

    df.output <- data.frame(biomarker = output.fs$feature,
                            selection_rate = output.fs$selection_rate,
                            average_weights = output.fs$feature_weights,
                            num.1,
                            num.2,
                            percent(round((num.1+num.2)/ncol(physeq1@otu_table@.Data),4)))
    foldChange <- c()
    for (i in 1:nrow(df.output)){
      foldChange[i] <- round((max(as.numeric(c((df.output[i,5] / sum(label.vec.num == 0)),
                                               (df.output[i,4] / sum(label.vec.num == 1))))) /
                                min(as.numeric(c((df.output[i,5] / sum(label.vec.num == 0)),
                                                 (df.output[i,4] / sum(label.vec.num == 1)))))), 
                             digits = 2)
    }
    df.output <- cbind(df.output, foldChange)
    colnames(df.output)[ncol(df.output)-3] <- label.vec.save[1]
    colnames(df.output)[ncol(df.output)-2] <- label.vec.save[2]
    colnames(df.output)[ncol(df.output)-1] <- "prevalence"
    colnames(df.output)[ncol(df.output)] <- "group size adjusted fold change"
    df.biomarker <- df.input[,which(colnames(df.input) %in% output.fs$feature)]
    return(list(df.output = df.output, 
                df.input = df.biomarker, 
                target.vec = label.vec.num,
                label.vec = label.vec))


  }



  observeEvent(input$goButtonBiomarker, {

     biomarker.vals <- reactiveValues(
      biomarker.list = suppressWarnings(getBiomarker())
     )
     biomarker.vals.2 <- reactiveValues(
       loocv.output.list = LOOAUC_simple_multiple_one_df(biomarker.vals$biomarker.list$df.input,
                                                         biomarker.vals$biomarker.list$target.vec)
     )

      output$featureSelectionTmp <- renderTable({
        biomarker.vals$biomarker.list$df.output
      })

      output$loocv_output_simple <- renderPlot({

        plot(biomarker.vals.2$loocv.output.list$loo.perf.plot, main="ROC curve",col="red",lwd=3, cex=4)
        auc <- performance(biomarker.vals.2$loocv.output.list$loo.pred.plot,"auc")
        auc <- unlist(slot(auc, "y.values"))
        aucRound <- paste("AUC: ", round(auc,3))
        abline(a=0,b=1,lwd=2,lty=2,col="gray")
        legend(0.7,0.5, aucRound)
      })


      output$loocv.violin <- renderPlotly({
        df.tmp <- data.frame(class.vec = biomarker.vals.2$loocv.output.list$testPredictionProb,
                             class = biomarker.vals$biomarker.list$label.vec)
        #print(df.tmp)
        p.tmp <- df.tmp %>%
          plot_ly(
            color = ~class,
            y = ~class.vec,
            type = 'box',
            boxpoints = "all", 
            jitter = 0.3,
            pointpos = -1.8,
            colors = c("#132B43", "#56B1F7")
          ) %>%
          layout(
            xaxis = list(
              title = "Class"
            ),
            yaxis = list(
              title = "Predicted Probability",
              zeroline = F
            )
          )
        p.tmp

      })

      bootstrap.out <- eventReactive(input$goButtonBiomarkerBoot, {
        suppressWarnings(Bootstrap_LOOCV_LR_AUC(biomarker.vals$biomarker.list$df.input,
                                                biomarker.vals$biomarker.list$target.vec,
                                                nboot = input$num.bootstrap.loocv))
      })

      output$loocv_output <- renderTable({
        bootstrap.out()
      })

  })




  ### check whether selected condition for biomarker has two levels or more
  output$biomarker_condition_type <- reactive({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    target.var.index <- which(pstat@sam_data@names == input$select_target_condition_biomarker)
    if (is.integer0(target.var.index)){
      target.var.index <- 1
    }    
    label.vec <- pstat@sam_data@.Data[[target.var.index]]
    label.level.num <- length(unique(label.vec))
    if (label.level.num == 2){
      return("binary")
    } else{
      return("multiple")
    }

  })
  outputOptions(output, "biomarker_condition_type", suspendWhenHidden = FALSE)

  # select 2 levels
  output$biomarker_condition_options <- renderUI({
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    variable.vec <- sample_data(pstat)[[
      which(pstat@sam_data@names == input$select_target_condition_biomarker)]]
    filter.option.vec <- sort(unique(variable.vec))
    tagList(
      selectInput("biomarker_condition_options_use", "Select 2 levels", choices = filter.option.vec, multiple = TRUE)
    )
  })

library(shiny)

source(file.path("utils", "old_server_stuff.R"),  local = TRUE)
source(file.path("utils", "old_ui_stuff.R"),  local = TRUE)

ui <- navbarPage(
  title = paste("PathoStat v", packageVersion("PathoStat"), sep = ""),
  id="PathoStat",
  fluid=TRUE,
  theme = "bootstrap.min.css",
  source(file.path("ui", "ui_01_upload.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_02_filter.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_03_ra.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_04_diversity.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_05_dimreduction.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_06_differential.R"),  local = TRUE)$value,
  source(file.path("ui", "ui_07_biomarker.R"),  local = TRUE)$value
)

server <- function(input, output, session) {

  source(file.path("server", "server_01_upload.R"),  local = TRUE)$value
  source(file.path("server", "server_02_filter.R"),  local = TRUE)$value
  source(file.path("server", "server_03_ra.R"),  local = TRUE)$value
  source(file.path("server", "server_04_diversity.R"),  local = TRUE)$value
  source(file.path("server", "server_05_dimreduction.R"),  local = TRUE)$value
  source(file.path("server", "server_06_differential.R"),  local = TRUE)$value
  source(file.path("server", "server_07_biomarker.R"),  local = TRUE)$value
}

shinyApp(ui = ui, server = server)

source(file.path("utils", "old_server_stuff.R"),  local = TRUE)

observeEvent(input$filterSampleButton,{
  withBusyIndicatorServer("filterSampleButton", {
    samples.remove <- input$filterSample
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    samples.remove.index <- which(colnames(pstat@otu_table@.Data) %in% samples.remove)
    pstat@otu_table@.Data <- pstat@otu_table@.Data[,-samples.remove.index]
    pstat@sam_data@.Data <-  lapply(pstat@sam_data@.Data, function(x) {x <- x[-samples.remove.index]})
    pstat@sam_data@row.names <- pstat@sam_data@row.names[-samples.remove.index]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput
    #updateCovariate()
    updateSample()
  })
})

observeEvent(input$reset_button,{
  withBusyIndicatorServer("reset_button", {
    vals$shiny.input <- vals$shiny.input.backup
    updateSample()
    vals$shiny.input <- vals$shiny.input.backup
    updateTaxLevel()
  })
})

output$download_rds <- downloadHandler(filename = function() {
  paste("pathostat", Sys.Date(), ".rds", sep="")
}, content = function(file) {
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  saveRDS(pstat, file=file)
})

output$summary_plot_top <- renderPlotly({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat 
  tables <- pstat.extraction(pstat)
  OTU_TABLE <- tables$OTU
  SAM_DATA <- tables$SAM
  SAM_DATA[,"Reads"] = colSums(OTU_TABLE[,rownames(SAM_DATA)])
  SAM_DATA[,"Taxnum"] = apply(OTU_TABLE , 2, function(x) sum(x >= 1) )

  if (input$filter_type == "By Microbes") {
    cov <- "Taxnum"
  } else {
    cov <- input$select_condition_sample_filter_sidebar
  }
  
  if (length(unique(unlist(SAM_DATA[,cov]))) > 8) {
    fit <- density(unlist(SAM_DATA[,cov]))
    num.density <- plot_ly(x=fit$x, y=fit$y, 
                           type="scatter", 
                           mode="lines", 
                           fill="tozeroy") %>%
                    layout(title=cov)
    num.density$elementId <- NULL
    return(num.density)
  } else {
    cat.df = data.frame(table(SAM_DATA[,cov]))
    cat.bar <- plot_ly(x = cat.df$Var1,
                       y = cat.df$Freq,
                       type = "bar",
                       showlegend = FALSE) %>%
                layout(title=cov,
                       xaxis = list(title = 'Diet',
                                    tickmode = "array",
                                    showticklabels = TRUE,
                                    categoryorder = 'trace'),
                       yaxis = list(title = 'Frequency'))
    cat.bar$elementId <- NULL
    return(cat.bar)
  }
})

output$summary_plot_bottom <- renderPlotly({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  OTU_TABLE <- tables$OTU
  SAM_DATA <- tables$SAM
  SAM_DATA[,"Reads"] = colSums(OTU_TABLE[,rownames(SAM_DATA)])
  SAM_DATA[,"Taxnum"] = apply(OTU_TABLE , 2, function(x) sum(x >= 1) )

  if (input$filter_type == "By Microbes") {
    cov <- "Taxnum"
  } else {
    cov <- input$select_condition_sample_filter_sidebar
  }

  if (length(unique(unlist(SAM_DATA[,cov]))) > 8) {
    vec <- unlist(SAM_DATA[,cov])
    num.scatter <- plot_ly(y = vec, 
                          jitter = 0.3, 
                          pointpos = -1.8, 
                          boxpoints = 'all',
                          marker = list(color = 'rgb(7,40,89)'),
                          line = list(color = 'rgb(7,40,89)'),
                          name = cov, type="box") %>%
                   layout(title=cov)
    num.scatter$elementId <- NULL
    return(num.scatter)
  } else {
    cat.df = data.frame(table(SAM_DATA[,cov]))
    cat.pie <- plot_ly(cat.df, labels = ~Var1, values = ~Freq,
                       type = "pie",
                       showlegend = FALSE) %>%
                layout(title=cov)
    cat.pie$elementId <- NULL
    return(cat.pie)
  }
})

output$filter_type <- reactive({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  if (input$select_condition_sample_filter_sidebar == "Reads") {
    return("num.continuous")
  }
  variable.vec <- sample_data(pstat)[[
        which(pstat@sam_data@names == input$select_condition_sample_filter_sidebar)]]
  if (is.numeric(variable.vec)){
    # catogorical or not
    num.levels <- length(unique(variable.vec))
    if (num.levels < 6) {
      return("cat")
    } else {
      return("num.continuous")
    }
  } else {
    # non-numeric
    return("cat")
  }
})

outputOptions(output, "filter_type", suspendWhenHidden = FALSE)

observeEvent(input$filter_num,{
  withBusyIndicatorServer("filter_num", {
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    # extract the feature vector

    if (input$select_condition_sample_filter_sidebar == "Reads"){
      feature.selected.vec <- colSums(pstat@otu_table@.Data)
    }else{
      feature.selected.vec <- pstat@sam_data@.Data[[
        which(pstat@sam_data@names == input$select_condition_sample_filter_sidebar)]]
    }

    # get the index of sample keeping
    samples.keep.index <- which(feature.selected.vec >= input$num_filter_min &
                                  feature.selected.vec <= input$num_filter_max)

    pstat@otu_table@.Data <- pstat@otu_table@.Data[,samples.keep.index]
    pstat@sam_data@.Data <- lapply(pstat@sam_data@.Data, function(x) {x <- x[samples.keep.index]})
    pstat@sam_data@row.names <- pstat@sam_data@row.names[samples.keep.index]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput

    #updateCovariate()
    updateSample()
  })
})

output$filter_cat_options <- renderUI({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  if (input$select_condition_sample_filter_sidebar == "Reads"){
    tagList(
      selectInput("cat_filter_options", "Keep these levels:", choices = c(""))
    )
  } else {
    variable.filter.index <- which(pstat@sam_data@names ==
    input$select_condition_sample_filter_sidebar)
    #print(variable.filter.index)
    variable.vec <- pstat@sam_data@.Data[[variable.filter.index]]
    filter.option.vec <- sort(unique(variable.vec))
    tagList(
      selectInput("cat_filter_options", "Keep these levels:", choices = filter.option.vec, multiple = TRUE)
    )
  }
})

observeEvent(input$filter_cat,{
  withBusyIndicatorServer("filter_cat", {
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    # extract the feature vector
    feature.selected.vec <- pstat@sam_data@.Data[[
      which(pstat@sam_data@names == input$select_condition_sample_filter_sidebar)]]

    # get the index of sample keeping
    samples.keep.index <- which(feature.selected.vec %in% input$cat_filter_options)

    pstat@otu_table@.Data <- pstat@otu_table@.Data[,samples.keep.index]
    pstat@sam_data@.Data <- lapply(pstat@sam_data@.Data, function(x) {x <- x[samples.keep.index]})
    pstat@sam_data@row.names <- pstat@sam_data@row.names[samples.keep.index]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput

    #updateCovariate()
    updateSample()
  })
})

observeEvent(input$filter_read_micro,{
  withBusyIndicatorServer("filter_read_micro", {
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat


    read.count.mat <- pstat@otu_table@.Data
    read.count.mean <- apply(read.count.mat, 1, mean)
    micro.keep.index <- which(read.count.mean >= input$read_filter_min_micro)

    pstat@otu_table@.Data <- pstat@otu_table@.Data[micro.keep.index,]
    pstat@tax_table@.Data <- pstat@tax_table@.Data[micro.keep.index,]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput

    updateTaxLevel()
  })
})

observeEvent(input$filter_ra_micro,{
  withBusyIndicatorServer("filter_ra_micro", {
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat


    read.count.mat <- pstat@otu_table@.Data
    ra.mat <- getRelativeAbundance(read.count.mat)
    ra.mean <- apply(ra.mat, 1, mean)
    micro.keep.index <- which(ra.mean >= input$ra_filter_min_micro)

    pstat@otu_table@.Data <- pstat@otu_table@.Data[micro.keep.index,]
    pstat@tax_table@.Data <- pstat@tax_table@.Data[micro.keep.index,]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput

    updateTaxLevel()
  })
})

observeEvent(input$filter_prev_micro,{
  withBusyIndicatorServer("filter_prev_micro", {
    shinyInput <- vals$shiny.input
    pstat <- shinyInput$pstat
    read.count.mat <- pstat@otu_table@.Data

    micro.prev <- apply(read.count.mat, 1, function(x) (sum(x >= 1)/ncol(read.count.mat)))
    micro.keep.index <- which(micro.prev >= input$prev_filter_min)

    pstat@otu_table@.Data <- pstat@otu_table@.Data[micro.keep.index,]
    pstat@tax_table@.Data <- pstat@tax_table@.Data[micro.keep.index,]
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput

    updateTaxLevel()
  })
})

# Render summary table
output$contents_summary <- renderTable({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  req(pstat)
  summarizeTable(pstat)
})

# Data input summary
output$contents.count <- DT::renderDataTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    if (!is.null(input$countsfile.pathoscope)){
        if (input$uploadChoice == "pathofiles"){
        req(input$countsfile.pathoscope)
        df <- read.csv(input$countsfile.pathoscope[[1, 'datapath']],
                       skip = 1,
                       header = TRUE,
                       sep = input$sep.ps)
        return(df)
        }
    }
},
options = list(
    paging = TRUE, scrollX = TRUE, pageLength = 5
))

output$contents.meta <- DT::renderDataTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    if (!is.null(input$annotfile.ps)){
        if (input$uploadChoice == "pathofiles"){
        req(input$countsfile.pathoscope)

        df <- read.csv(input$annotfile.ps$datapath,
                       header = input$header.ps,
                       sep = input$sep.ps)
        return(df)
        }
    }
},
options = list(
    paging = TRUE, scrollX = TRUE, pageLength = 5
))

### data input summary
output$contents.count.2 <- DT::renderDataTable({

  # input$file1 will be NULL initially. After the user selects
  # and uploads a file, head of that data file by default,
  # or all rows if selected, will be shown.

  if (!is.null(input$countsfile)){
    if (input$uploadChoice == "count"){
      req(input$countsfile)
      df <- read.csv(input$countsfile$datapath,
                     header = input$header.count,
                     sep = input$sep.count)
      return(df)
    }
  }
},
options = list(
  paging = TRUE, scrollX = TRUE, pageLength = 5
))

output$contents.meta.2 <- DT::renderDataTable({

  # input$file1 will be NULL initially. After the user selects
  # and uploads a file, head of that data file by default,
  # or all rows if selected, will be shown.

  if (!is.null(input$annotfile.count)){
    if (input$uploadChoice == "count"){
      req(input$annotfile.count)
      df <- read.csv(input$annotfile.count$datapath,
                     header = input$header.count,
                     sep = input$sep.count)
      return(df)
    }
  }
},
options = list(
  paging = TRUE, scrollX = TRUE, pageLength = 5
))

output$contents.taxonomy <- DT::renderDataTable({

  # input$file1 will be NULL initially. After the user selects
  # and uploads a file, head of that data file by default,
  # or all rows if selected, will be shown.


  if (!is.null(input$taxon.table)){
    if (input$uploadChoice == "count"){
      req(input$taxon.table)

      df <- read.csv(input$taxon.table$datapath,
                     header = input$header.count,
                     sep = input$sep.count)
      return(df)
    }
  }
},
options = list(
  paging = TRUE, scrollX = TRUE, pageLength = 5
))

# setInputs(FALSE)
findAllTaxData <- function(taxonLevel) {
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  if ((taxonLevel == "no rank"))  {
    taxcountdata <- pstat@otu_table
    taxdata <- findRAfromCount(taxcountdata)
    taxcountdata <- data.frame(taxcountdata)
    taxdata <- data.frame(taxdata)
  } else  {
    physeq2 <- tax_glom(pstat, taxonLevel)
    taxcountdata <- physeq2@otu_table
    taxdata <- findRAfromCount(taxcountdata)
    taxcountdata <- data.frame(taxcountdata)
    taxdata <- data.frame(taxdata)
    labvec <- as(tax_table(physeq2)[, taxonLevel], "character")
    labvec <- unlist(lapply(labvec,
                            function(x){paste0( taxonLevel, "|", x)}))
    labvec[is.na(labvec)] <- ""
    names <- rownames(taxcountdata)
    for (i in 1:length(names))  {
      #tid <- grepTid(names[i])
      #labvec[i] <- paste0( "ti|", tid, "|", labvec[i])
      labvec[i] <- paste0(names[i], labvec[i])
    }
    rownames(taxcountdata) <- labvec
    rownames(taxdata) <- labvec
  }

    vals$taxcountdata <- taxcountdata
    vals$taxdata <- taxdata

}
findTaxData <- reactive({
  findAllTaxData(input$taxl)
  vals$taxdata
})

findTaxCountData <- reactive({
  findAllTaxData(input$taxl)
  vals$taxcountdata
})

findTaxDataTable <- reactive({
    findAllTaxData(input$taxlTable)
    vals$taxdata
})

findTaxCountDataTable <- reactive({
    findAllTaxData(input$taxlTable)
    vals$taxcountdata
})

findTaxCountDataDE <- reactive({
  findAllTaxData(input$taxlde)
  vals$taxcountdata
})

findCountTable <- reactive({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  if (input$taxlTable !="no rank"){
    pstat <- tax_glom(pstat, input$taxlTable)
  }
  df.out <- pstat@otu_table@.Data
  rownames(df.out) <- make.names(TranslateIdToTaxLevel(pstat, rownames(df.out),
                                                       input$taxlTable), unique=TRUE)
  df.out
})

findRATable <- reactive({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  if (input$taxlTable !="no rank"){
    pstat <- tax_glom(pstat, input$taxlTable)
  }
  df.ra <- getRelativeAbundance(pstat@otu_table@.Data)
  rownames(df.ra) <- make.names(TranslateIdToTaxLevel(pstat, rownames(df.ra),
                                                     input$taxlTable), unique=TRUE)
  df.ra
})

output$TaxRAsummary <- renderPrint({
  summary(findTaxData())
})

# These are options for rendering datatables
dtopts <- list(scrollX=TRUE, paging=TRUE)

# Render relative abundance table
output$TaxRAtable <- DT::renderDataTable(findRATable(),
                                         options=dtopts, rownames=T)


# Render count table
output$TaxCountTable <- DT::renderDataTable(findCountTable(),
options=dtopts,
rownames=T)

output$downloadData <- downloadHandler(filename = function() {
  paste0("sample_data_", input$taxlTable, ".csv", sep = "")
}, content = function(file) {
  df.out <- findRATable()
  write.csv(df.out, file)
})

output$downloadCountData <- downloadHandler(filename = function() {
  paste0("sample_data_count_", input$taxlTable, ".csv", sep = "")
}, content = function(file) {
  df.out <- findCountTable()
  write.csv(df.out, file)
})

output$nbins <- renderUI({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  SAM_DATA <- tables$SAM
  vals <-unlist(SAM_DATA[,input$bin_cov,drop=TRUE])
  sliderInput("nbins", label="Number of Bins", min=2, max=length(unique(vals)), value=2, step=1)
})
output$bin_to1 <- renderPrint({
  x <- sort(as.numeric(unlist(strsplit(input$bin_breaks,","))))
  print(x)
})
output$bin_to2 <- renderPrint({
  x <- unlist(strsplit(input$bin_labels,","))
  print(x)
})

output$unbin_plot <- renderPlotly({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat  
  tables <- pstat.extraction(pstat)
  SAM_DATA <- tables$SAM
  unbinned = as.numeric(unlist(SAM_DATA[,input$bin_cov]))
  fit <- density(unbinned)
  p <- plot_ly(x=fit$x, y=fit$y, type="scatter", mode="lines", fill="tozeroy") %>%
         layout(title="Unbinned Density",
                margin = list(l=0,r=0,t=30,b=30))
  p$elementId <- NULL
  return(p)
})

output$bin_plot <- renderPlotly({
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  SAM_DATA <- tables$SAM
  unbinned <- as.numeric(unlist(SAM_DATA[,input$bin_cov]))
  
  # Bins
  nbins <- input$nbins
  n <- input$nbins

  if (!is.null(input$nbins)) {
    # Overrides numnber of bins
    bin_breaks = sort(as.numeric(unlist(strsplit(input$bin_breaks,","))))
    if (length(bin_breaks) > 1) {
      nbins = bin_breaks
      n = length(bin_breaks)-1
    }
    # Overrides labels of bins
    bin_labels = unlist(strsplit(input$bin_labels,","))
    labels <- NULL
    if (length(bin_labels) == n) {
      labels <- bin_labels
    }
    binned <- cut.default(unbinned, nbins, labels=labels)
    p <- plot_ly(y = binned, type = "histogram") %>%
         layout(title = input$bin_cov,
                xaxis = list(title = "Frequency"),
                yaxis = list(title = "Bins"),
                margin = list(l=80)
         )
    p$elementId <- NULL
    return(p)
  }
  return()
})

observeEvent(input$create_bins, {
  shinyInput <- vals$shiny.input
  pstat <- shinyInput$pstat
  tables <- pstat.extraction(pstat)
  SAM_DATA <- tables$SAM
  unbinned <- as.numeric(unlist(SAM_DATA[,input$bin_cov]))
  
  # Bins
  nbins <- input$nbins
  n <- input$nbins

  if (!is.null(input$nbins)) {
    # Overrides numnber of bins
    bin_breaks = sort(as.numeric(unlist(strsplit(input$bin_breaks,","))))
    if (length(bin_breaks) > 1) {
      nbins = bin_breaks
      n = length(bin_breaks)-1
    }
    # Overrides labels of bins
    bin_labels = unlist(strsplit(input$bin_labels,","))
    labels <- NULL
    if (length(bin_labels) == n) {
      labels <- bin_labels
    }
    binned <- cut.default(unbinned, nbins, labels=labels)

    SAM_DATA[,input$new_covariate] <- as.character(binned)
    pstat@sam_data <- SAM_DATA
    shinyInput <- list(pstat = pstat)
    vals$shiny.input <- shinyInput
    updateCovariate()
    updateSample()
  }
})

tabPanel("Biomarker",
    tabsetPanel(
        tabPanel("Classification Model",
                 br(),
                 sidebarLayout(
                     sidebarPanel(
                         br(),
                         selectizeInput('taxl_biomarker', 'Taxonomy Level', choices = tax.name,
                                        selected='genus'),
                         selectInput("select_target_condition_biomarker", "Select Target Condition:",
                                     covariates.colorbar),
                         conditionalPanel(condition = "output.biomarker_condition_type == 'multiple'",
                                          helpText("Please select 2 levels to compare"),
                                          uiOutput("biomarker_condition_options")
                         ),
                         numericInput("num.cv.nfolds", "Number of CV nfolds", value = 3, max = 20, min = 3),
                         numericInput("num.biomarker.run", "Total runs", value = 100, max = 500, min = 50),
                         selectInput("select_covariate_condition_biomarker", "Select Covarites Conditions:",
                                     covariates, multiple = TRUE),
                         selectInput("select_model_biomarker", "Select Model", c("Lasso Logistic Regression")),
                         actionButton("goButtonBiomarker", "Run!"),
                         width=3
                     ),
                     mainPanel(
                         tabsetPanel(
                             tabPanel("Feature selection",
                                      br(),
                            tableOutput("featureSelectionTmp")),
                            tabPanel("LOOCV",
                                     br(),
                                     plotOutput("loocv_output_simple",height=300),
                                     plotlyOutput("loocv.violin", height=300)),
                            tabPanel("LOOCV Bootstrap",
                                     br(),
                                     numericInput("num.bootstrap.loocv", "Number of Bootstrap LOOCV", value = 20, max = 100, min = 5),
                                     actionButton("goButtonBiomarkerBoot", "Run!"),
                                     tableOutput("loocv_output"))
                         ), width=9
                     )
                 )
        )
    )
)

#' Find the taxonomy for the given taxon id name
#'
#' @param pstat pathostat object
#' @param input.id.vec names containing id
#' @param tax.level target taxon level
#' @return target taxon level names
#' @export
#' @examples
#' data_dir_test <- system.file("data", package = "PathoStat")
#' pstat_test <- loadPstat(indir=data_dir_test,
#' infileName="pstat_data_2_L1.rda")
#' names.new <- TranslateIdToTaxLevel(pstat_test,
#' c("ti|862962|org|Bacteroides_fragilis_638R",
#' "ti|697329|org|Ruminococcus_albus_7" ),
#' "genus")
TranslateIdToTaxLevel <- function(pstat, input.id.vec, tax.level){
    tax.df <- pstat@tax_table@.Data
    name.out <- as.character(tax.df[match(input.id.vec, rownames(tax.df)),
                                    which(colnames(tax.df) == tax.level)])
    return(name.out)
}


#' Find the taxonomy for maximum 300 tids
#'
#' @param tids Given taxonomy ids
#' @return taxondata Data with the taxonomy information
#' @import rentrez
#' @export
#' @examples
#' example_data_dir <- system.file("example/data", package = "PathoStat")
#' pathoreport_file_suffix <- "-sam-report.tsv"
#' datlist <- readPathoscopeData(example_data_dir, 
#' pathoreport_file_suffix, input.files.name.vec = as.character(1:6))
#' dat <- datlist$data
#' ids <- rownames(dat)
#' tids <- unlist(lapply(ids, FUN = grepTid))
#' # taxonLevels <- findTaxonomy300(tids[1:5])

findTaxonomy300 <- function(tids) {
    if (is.null(tids)) {
        return(NULL)
    }

    na.vec <- c()
    for (i in seq_len(length(tids))){
        if(is.na(tids[i])){
            na.vec <- c(na.vec, i)
        }
    }

    r_fetch <- entrez_fetch(db = "taxonomy", id = tids, rettype = "xml")
    dat <- XML::xmlToList(r_fetch)
    taxonLevels <- lapply(dat, function(x) x$LineageEx)
    if(!is.null(na.vec)){
        for(i in seq_len(length(na.vec))){
        taxonLevels <- append(taxonLevels, list(NA), na.vec[i]-1)
        }
    }


    return(taxonLevels)
}



#' Find the taxonomy for unlimited tids
#'
#' @param tids Given taxonomy ids
#' @return taxondata Data with the taxonomy information
#' @import rentrez
#' @export
#' @examples
#' example_data_dir <- system.file("example/data", package = "PathoStat")
#' pathoreport_file_suffix <- "-sam-report.tsv"
#' datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
#' input.files.name.vec = as.character(1:6))
#' dat <- datlist$data
#' ids <- rownames(dat)
#' tids <- unlist(lapply(ids, FUN = grepTid))
#' # taxonLevels <- findTaxonomy(tids[1:5])

findTaxonomy <- function(tids) {
    if (is.null(tids)) {
        return(NULL)
    }
    if (length(tids) <= 300){
        taxonLevels <- findTaxonomy300(tids)
    } else{
        taxonLevels <- list()
        batch.num <- ceiling(length(tids)/300)
        for (i in seq_len(batch.num)){
            if (i == batch.num){
                tids.batch <- tids[((i-1)*300 + 1):length(tids)]
            }else{
                tids.batch <- tids[((i-1)*300 + 1):(i*300)]
            }
        taxonLevels <- c(taxonLevels, findTaxonomy300(tids.batch))
        print(i) 
        }
    }


    return(taxonLevels)
}


findSelectedTaxonId <- function(tLineageEx, level) {
    id <- "others"
    if (is.null(tLineageEx)) {
        return(id)
    }
    if (is.na(tLineageEx[1])) {
        return(id)
    }
    for (i in seq_len(length(tLineageEx))) {
        rank <- tLineageEx[[i]]["Rank"]
        scientificName <- tLineageEx[[i]]["ScientificName"]
        taxid <- tLineageEx[[i]]["TaxId"]
        if (!is.null(rank)) {
            if (rank == level) {
                id <- paste0("ti|", taxid, "|", rank, "|", scientificName)
                break
            }
        }
    }
    return(id)
}

findTaxonLevels <- function(data) {
    ids <- rownames(data)
    tids <- unlist(lapply(ids, FUN = grepTid))
    taxonLevels <- findTaxonomy(tids)
    return(taxonLevels)
}

taxon_hash_update <- function(taxon_hash, taxon, length) {
    if (is.null(taxon_hash)) {
        taxon_hash <- new.env()
    }
    index <- length + 1
    newrow <- FALSE
    if (is.null(taxon_hash[[as.character(taxon)]])) {
        taxon_hash[[as.character(taxon)]] <- index
        length <- length + 1
        newrow <- TRUE
    } else {
        index <- taxon_hash[[as.character(taxon)]]
    }
    return(list(taxon_hash = taxon_hash, length = length, index = index,
        newrow = newrow))
}


#' Find the Taxonomy Information Matrix
#'
#' @param names Row names of the taxonomy matrix
#' @param taxonLevels Taxon Levels of all tids
#' @return taxmat Taxonomy Information Matrix
#' @export
#' @examples
#' example_data_dir <- system.file("example/data", package = "PathoStat")
#' pathoreport_file_suffix <- "-sam-report.tsv"
#' datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
#' input.files.name.vec = as.character(1:6))
#' dat <- datlist$data
#' ids <- rownames(dat)
#' tids <- unlist(lapply(ids, FUN = grepTid))
#' # taxonLevels <- findTaxonomy(tids[1:5])
#' # taxmat <- findTaxonMat(ids[1:5], taxonLevels)

findTaxonMat <- function(names, taxonLevels) {
    # tax.name <- c('superkingdom', 'kingdom', 'phylum', 'class', 'order',
    #     'suborder', 'family', 'subfamily', 'genus', 'subgenus', 'species',
    #     'subspecies', 'no rank')
    tax.name <- c('superkingdom', 'kingdom', 'phylum', 'class', 'order',
        'family', 'genus', 'species', 'no rank')
    tl <- c()
    for (i in seq_len(length(tax.name))) {
        tl[tax.name[i]] <- "others"
    }
    taxmat <- NULL
    for (i in seq_len(length(taxonLevels))) {
        taxrow <- tl
        tLineageEx <- taxonLevels[[i]]
        for (j in seq_len(length(tLineageEx))) {
            rank <- tLineageEx[[j]]["Rank"]
            #taxid <- tLineageEx[[j]]["TaxId"]
            scientificName <- tLineageEx[[j]]["ScientificName"]
            if (!is.null(rank) && !is.na(rank) && rank %in% tax.name) {
                #taxrow[as.character(rank)] <- as.character(taxid)
                taxrow[as.character(rank)] <- as.character(scientificName)
            }
        }
        taxmat <- rbind(taxmat, taxrow)
    }

    rownames(taxmat) <- names
    #print(taxmat)
    return(taxmat)
}

#' Use Lasso to do feature selection
#'
#' @param df.input Row is sample, column is feature. Required
#' @param target.vec y vector. Required
#' @param nfolds glmnet CV nfolds
#' @param logisticRegression doing logistic regression or linear regression.
#' @param nRun number of glmnet runs
#' @param alpha same as in glmnet
#' @import glmnet
#' @return signature
#' @export
#' @examples
#' data('iris')
#' getSignatureFromMultipleGlmnet(iris[,1:4],
#' c(rep(1,100), rep(0,50)), nfolds = 3, nRun = 10)

getSignatureFromMultipleGlmnet <- function(df.input,
    target.vec,
    nfolds = 10,
    logisticRegression = TRUE,
    nRun=100,
    alpha = 1){

    # make target numeric
    if (!is.numeric(target.vec)){
        target.vec[target.vec == target.vec[1]] <- 1
        target.vec[target.vec != 1] <- 0
    }

    # factorize catogorical variables
    factor.col.index <- c()
    factor.variable.name <- c()
    for (i in seq_len(ncol(df.input))){
        if (is.character(df.input[,i]) | is.factor(df.input[,i])){
            df.input[,i] <- as.factor(df.input[,i])
            factor.variable.name <- c(factor.variable.name,
    colnames(df.input)[i])
            factor.col.index <- c(factor.col.index, i)
        }
    }

    #return(head(df.input))

    if (is.null(factor.col.index)){
        x <- as.matrix(df.input)
    } else{
        df.input.factor <- df.input[,factor.col.index]
        df.input.factor.onehot <- model.matrix( ~ .-1,
        data.frame(df.input.factor))
        df.input.factor.numeric <- df.input[,-factor.col.index]
        x <- as.matrix(cbind(df.input.factor.numeric,
        df.input.factor.onehot))
    }

    #return(x)


    #body


    featureDict <- list()
    featureNum <- c()
    weights.vec <- rep(0,ncol(x))
    for (i in seq(1,nRun,1)) {
        if (logisticRegression == FALSE){
            fit2 <- suppressWarnings(cv.glmnet(x, target.vec,
    alpha = alpha,
    nfolds = nfolds))
        } else{
            target.vec <- as.factor(target.vec)
            fit2 <- suppressWarnings(cv.glmnet(x, target.vec,
    alpha = alpha,
    family = "binomial",
    type.measure = "auc",
    nfolds = nfolds))
        }
        weights.mat <- as.matrix(coef(fit2, s = "lambda.min"))
        weights.vec <- (weights.vec + weights.mat[-1,1])

        tmp_vec <- as.vector((coef(fit2, s="lambda.min") != 0))
        if (sum(tmp_vec) <= 1){
    next
        }
        featureFromGlmnet <- colnames(x)[tmp_vec[-1]]
        featureNum <- c(featureNum, length(featureFromGlmnet))
        for (k in seq(1,length(featureFromGlmnet),1)){
            gene <- featureFromGlmnet[k]
            if (gene %in% names(featureDict)){
                featureDict[[gene]] <- featureDict[[gene]] + 1
            }
            else{
                if (is.na(gene) == FALSE){
                    featureDict[[gene]] <- 1
                }

            }
        }
    }
    #print(featureDict)

    featureSelectionComplete <- names(featureDict)

    numFloor <- floor(mean(featureNum))

    featureDictInverse <- list()
    for (i in seq(1,length(featureDict),1)){
        numTmp <- featureDict[[i]]
        #print(numTmp)
        numTmpChr <- as.character(numTmp)
        if (numTmp %in% names(featureDictInverse)){
            featureDictInverse[[numTmpChr]] <-
    c(featureDictInverse[[numTmpChr]],
    names(featureDict)[i])
        }
        else {
            featureDictInverse[[numTmpChr]] <- c(names(featureDict)[i])
        }
    }

    numIndex <- sort(as.numeric(names(featureDictInverse)),
    decreasing = TRUE)
    featureSelectionFloor <- c()

    for (i in seq(1,length(numIndex),1)){
        numTmp <- numIndex[i]
        numTmpChr <- as.character(numTmp)
        featureSelectionFloor <- c(featureSelectionFloor,
    featureDictInverse[[numTmpChr]])
        if (length(featureSelectionFloor) > numFloor) {
            break
        }
    }

    return.list <- list()
    return.list[["feature"]] <- featureSelectionFloor
    return.list[["counts"]] <- featureDict
    return.list[["counts.inverse"]] <- featureDictInverse
    return.list[["weights"]] <- weights.vec

    selection.rate <- c()
    for (i in seq_len(length(featureSelectionFloor))){
    selection.rate <- c(selection.rate,
    round(featureDict[[
    paste(featureSelectionFloor[i])]]/nRun, 2))
    }
    percent <- function(x, digits = 2, format = "f", ...) {
    paste0(formatC(100 * x, format = format, digits = digits, ...), "%")
    }
    return.list[["selection_rate"]] <- percent(selection.rate)


    return.list[["feature_weights"]] <- weights.vec[match(featureSelectionFloor,
    colnames(x))]/nRun

    if (sum(selection.rate > 1) >0){
    feature.remove.index <- which(selection.rate > 1)
    return.list <- lapply(return.list, function(x)
        {x <- x[-feature.remove.index]} )
    }
    if (sum(startsWith(featureSelectionFloor,"others")) > 0){
    feature.remove.index <- which(startsWith(featureSelectionFloor,
    "others"))
    return.list <- lapply(return.list, function(x)
        {x <- x[-feature.remove.index]} )

    }

    # Here we get the features:
    return(return.list)

}

#' Do bootstrap and LOOCV
#'
#' @param df Row is sample, column is feature. Required
#' @param targetVec y vector. Required
#' @param nboot number of BOOTSTRAP
#' @importFrom gmodels ci
#' @return bootstrap loocv result dataframe
#' @export
#' @examples
#' data('iris')
#' Bootstrap_LOOCV_LR_AUC(iris[,1:4],
#' c(rep(1,100), rep(0,50)), nboot = 3)
Bootstrap_LOOCV_LR_AUC <- function(df, targetVec, nboot=50){
    output.auc.vec <- c()
    output.other.df <- NULL

    # make target numeric
    if (!is.numeric(targetVec)){
        targetVec[targetVec == targetVec[1]] <- 1
        targetVec[targetVec != 1] <- 0
    }
    targetVec <- as.numeric(targetVec)
    auc.vec <- c()
    for (i in seq_len(nboot)){
        index.boot <- sample(seq_len(nrow(df)), nrow(df), replace = TRUE)
        df.tmp <- df[index.boot,]
        auc.vec <- c(auc.vec, LOOAUC_simple_multiple_noplot_one_df(df.tmp,
        targetVec[index.boot]))
    }

    result.type <- c("AUC Estimate","CI lower","CI upper","Std. Error")
    output.df <- data.frame(result.type, ci(auc.vec))
    colnames(output.df) <- c("Type", "Value")
    return(output.df)

}




#' LOOCV
#'
#' @param df Row is sample, column is feature. Required
#' @param targetVec y vector. Required
#' @importFrom ROCR prediction performance
#' @return mean auc
#' @export
#' @examples
#' data('iris')
#' LOOAUC_simple_multiple_noplot_one_df(iris[,1:4],
#' c(rep(1,100), rep(0,50)))

LOOAUC_simple_multiple_noplot_one_df <- function(df, targetVec){
    auc.vec <- c()
    nSample <- nrow(df)
    vecProbTmp <- c()
    testPredictionClassVec <- c()
    for (j in seq_len(nSample)){
        train = as.matrix(df[-j,])
        test = matrix(as.numeric(df[j,]), nrow = 1)
        fit <- glmnet(train, targetVec[-j], family = "binomial")
        testProb <- predict(fit,type="response", newx = test, s = 0)
        vecProbTmp <- c(vecProbTmp, testProb)
        testPredictionClassVec[j] <- predict(fit,type="class",
        newx = test, s = 0)
    }
    loo.pred = prediction(vecProbTmp, targetVec)
    loo.perf = performance(loo.pred,"tpr","fpr")
    auc <- performance(loo.pred,"auc")
    auc <- unlist(slot(auc, "y.values"))
    aucRound <- round(auc,3)
    auc.vec <- c(auc.vec, aucRound)

    return(mean(auc.vec))
}



#' LOOCV with ROC curve
#'
#' @param df Row is sample, column is feature. Required
#' @param targetVec y vector. Required
#' @importFrom ROCR prediction performance
#' @return the ROC
#' @export
#' @examples
#' data('iris')
#' LOOAUC_simple_multiple_one_df(iris[,1:4],
#' c(rep(1,100), rep(0,50)))

LOOAUC_simple_multiple_one_df <- function(df, targetVec){

    auc.vec <- c()
    nSample <- nrow(df)
    vecProbTmp <- c()
    testPredictionClassVec <- c()

    # make target numeric
    if (!is.numeric(targetVec)){
        targetVec[targetVec == targetVec[1]] <- 1
        targetVec[targetVec != 1] <- 0
    }
    targetVec <- as.numeric(targetVec)

    for (j in seq_len(nSample)){
        train = as.matrix(df[-j,])
        test = matrix(as.numeric(df[j,]), nrow = 1)
        fit <- glmnet(train, targetVec[-j], family = "binomial")
        testProb <- predict(fit,type="response", newx = test, s = 0)
        vecProbTmp <- c(vecProbTmp, testProb)
        testPredictionClassVec[j] <- predict(fit,
        type="class",
        newx = test,
        s = 0)
    }
    loo.pred.plot = prediction(vecProbTmp, targetVec)
    loo.perf.plot = performance(loo.pred.plot,"tpr","fpr")
    return(list(loo.pred.plot = loo.pred.plot,
    loo.perf.plot = loo.perf.plot,
    testPredictionClassVec = testPredictionClassVec,
    testPredictionProb = vecProbTmp))
}

---
title: "Run PathoStat"
author: "Solaiappan Manimaran, Yue Zhao"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{PathoStat intro}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```


## Introduction

PathoStat is a R shiny package, designed for performing 
Statistical Microbiome Analysis on metagenomics results 
from sequencing data samples. In particular, it supports 
analyses on the PathoScope generated report files.

The package includes:

* Data Summary and Filtering 
* Relative Abundance plots (Stacked Bar Plot, Heatmap)
* Multiple species boxplot visualization
* Diversity analysis (Alpha and Beta diversity)
* Differential Expression (DEseq2, edgeR)
* Dimension Reduction (PCA, PCoA)
* Biomarker identification

## Installation

To begin, install [Bioconductor](http://www.bioconductor.org/) and simply
run the following to automatically install PathoStat and all the dependencies 
as follows.

```r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("PathoStat")
```

If you want to install the latest development version of PathoStat from Github, 
use [devtools](https://github.com/hadley/devtools) to install it as follows:
```r
require(devtools)
install_github("mani2012/PathoStat", build_vignettes=TRUE)
```

## Run Pathostat

```{r eval=FALSE}
require(PathoStat)
runPathoStat()
```


# Session info

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{percent}
\alias{percent}
\title{Compute percentage}
\usage{
percent(x, digits = 2, format = "f")
}
\arguments{
\item{x}{a number or a vector}

\item{digits}{how many digit of percentage}

\item{format}{numeric format, "f" for float}
}
\value{
the percentage
}
\description{
Compute percentage
}
\examples{
pecent.vec <- percent(c(0.9, 0.98))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotPCAPlotly}
\alias{plotPCAPlotly}
\title{Plot PCA}
\usage{
plotPCAPlotly(df.input, condition.color.vec,
  condition.color.name = "condition", condition.shape.vec = NULL,
  condition.shape.name = "condition", columnTitle = "Title",
  pc.a = "PC1", pc.b = "PC2")
}
\arguments{
\item{df.input}{Input data object that contains
the data to be plotted. Required}

\item{condition.color.vec}{color vector. Required}

\item{condition.color.name}{color variable name. Required}

\item{condition.shape.vec}{shape vector. Required}

\item{condition.shape.name}{shape variable name. Required}

\item{columnTitle}{Title to be displayed at top of heatmap.}

\item{pc.a}{pc.1}

\item{pc.b}{pc.2}
}
\value{
the plot
}
\description{
Plot PCA
}
\examples{
data('iris')
plotPCAPlotly(t(iris[,1:4]),
condition.color.vec = c(rep(1,100), rep(0,50)),
condition.shape.vec = c(rep(0,100), rep(1,50)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{setShinyInput}
\alias{setShinyInput}
\title{Setter function to set the shinyInput option}
\usage{
setShinyInput(x)
}
\arguments{
\item{x}{shinyInput option}
}
\value{
shinyInput option
}
\description{
Setter function to set the shinyInput option
}
\examples{
setShinyInput(NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxonomy.R
\name{findTaxonomy}
\alias{findTaxonomy}
\title{Find the taxonomy for unlimited tids}
\usage{
findTaxonomy(tids)
}
\arguments{
\item{tids}{Given taxonomy ids}
}
\value{
taxondata Data with the taxonomy information
}
\description{
Find the taxonomy for unlimited tids
}
\examples{
example_data_dir <- system.file("example/data", package = "PathoStat")
pathoreport_file_suffix <- "-sam-report.tsv"
datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
input.files.name.vec = as.character(1:6))
dat <- datlist$data
ids <- rownames(dat)
tids <- unlist(lapply(ids, FUN = grepTid))
# taxonLevels <- findTaxonomy(tids[1:5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{summarizeTable}
\alias{summarizeTable}
\title{Summarize sample}
\usage{
summarizeTable(pstat)
}
\arguments{
\item{pstat}{Input pstat}
}
\value{
A data.frame object of summary metrics.
}
\description{
Creates a table of summary metrics
}
\examples{
data_dir_test <- system.file("data", package = "PathoStat")
pstat_test <- loadPstat(indir=data_dir_test,
infileName="pstat_data_2_L1.rda")
st.tmp <- summarizeTable(pstat_test)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{loadPathoscopeReports}
\alias{loadPathoscopeReports}
\title{Loads all data from a set of PathoID reports.
For each column in the PathoID report, construct a
matrix where the rows are genomes and the columns are samples.
Returns a list where each element is named according to
the PathoID column. For example, ret[["Final.Best.Hit.Read.Numbers"]]
on the result of this function will get you the final count matrix.
Also includes elements "total_reads" and "total_genomes" from the
first line of the PathoID report.}
\usage{
loadPathoscopeReports(reportfiles, nrows = NULL)
}
\arguments{
\item{reportfiles}{Paths to report files}

\item{nrows}{Option to read first N rows of PathoScope reports}
}
\value{
Returns a list where each element is named according to
the PathoID column. For example, ret[["Final.Best.Hit.Read.Numbers"]]
on the result of this function will get you the final count matrix.
Also includes elements "total_reads" and "total_genomes" from the
first line of the PathoID report.
}
\description{
Loads all data from a set of PathoID reports.
For each column in the PathoID report, construct a
matrix where the rows are genomes and the columns are samples.
Returns a list where each element is named according to
the PathoID column. For example, ret[["Final.Best.Hit.Read.Numbers"]]
on the result of this function will get you the final count matrix.
Also includes elements "total_reads" and "total_genomes" from the
first line of the PathoID report.
}
\examples{
input_dir <- system.file("example/data", package = "PathoStat")
reportfiles <- list.files(input_dir, pattern = "*-sam-report.tsv",
full.names = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{savePstat}
\alias{savePstat}
\title{Save the pathostat object to R data(.rda) file}
\usage{
savePstat(pstat, outdir = ".", outfileName = "pstat_data.rda")
}
\arguments{
\item{pstat}{pathostat object}

\item{outdir}{Output Directory of the .rda file}

\item{outfileName}{File name of the .rda file}
}
\value{
outfile .rda file
}
\description{
Save the pathostat object to R data(.rda) file
}
\examples{
data_dir_test <- system.file("data", package = "PathoStat")
pstat_test <- loadPstat(indir=data_dir_test,
infileName="pstat_data_2_L1.rda")
outfile <- savePstat(pstat_test)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{findRAfromCount}
\alias{findRAfromCount}
\title{Return the Relative Abundance (RA) data for the given count OTU table}
\usage{
findRAfromCount(count_otu)
}
\arguments{
\item{count_otu}{Count OTU table}
}
\value{
ra_otu Relative Abundance (RA) OTU table
}
\description{
Return the Relative Abundance (RA) data for the given count OTU table
}
\examples{
data_dir <- system.file("data", package = "PathoStat")
infileName <- "pstat_data.rda"
pstat_test <- loadPstat(data_dir, infileName)
ra_otu <- findRAfromCount(phyloseq::otu_table(pstat_test))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{readPathoscopeData}
\alias{readPathoscopeData}
\title{Reads the data from PathoScope reports and returns a list of
final guess relative abundance and count data}
\usage{
readPathoscopeData(input_dir = ".",
  pathoreport_file_suffix = "-sam-report.tsv", use.input.files = FALSE,
  input.files.path.vec = NULL, input.files.name.vec = NULL)
}
\arguments{
\item{input_dir}{Directory where the tsv files from PathoScope are located}

\item{pathoreport_file_suffix}{PathoScope report files suffix}

\item{use.input.files}{whether input dir to pathoscope files
or directly pathoscope files}

\item{input.files.path.vec}{vector of pathoscope file paths}

\item{input.files.name.vec}{vector of pathoscope file names}
}
\value{
List of final guess relative abundance and count data
}
\description{
Reads the data from PathoScope reports and returns a list of
final guess relative abundance and count data
}
\examples{
example_data_dir <- system.file("example/data", package = "PathoStat")
pathoreport_file_suffix <- "-sam-report.tsv"
datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
input.files.name.vec = as.character(1:6))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxonomy.R
\name{TranslateIdToTaxLevel}
\alias{TranslateIdToTaxLevel}
\title{Find the taxonomy for the given taxon id name}
\usage{
TranslateIdToTaxLevel(pstat, input.id.vec, tax.level)
}
\arguments{
\item{pstat}{pathostat object}

\item{input.id.vec}{names containing id}

\item{tax.level}{target taxon level}
}
\value{
target taxon level names
}
\description{
Find the taxonomy for the given taxon id name
}
\examples{
data_dir_test <- system.file("data", package = "PathoStat")
pstat_test <- loadPstat(indir=data_dir_test,
infileName="pstat_data_2_L1.rda")
names.new <- TranslateIdToTaxLevel(pstat_test,
c("ti|862962|org|Bacteroides_fragilis_638R",
"ti|697329|org|Ruminococcus_albus_7" ),
"genus")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{getShinyInputCombat}
\alias{getShinyInputCombat}
\title{Getter function to get the shinyInputCombat option}
\usage{
getShinyInputCombat()
}
\value{
shinyInputCombat option
}
\description{
Getter function to get the shinyInputCombat option
}
\examples{
getShinyInputCombat()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialAnalysis.R
\name{GET_PAM}
\alias{GET_PAM}
\title{transform cpm counts to presence-absence matrix}
\usage{
GET_PAM(df)
}
\arguments{
\item{df}{Input data object that contains the data to be tested.
Required}
}
\value{
df.output object
}
\description{
transform cpm counts to presence-absence matrix
}
\examples{
GET_PAM(data.frame(a = c(1,3,0), b = c(0,0.1,2)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{runPathoStat}
\alias{runPathoStat}
\title{Statistical Microbiome Analysis on the pathostat input and generates a
html report and produces interactive shiny app plots}
\usage{
runPathoStat(pstat = NULL, report_dir = ".",
  report_option_binary = "111111111", interactive = TRUE)
}
\arguments{
\item{pstat}{phyloseq extension pathostat object}

\item{report_dir}{Output report directory path}

\item{report_option_binary}{9 bits Binary String representing the plots to
display and hide in the report}

\item{interactive}{when TRUE, opens the interactive shinyApp}
}
\value{
outputfile The output file with all the statistical plots
}
\description{
Statistical Microbiome Analysis on the pathostat input and generates a
html report and produces interactive shiny app plots
}
\examples{
runPathoStat(interactive = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plotPCoAPlotly}
\alias{plotPCoAPlotly}
\title{Plot PCoA}
\usage{
plotPCoAPlotly(physeq.input, condition.color.vec,
  condition.color.name = "condition", condition.shape.vec = NULL,
  condition.shape.name = "condition", method = "bray",
  columnTitle = "Title", pc.a = "Axis.1", pc.b = "Axis.2")
}
\arguments{
\item{physeq.input}{Input data object that contains
the data to be plotted. Required}

\item{condition.color.vec}{color vector. Required}

\item{condition.color.name}{color variable name. Required}

\item{condition.shape.vec}{shape vector. Required}

\item{condition.shape.name}{shape variable name. Required}

\item{method}{which distance metric}

\item{columnTitle}{Title to be displayed at top of heatmap.}

\item{pc.a}{pc.1}

\item{pc.b}{pc.2}
}
\value{
the plot
}
\description{
Plot PCoA
}
\examples{
data_dir_test <- system.file("data", package = "PathoStat")
pstat_test <- loadPstat(indir=data_dir_test,
infileName="pstat_data_2_L1.rda")
plotPCoAPlotly(pstat_test, condition.color.vec = rbinom(33,1,0.5),
condition.shape.vec = rbinom(33,1,0.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{getShinyInput}
\alias{getShinyInput}
\title{Getter function to get the shinyInput option}
\usage{
getShinyInput()
}
\value{
shinyInput option
}
\description{
Getter function to get the shinyInput option
}
\examples{
getShinyInput()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{loadPstat}
\alias{loadPstat}
\title{Load the R data(.rda) file with pathostat object}
\usage{
loadPstat(indir = ".", infileName = "pstat_data.rda")
}
\arguments{
\item{indir}{Input Directory of the .rda file}

\item{infileName}{File name of the .rda file}
}
\value{
pstat pathostat object (NULL if it does not exist)
}
\description{
Load the R data(.rda) file with pathostat object
}
\examples{
data_dir <- system.file("data", package = "PathoStat")
infileName <- "pstat_data.rda"
pstat <- loadPstat(data_dir, infileName)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{getShinyInputOrig}
\alias{getShinyInputOrig}
\title{Getter function to get the shinyInputOrig option}
\usage{
getShinyInputOrig()
}
\value{
shinyInputOrig option
}
\description{
Getter function to get the shinyInputOrig option
}
\examples{
getShinyInputOrig()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{log2CPM}
\alias{log2CPM}
\title{Compute log2(counts per mil reads) and library size for each sample}
\usage{
log2CPM(qcounts, lib.size = NULL)
}
\arguments{
\item{qcounts}{quantile normalized counts}

\item{lib.size}{default is colsums(qcounts)}
}
\value{
list containing log2(quantile counts per mil reads) and library sizes
}
\description{
Compute log2(counts per mil reads) and library size for each sample
}
\examples{
log2CPM(matrix(1:12, nrow = 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{formatTaxTable}
\alias{formatTaxTable}
\title{Format taxonomy table for rendering}
\usage{
formatTaxTable(ttable)
}
\arguments{
\item{ttable}{Taxonomy table}
}
\value{
Formatted table suitable for rendering with. DT::renderDataTable
}
\description{
Format taxonomy table for rendering
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxonomy.R
\name{findTaxonMat}
\alias{findTaxonMat}
\title{Find the Taxonomy Information Matrix}
\usage{
findTaxonMat(names, taxonLevels)
}
\arguments{
\item{names}{Row names of the taxonomy matrix}

\item{taxonLevels}{Taxon Levels of all tids}
}
\value{
taxmat Taxonomy Information Matrix
}
\description{
Find the Taxonomy Information Matrix
}
\examples{
example_data_dir <- system.file("example/data", package = "PathoStat")
pathoreport_file_suffix <- "-sam-report.tsv"
datlist <- readPathoscopeData(example_data_dir, pathoreport_file_suffix,
input.files.name.vec = as.character(1:6))
dat <- datlist$data
ids <- rownames(dat)
tids <- unlist(lapply(ids, FUN = grepTid))
# taxonLevels <- findTaxonomy(tids[1:5])
# taxmat <- findTaxonMat(ids[1:5], taxonLevels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{setShinyInputOrig}
\alias{setShinyInputOrig}
\title{Setter function to set the shinyInputOrig option}
\usage{
setShinyInputOrig(x)
}
\arguments{
\item{x}{shinyInputOrig option}
}
\value{
shinyInputOrig option
}
\description{
Setter function to set the shinyInputOrig option
}
\examples{
setShinyInputOrig(NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{grepTid}
\alias{grepTid}
\title{Greps the tid from the given identifier string}
\usage{
grepTid(id)
}
\arguments{
\item{id}{Given identifier string}
}
\value{
tid string
}
\description{
Greps the tid from the given identifier string
}
\examples{
grepTid("ti|700015|org|Coriobacterium_glomerans_PW2")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomarker.R
\name{LOOAUC_simple_multiple_noplot_one_df}
\alias{LOOAUC_simple_multiple_noplot_one_df}
\title{LOOCV}
\usage{
LOOAUC_simple_multiple_noplot_one_df(df, targetVec)
}
\arguments{
\item{df}{Row is sample, column is feature. Required}

\item{targetVec}{y vector. Required}
}
\value{
mean auc
}
\description{
LOOCV
}
\examples{
data('iris')
LOOAUC_simple_multiple_noplot_one_df(iris[,1:4],
c(rep(1,100), rep(0,50)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialAnalysis.R
\name{Wilcox_Test_df}
\alias{Wilcox_Test_df}
\title{Mann-whitney test for a dataframe}
\usage{
Wilcox_Test_df(df, label.vec.num, pvalue.cutoff = 0.05)
}
\arguments{
\item{df}{Input data object that contains the data to be tested. Required}

\item{label.vec.num}{The target binary condition. Required}

\item{pvalue.cutoff}{choose p-value cut-off}
}
\value{
df.output object
}
\description{
Mann-whitney test for a dataframe
}
\examples{
data('iris')
Wilcox_Test_df(t(iris[,1:4]),
c(rep(1,100), rep(0,50)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialAnalysis.R
\name{Chisq_Test_Pam}
\alias{Chisq_Test_Pam}
\title{Given PAM and disease/control annotation,
do Chi-square test for each row of PAM}
\usage{
Chisq_Test_Pam(pam, label.vec.num, pvalue.cutoff = 0.05)
}
\arguments{
\item{pam}{Input data object that contains the data to be tested. Required}

\item{label.vec.num}{The target binary condition. Required}

\item{pvalue.cutoff}{choose p-value cut-off}
}
\value{
df.output object
}
\description{
Given PAM and disease/control annotation,
do Chi-square test for each row of PAM
}
\examples{
tmp <- matrix(rbinom(12,1,0.5), nrow = 3)
rownames(tmp) <- c("a", "b", "c")
Chisq_Test_Pam(tmp, c(1,1,0,0))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomarker.R
\name{getSignatureFromMultipleGlmnet}
\alias{getSignatureFromMultipleGlmnet}
\title{Use Lasso to do feature selection}
\usage{
getSignatureFromMultipleGlmnet(df.input, target.vec, nfolds = 10,
  logisticRegression = TRUE, nRun = 100, alpha = 1)
}
\arguments{
\item{df.input}{Row is sample, column is feature. Required}

\item{target.vec}{y vector. Required}

\item{nfolds}{glmnet CV nfolds}

\item{logisticRegression}{doing logistic regression or linear regression.}

\item{nRun}{number of glmnet runs}

\item{alpha}{same as in glmnet}
}
\value{
signature
}
\description{
Use Lasso to do feature selection
}
\examples{
data('iris')
getSignatureFromMultipleGlmnet(iris[,1:4],
c(rep(1,100), rep(0,50)), nfolds = 3, nRun = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\docType{data}
\name{pstat_data}
\alias{pstat_data}
\alias{pstat}
\title{pathostat object generated from example pathoscope report files}
\format{pathostat object extension of phyloseq-class experiment-level object:
\describe{
    \item{otu_table}{OTU table with  41 taxa and 33 samples}
    \item{sample_data}{Sample Data with 33 samples by 18 sample variables}
    \item{tax_table}{Taxonomy Table with 41 taxa by 9 taxonomic ranks}
    \item{sample_data}{Phylogenetic Tree with 41 tips and 40 internal nodes}
}}
\usage{
pstat
}
\value{
pathostat object
}
\description{
This example data consists of 33 samples from a diet study with 11 subjects
taking 3 different diets in random order
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxonomy.R
\name{findTaxonomy300}
\alias{findTaxonomy300}
\title{Find the taxonomy for maximum 300 tids}
\usage{
findTaxonomy300(tids)
}
\arguments{
\item{tids}{Given taxonomy ids}
}
\value{
taxondata Data with the taxonomy information
}
\description{
Find the taxonomy for maximum 300 tids
}
\examples{
example_data_dir <- system.file("example/data", package = "PathoStat")
pathoreport_file_suffix <- "-sam-report.tsv"
datlist <- readPathoscopeData(example_data_dir, 
pathoreport_file_suffix, input.files.name.vec = as.character(1:6))
dat <- datlist$data
ids <- rownames(dat)
tids <- unlist(lapply(ids, FUN = grepTid))
# taxonLevels <- findTaxonomy300(tids[1:5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialAnalysis.R
\name{phyloseq_to_edgeR}
\alias{phyloseq_to_edgeR}
\title{Convert phyloseq OTU count data into DGEList for edgeR package}
\usage{
phyloseq_to_edgeR(physeq, group, method = "RLE", ...)
}
\arguments{
\item{physeq}{(Required).}

\item{group}{(Required). A character vector or factor giving the experimental
group/condition for each sample/library.}

\item{method}{(Optional).}

\item{...}{Additional arguments passed on to}
}
\value{
dispersion
}
\description{
Further details.
}
\examples{
data_dir_test <- system.file("data", package = "PathoStat")
pstat_test <- loadPstat(indir=data_dir_test,
infileName="pstat_data_2_L1.rda")
phyloseq_to_edgeR(pstat_test, group="Sex")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomarker.R
\name{Bootstrap_LOOCV_LR_AUC}
\alias{Bootstrap_LOOCV_LR_AUC}
\title{Do bootstrap and LOOCV}
\usage{
Bootstrap_LOOCV_LR_AUC(df, targetVec, nboot = 50)
}
\arguments{
\item{df}{Row is sample, column is feature. Required}

\item{targetVec}{y vector. Required}

\item{nboot}{number of BOOTSTRAP}
}
\value{
bootstrap loocv result dataframe
}
\description{
Do bootstrap and LOOCV
}
\examples{
data('iris')
Bootstrap_LOOCV_LR_AUC(iris[,1:4],
c(rep(1,100), rep(0,50)), nboot = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathoStat.R
\name{setShinyInputCombat}
\alias{setShinyInputCombat}
\title{Setter function to set the shinyInputCombat option}
\usage{
setShinyInputCombat(x)
}
\arguments{
\item{x}{shinyInputCombat option}
}
\value{
shinyInputCombat option
}
\description{
Setter function to set the shinyInputCombat option
}
\examples{
setShinyInputCombat(NULL)
}
